package PBXSComponents;

import interfaces.ArrowBorderSlidable;
import interfaces.BorderFindable;
import interfaces.LinePullable;
import interfaces.Stickable;

import java.util.Vector;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import queries.GetBorder;
import test.interfaces.Hiligthtable;
import test.interfaces.Selectable;
import utils.AWK;
import utils.InstanceManager;
import utils.ListCVarPersistant;
import utils.ParameterChecker;
import actions.BorderSlide;
import actions.Stick;
import dopidom.svg.SVGComponent;
import dopidom.svg.SVGScene;

/*
 * Created on 31 mars 2005
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */

/**
 * @author Fabrice H
 * 
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
public class AnchorPoint extends SVGComponent {

	ListCVarPersistant links = new ListCVarPersistant(this.getWrappedElement(),
			"AnchorPoint", "links");

	protected Node linkNode = null;

	public AnchorPoint(Element wrappedElement) {
		super(wrappedElement);
		new Hiligthtable(this);
		new Selectable(this);
		new BorderFindable(this);
		new LinePullable(this);
		new Stickable(this);
		checkInterfaces();
		getParameters();
	}

	private void checkInterfaces() {
		if (!this.canDoAction(Stick.class))
			throw new RuntimeException("An anchorPoint must be stickable");
		if (!this.canDoQuery(GetBorder.class))
			throw new RuntimeException("An anchorPoint must be BorderFindable");
	}

	private void getParameters() {
		String linkType;
		Element node = (Element) getWrappedElement();
		ParameterChecker pc = new ParameterChecker(node, "AnchorPoint");
//		if (pc.checkElementType(BorderFindable.acceptedTypes) == null)
//			return;

		linkType = pc.getStringAttr("linkType");
		if (linkType == null)
			return;
		this.linkNode = SVGScene.getInstance().getCanvas().getSVGDocument()
				.getElementById(AWK.replace(linkType, "#", ""));
		if (this.linkNode == null) {
			System.err.println("Unable to find the specified link ID : \""
					+ linkType + "\"");
			return;
		}
	}

	public void attachTo(AnchorPoint anchor) {
		if (this.linkNode == null) {
			System.err
					.println("cannot attach because the 'linkType' attribute in the AnchorPoint description doesn't point on a valid node ! check the ID name !");
			return;
		}

		InstanceManager im = new InstanceManager();
		Link link = (Link) im.instanciateFromNode(this.linkNode);
		ArrowBorderSlidable borderSlidableStart = (ArrowBorderSlidable) link
				.getStartArrow().getActionInterface(BorderSlide.class);
		ArrowBorderSlidable borderSlidableEnd = (ArrowBorderSlidable) link
				.getEndArrow().getActionInterface(BorderSlide.class);
		if (borderSlidableStart == null || borderSlidableEnd == null)
			throw new RuntimeException("An arrow must be borderSlidable");
		borderSlidableStart.attachTo((BorderFindable) this
				.getQueryInterface(GetBorder.class));
		borderSlidableEnd.attachTo((BorderFindable) anchor
				.getQueryInterface(GetBorder.class));
		links.add(link);
	}

	public Vector getLinks() {
		return links.getVector();
	}

	public void setLinks(ListCVarPersistant links) {
		this.links = links;
	}
}