package PBXSComponents;

import interfaces.ArrowBorderSlidable;
import interfaces.DirectionAdjustable;
import interfaces.LinePullable;
import interfaces.OriginGettable;
import interfaces.Stickable;
import interfaces.Translatable;

import java.awt.geom.Point2D;

import org.apache.batik.util.SVGConstants;
import org.w3c.dom.Element;
import org.w3c.dom.svg.SVGMatrix;
import org.w3c.dom.svg.SVGTransformable;

import test.interfaces.Locatable;
import test.interfaces.Selectable;
import utils.ComponentVarPersistant;
import utils.IntVarPersistant;
import utils.ParameterChecker;
import actions.DirectionAdjust;
import dopidom.svg.SVGComponent;

/*
 * Created on 31 mars 2005
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */

/**
 * @author Fabrice H
 * 
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
public class Arrow extends SVGComponent {
	// FIXME : attention il y avait un FINAL attached component ICI
	final protected IntVarPersistant position = new IntVarPersistant(this
			.getWrappedElement(), "Arrow", "position");

	final protected ComponentVarPersistant attachedLink = new ComponentVarPersistant(
			this.getWrappedElement(), "Arrow", "attachedLink");

	protected Point2D.Double linkAnchor;

	public Arrow(Element wrappedElement) {
		super(wrappedElement);
		new Locatable(this);
		new ArrowBorderSlidable(this);
		new Selectable(this);
		new Translatable(this);
		new DirectionAdjustable(this);
		new OriginGettable(this);
		new Stickable(this);
		getParameters();
	}

	private void getParameters() {
		String ax;
		String ay;
		Element node = (Element) getWrappedElement();
		ParameterChecker pc = new ParameterChecker(node, "Arrow");
		ax = pc.getStringAttr("ax");
		if (ax == null)
			return;
		ay = pc.getStringAttr("ay");
		if (ay == null)
			return;

		this.linkAnchor = new Point2D.Double(Double.parseDouble(ax), Double
				.parseDouble(ay));
	}

	public Link getAttachedLink() {
		return (Link) attachedLink.get();
	}

	public void setAttachedLink(Link attachedLink) {
		this.attachedLink.set(attachedLink);
	}

	public Point2D getLinkAnchorLocation() {
		SVGTransformable transformable = (SVGTransformable) this
				.getWrappedElement();
		if (((Element) transformable)
				.getAttributeNode(SVGConstants.SVG_TRANSFORM_ATTRIBUTE) == null) {
			((Element) transformable).setAttribute(
					SVGConstants.SVG_TRANSFORM_ATTRIBUTE,
					SVGConstants.TRANSFORM_TRANSLATE + "(0,0)");
		}
		SVGMatrix matrix = transformable.getTransform().getBaseVal()
				.consolidate().getMatrix();
		return getLinkAnchorLocation(matrix);
	}

	public Point2D getLinkAnchorLocation(SVGMatrix matrix) {
		//Point2D.Double linkAnchor = new Point2D.Double
		// (this.linkAnchor.getX(), this.linkAnchor.getY());
		//		linkAnchor.setLocation(matrix.getA()*this.linkAnchor.getX()+
		// matrix.getC()*this.linkAnchor.getY() + matrix.getE(),
		// matrix.getB()*this.linkAnchor.getX()+
		// matrix.getD()*this.linkAnchor.getY()+matrix.getF());
		//		return linkAnchor;
		return new Point2D.Double(matrix.getA() * this.linkAnchor.getX()
				+ matrix.getC() * this.linkAnchor.getY() + matrix.getE(),
				matrix.getB() * this.linkAnchor.getX() + matrix.getD()
						* this.linkAnchor.getY() + matrix.getF());
	}

	public void adjustArrow() {
		if (!hasLink()) {
			System.err
					.println("the arrow doesn't have any Link attached.. can't adjust Direction !");
			return;
		}
		Point2D linkAnchor;
		if (isStartArrow()) {
			if (((Link) this.attachedLink.get()).getCurvedLine()
					.getLineDescription().getPath().size() > 1) {
				Point2D hn = ((Link) this.attachedLink.get()).getCurvedLine()
						.getLineDescription().getHandle(1);
				this.getActionInterface(DirectionAdjust.class).doAction(
						new DirectionAdjust((float) hn.getX(), (float) hn
								.getY()));
				;
				linkAnchor = getLinkAnchorLocation();
				((Link) this.attachedLink.get()).getCurvedLine().setPoint(0,
						linkAnchor);
			}
		} else if (isEndArrow()) {
			if (((Link) this.attachedLink.get()).getCurvedLine()
					.getLineDescription().getPath().size() > 1) {
				Point2D hn = ((Link) this.attachedLink.get()).getCurvedLine()
						.getLineDescription().getHandle(
								((Link) this.attachedLink.get())
										.getCurvedLine().getLineDescription()
										.getPath().size() - 1);
				this.getActionInterface(DirectionAdjust.class).doAction(
						new DirectionAdjust((float) hn.getX(), (float) hn
								.getY()));
				linkAnchor = getLinkAnchorLocation();
				((Link) this.attachedLink.get()).getCurvedLine().setPoint(
						((Link) this.attachedLink.get()).getCurvedLine()
								.getLineDescription().getPath().size() - 1,
						linkAnchor);
			}
		}
	}

	public boolean hasLink() {
		return (this.attachedLink.isSet());
	}

	public void setStartArrow() {
		this.position.set(0);
	}

	public void setEndArrow() {
		this.position.set(1);
	}

	public boolean isStartArrow() {
		if (!this.position.isSet())
			throw new RuntimeException(
					"you're trying to know if the arrow is a startArrow or an endArrow, but this information hasn't been defined !");
		return (this.position.get() == 0);
	}

	public boolean isEndArrow() {
		if (!this.position.isSet())
			throw new RuntimeException(
					"you're trying to know if the arrow is a startArrow or an endArrow, but this information hasn't been defined !");
		return (this.position.get() == 1);
	}

	/*public Point2D.Double getLinkAnchor() {
		return linkAnchor;
	}
*//*
	public void setLinkAnchor(Point2D.Double linkAnchor) {
		this.linkAnchor = linkAnchor;
	}*/
}