//package testFab;
package PBXSComponents;

import java.awt.geom.Point2D;

import org.apache.batik.dom.svg.SVGOMPathElement;
import org.apache.batik.util.SVGConstants;
import org.w3c.dom.Element;
import org.w3c.dom.svg.SVGMatrix;
import org.w3c.dom.svg.SVGTransformable;

import test.interfaces.Deletable;
import test.interfaces.Selectable;
import utils.ComponentVarPersistant;
import utils.HandleDescription;
import utils.LineDescription;
import utils.ParameterChecker;
import dopidom.svg.SVGComponent;

/*
 * Created on 31 mars 2005
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */

/**
 * @author Fabrice H
 * 
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
public class CurvedLine extends SVGComponent {
	protected LineDescription lineDescription = null;

	protected ComponentVarPersistant parentLink = new ComponentVarPersistant(
			getWrappedElement(), "CurvedLine", "parentLink");

	public CurvedLine(Element wrappedElement) {
		super(wrappedElement);
		new Selectable(this);
		new Deletable(this);
		getParameters();
	}

	private void getParameters() {
		this.lineDescription = new LineDescription();
		ParameterChecker pc = new ParameterChecker(this.getWrappedElement(),
				"CurvedLine");
		String path;
		String strokeWidth;
		String stroke;
		stroke = pc.getStringAttr("stroke");
		this.lineDescription.setColor(stroke);
		strokeWidth = pc.getStringAttr("stroke-width");
		this.lineDescription.setWidth(Double.parseDouble(strokeWidth));
		path = pc.getStringAttr("d");
		this.lineDescription.setPathString(path);
		// if not an loaded instance, then there is no handle -> must create
		// them for each points
		if (!parentLink.isFilling()
				&& !this.lineDescription.getPath().isEmpty()) {
			for (int i = 0; i < this.lineDescription.getPath().size() - 1; i++)
				createHandle(i);
		}

	}

	protected void synchronizePathAttribute() {
		SVGOMPathElement p = (SVGOMPathElement) this.getWrappedElement();
		p.setAttribute("d", this.lineDescription.getPathString());
	}

	protected void createHandle(int i) { // create an handle between i and i+1
		if (this.lineDescription.getPath().size() - 1 <= i)
			throw new RuntimeException(
					"trying to create an Handle with an index out of range ("
							+ new Integer(i).toString() + ")");
		if (this.lineDescription.getHandle(i + 1) == null) {
			System.err
					.println("you want to create a LineHandle on a index corresponding to part of the line that is not supposed to be curved ! does nothing...");
			return;
		}
		Point2D hn = this.lineDescription.getHandle(i + 1);
		//System.out.println("new handle is at ("+new
		// Double(hn.getX()).toString()+", "+new
		// Double(hn.getY()).toString()+")");
		System.out.println("creating handle...");
		LineHandle lineHandle = (LineHandle) new HandleDescription(hn, 5, 1,
				"darkseagreen", "black", "2,2").createCircle();
		SVGTransformable transformable = (SVGTransformable) lineHandle
				.getWrappedElement();
		if (((Element) transformable)
				.getAttributeNode(SVGConstants.SVG_TRANSFORM_ATTRIBUTE) == null) {
			((Element) transformable).setAttribute(
					SVGConstants.SVG_TRANSFORM_ATTRIBUTE,
					SVGConstants.TRANSFORM_TRANSLATE + "(0,0)");
		}
	
		SVGMatrix matrix = transformable.getTransform().getBaseVal()
				.consolidate().getMatrix();
		matrix.setE((float) hn.getX());
		matrix.setF((float) hn.getY());
		lineHandle.setAttachedLine(this);
		this.lineDescription.setLineHandle(i + 1, lineHandle);
		synchronizePathAttribute();
	}

	public int addPoint(Point2D pt) {
		int index;
		if (this.lineDescription.getPath().size() == 0) {
			index = this.lineDescription.addPoint(pt, null, null);
			synchronizePathAttribute();
		} else {
			Point2D pt0 = this.lineDescription.getPoint(this.lineDescription
					.getPath().size() - 1);
			Point2D hn = new Point2D.Double((pt0.getX() + pt.getX()) / 2, (pt0
					.getY() + pt.getY()) / 2);

			index = this.lineDescription.addPoint(pt, hn, null);
			createHandle(index);
			synchronizePathAttribute();
		}
		return index;
	}

	public void setPoint(int index, Point2D pt) {
		if (lineDescription == null) {
			System.err
					.println("the points description of the link Component hasn't been set");
			return;
		}
		this.lineDescription.setPoint(index, pt);
		synchronizePathAttribute();
	}

	public void setHandle(int index, Point2D hn) {
		if (lineDescription == null) {
			System.err
					.println("the points description of the link Component hasn't been set");
			return;
		}
		this.lineDescription.setHandle(index, hn);
		SVGOMPathElement p = (SVGOMPathElement) this.getWrappedElement();
		p.setAttribute("d", this.lineDescription.getPathString());
		//change handle position
		LineHandle handle = this.lineDescription.getLineHandle(index);

		SVGTransformable transformable = (SVGTransformable) handle
				.getWrappedElement();
		if (((Element) transformable)
				.getAttributeNode(SVGConstants.SVG_TRANSFORM_ATTRIBUTE) == null) {
			((Element) transformable).setAttribute(
					SVGConstants.SVG_TRANSFORM_ATTRIBUTE,
					SVGConstants.TRANSFORM_TRANSLATE + "(0,0)");
		}
		SVGMatrix matrix = transformable.getTransform().getBaseVal()
				.consolidate().getMatrix();
		matrix.setE((float) hn.getX());
		matrix.setF((float) hn.getY());

		// move arrow if necessary
		if (index == 1) {
			Link link = (Link) this.parentLink.get();
			if (link.getStartArrow() == null) {
				System.err.println("There is no start Arrow attached !");
				return;
			} else {
				((Link) this.parentLink.get()).getStartArrow().adjustArrow();
			}
		}
		if (index == this.lineDescription.getPath().size() - 1) {
			if (((Link) this.parentLink.get()).getEndArrow() == null) {
				System.err.println("There is no end Arrow attached !");
				return;
			} else {
				((Link) this.parentLink.get()).getEndArrow().adjustArrow();
			}
		}
	}


	public LineDescription getLineDescription() {
		return lineDescription;
	}

	public void setLineDescription(LineDescription lineDescription) {
		this.lineDescription = lineDescription;
		synchronizePathAttribute();
	}

	public Link getParentLink() {
		return (Link) parentLink.get();
	}

	public void setParentLink(Link parentLink) {
		this.parentLink.set(parentLink);
	}
}