package PBXSComponents;

import ihm04.actions.Translate;
import interfaces.OriginGettable;
import interfaces.Stickable;

import java.awt.geom.Point2D;
import java.util.Random;

import org.w3c.dom.Element;

import queries.GetOrigin;
import test.interfaces.Hiligthtable;
import test.interfaces.Locatable;
import test.interfaces.Selectable;
import test.queries.Location;
import utils.InstanceManager;
import utils.IntVarPersistant;
import utils.ListCVarPersistant;
import utils.ParameterChecker;
import actions.Position;
import actions.Stick;
import dopidom.Component;
import dopidom.svg.SVGComponent;

/*
 * Created on 31 mars 2005
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */

/**
 * @author Fabrice H
 * 
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
public class GraphicContainer extends SVGComponent {
	public static final int RECTANGLE = 0;

	public static final int CIRCLE = 1;


	protected Element graphicComponentModel = null;

	protected ListCVarPersistant components = new ListCVarPersistant(this
			.getWrappedElement(), "GraphicContainer", "components");
	protected IntVarPersistant shape = new IntVarPersistant(this.getWrappedElement(), "GraphicContainer", "shape");

	protected Point2D.Double center = null;
	protected double boundingParameter = -1;

	protected double circleRadius;
	protected Point2D.Double shapeOffset = new Point2D.Double();

	public GraphicContainer(Element wrappedElement) {
		super(wrappedElement);
		new Locatable(this);
		new Selectable(this);
		new Hiligthtable(this);
		new OriginGettable(this);
		//new Translatable(this);
		//new Positionable(this);
		//new GFillable(this);
		new Stickable(this);
		if (!getParameter())
			System.err
					.println("Error when loading GraphicContainer Component !");
	}

	private boolean getParameter() {
		InstanceManager im = new InstanceManager();
		Element node = getWrappedElement();
		String type;
		String[] liste = { "rect", "circle" };
		ParameterChecker pc = new ParameterChecker(node, "GraphicContainer");
		type = pc.checkElementType(liste);
		if (type == null)
			return false;
		String id = pc.getStringAttr("graphic");
		if (id == null)
			return false;

		if (type.equalsIgnoreCase("circle")) {
			this.circleRadius = Double.parseDouble(pc.getStringAttr("r"));
			this.shapeOffset = new Point2D.Double(Double.parseDouble(pc.getStringAttr("cx")), Double.parseDouble(pc.getStringAttr("cy")));
			this.shape.set(CIRCLE);
		} else if (type.equalsIgnoreCase("rect")) {
			this.shapeOffset = new Point2D.Double(Double.parseDouble(pc.getStringAttr("x")) + Double.parseDouble(pc.getStringAttr("width"))/2, Double.parseDouble(pc.getStringAttr("y"))+Double.parseDouble(pc.getStringAttr("height"))/2);
			this.shape.set(RECTANGLE);
		} else {
			throw new RuntimeException("atention ca n'est pas un cercle, l'aquisition du centre pour le rectangle n'as pas encore t implmente !");
		}
		getCenter();

		this.graphicComponentModel = (Element) im.getNodeById(id);
		//if (this.)
		if (this.graphicComponentModel == null)
			return false;
		return true;

	}

	private void getCenter() {
		Point2D.Double pt = getPos(this);
		this.center = new Point2D.Double(pt.getX() + this.shapeOffset.getX(),
		pt.getY()+this.shapeOffset.getY());
		//this.center = new Point2D.Double(pt.getX(), pt.getY());
	}

	private Point2D.Double getPos(Component c) {
		if (!c.canDoQuery(GetOrigin.class)) {
			System.err.println(c.toString() + "must be originGettable !");
			throw new RuntimeException("error in get pos");
		}
		GetOrigin origin = new GetOrigin();
		c.getQueryInterface(GetOrigin.class).doQuery(origin);
		return new Point2D.Double(origin.getX(), origin.getY());
	}

	private void setPos(Component c, double x, double y) {
		if (!c.canDoAction(Position.class)) {
			System.err.println(c.toString() + " must be positionable !");
			throw new RuntimeException("error in setPos");
		}
		Position position = new Position(x, y);
		c.getActionInterface(Position.class).doAction(position);
	}

	private void getBounding() {
		Location loc = new Location();
		((Component)this.components.get(0)).getQueryInterface(Location.class).doQuery(loc);
		
		this.boundingParameter = (double)Math.max(loc.getWidth(),loc.getHeight()); 

	}
	
	protected void drawRectangleGraphics() {
		int size = this.components.size();
		getCenter();
		double begin = this.center.getX()-(size*this.boundingParameter)/2; 
		for (int i = 0; i < size ; i++) {
			setPos((Component)this.components.get(i), begin, this.center.getY());
			begin += this.boundingParameter;
		}
	}

	protected void drawCircleGraphics() {
		if (this.components.size()==0)
			return;
		Translate depl;
		ParameterChecker pc = new ParameterChecker(this.graphicComponentModel,
				"model");
		//float this.boundingParameter = 2 * Float.parseFloat(pc.getStringAttr("r")); // FIXME:
																			  // to
																			  // change
																			  // !
		double orbitRadius = this.boundingParameter * 1.1;
		double alpha;
		getCenter();
		int n = 0;
		if (this.components.size() == 0)
			return;
		setPos((ContainedGraphic) this.components.get(0), this.center
				.getX(), this.center.getY());
		if (this.components.size() <= 1)
			return;
		for (int i = 1; i < this.components.size(); i++) {
			alpha = (n * this.boundingParameter * 1.1) / orbitRadius;
			if (alpha + (this.boundingParameter * 1.1) / orbitRadius > Math.PI * 2) {
				n = 0;
				alpha = 0;
				orbitRadius += (this.boundingParameter * 1.1);
				System.out.println("changing orbit");
			}

			double x = this.center.getX() + orbitRadius * Math.sin(alpha);
			double y = this.center.getY() + orbitRadius * Math.cos(alpha);
			Random rn = new Random();
			setPos((ContainedGraphic) this.components.get(i), x, y);
			n++;
		}

		// translation to center the points
		double sx = 0;
		double sy = 0;
		int size = this.components.size();
		Point2D.Double pt;
		getCenter();
		for (int i = 0; i < size; i++) {
			ContainedGraphic cg = (ContainedGraphic) this.components.get(i);
			pt = getPos(cg);
			sx += pt.getX();
			sy += pt.getY();
		}
		sx = sx / size;
		sy = sy / size;
		Translate trans = new Translate(this.center.getX() - sx, this.center
				.getY()
				- sy);
		for (int i = 0; i < size; i++) {
			ContainedGraphic cg = (ContainedGraphic) this.components.get(i);
			cg.getActionInterface(Translate.class).doAction(trans);
		}

	}
	
	public void setGraphicNumber(int nb) {
		if (nb > this.components.size()) {
			while (nb > this.components.size())
				addGraphic();
		} else if (nb < this.components.size()) {
			while (nb < this.components.size())
				removeGraphic();
		}
		drawGraphics();
	}
	public int getGraphicNumber() {
		return this.components.size();
	}

	public void addGraphic() {
		InstanceManager im = new InstanceManager();
		Component c = im.instanciateFromNode(this.graphicComponentModel);
		if (!(c instanceof ContainedGraphic)) {
			System.err
					.println("The graphic pointed by the 'GraphicContainer' must be a 'ContainedGraphic' type");
			return;
		}
		this.components.add(c);
		((Stickable) this.getActionInterface(Stick.class)).addSticker(c);
		if (this.boundingParameter==-1)
			getBounding();
		drawGraphics();
	}
	
	public void drawGraphics() {
		switch(this.shape.get()) {
		case RECTANGLE :
			drawRectangleGraphics();
			break;
		case CIRCLE :
			drawCircleGraphics();
			break;
		default :
			throw new RuntimeException("bad initialization of the GraphicContainer : the shape is unknown");
				
		}
	}

	public void removeGraphic() {
		if (this.components.size() == 0) {
			System.err
					.println("You try to remove components but there isn't any !");
			return;
		}
		InstanceManager im = new InstanceManager();
		ContainedGraphic c = (ContainedGraphic) this.components
				.get(this.components.size() - 1);
		im.removeFromNode(c.getWrappedElement());
		this.components.remove(c);
		((Stickable) this.getActionInterface(Stick.class)).removeSticker(c);
	}
}