package PBXSComponents;

import ihm04.actions.Translate;
import ihm04.interfaces.ColorPickable;

import java.awt.geom.Point2D;

import javax.swing.SwingUtilities;

import org.apache.batik.util.CSSConstants;
import org.apache.batik.util.SVGConstants;
import org.w3c.dom.Element;
import org.w3c.dom.events.Event;
import org.w3c.dom.svg.SVGMatrix;
import org.w3c.dom.svg.SVGTransformable;

import queries.GetOrigin;

import actions.Position;

import test.actions.SelectAction;
import test.interfaces.Locatable;
import test.interfaces.Selectable;
import test.queries.Location;
import utils.ComponentVarPersistant;
import utils.InstanceManager;
import utils.LineDescription;
import utils.ParameterChecker;
import dopidom.Action;
import dopidom.ActionEvent;
import dopidom.ActionInterface;
import dopidom.Component;
import dopidom.dom.DOMComponent;
import dopidom.svg.SVGComponent;

/*
 * Created on 31 mars 2005
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */

/**
 * @author Fabrice H
 * 
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */

public class Link extends SVGComponent {
	/**
	 *  
	 */
	protected ComponentVarPersistant curvedLine = new ComponentVarPersistant(
			getWrappedElement(), "Link", "curvedLine");

	protected ComponentVarPersistant startArrow = new ComponentVarPersistant(
			getWrappedElement(), "Link", "startArrow");

	protected ComponentVarPersistant endArrow = new ComponentVarPersistant(
			getWrappedElement(), "Link", "endArrow");
/*
	protected ComponentVarPersistant startArrowNode = new ComponentVarPersistant(
			getWrappedElement(), "Link", "startArrowNode");

	protected ComponentVarPersistant endArrowNode = new ComponentVarPersistant(
			getWrappedElement(), "Link", "endArrowNode");
*/
	public Link(Element wrappedElement) {
		super(wrappedElement);
		new ColorPickable(this);
		new Locatable(this);
		getParameters();
	}

	private void getParameters() {
		// en ce qui concerne le path (Link Body)
		/*
		 * Politique de remplissage du lineDescription priorits: (fab:1)
		 * affectation de la variable curvedLink (creation nouveau lien ou
		 * composant deja existant dans scene)
		 * 
		 * if (il y a un pointeur d'instance dfinis):
		 * pathDefinis(id=instanceID) else if (il y a un pointeur de model
		 * dfinis, attribut SVG): pathDefinis createLink(id=modelID) else :
		 * !pathDefinis createLink()
		 * 
		 * 
		 * createLink([id]) link = node (groupe SVG) correspondant au link if
		 * (pathDefinis): path = node correspondant au 'id' if
		 * (path.hasAttribute(stroke)): lineDescription.stroke = path.stroke
		 * else if (link.hasAttribute(stroke)): lineDescription.stroke =
		 * link.stroke else: error else : crer un lien vide avec les attributs
		 * du 'link' idem avec les autres attributs
		 *  
		 */
		LineDescription lineDescription = new LineDescription();
		String parameters;
		String strokeColor;
		String strokeWidth;
		String arrow;
		InstanceManager im = new InstanceManager();
		Element node = (Element) getWrappedElement();
		ParameterChecker pc = new ParameterChecker(node, "Link");
		String path = null;
		if (node.getAttributeNode("path") != null)
			path = node.getAttribute("path");
		if (!pc.checkElementType("g"))
			return;
		if (!this.curvedLine.isFilling()) {
			//l'afectation de la valeur de curvedLink a t prvue en
			// invokeLater
			if (path != null) { // il n'y a pas de pointeur d'instance et on a
								// dfinis un lien
				//this.curvedLine.set(im.instanciateFromId(path));
				Component cl = DOMComponent.getFactory().getOwnerComponent(
						(Element) im.getNodeById(path));
				if (!(cl instanceof CurvedLine))
					throw new RuntimeException(
							"the id in path doesn't point a Path Type :" + path);
				this.curvedLine.set((CurvedLine) cl);
				if(!((CurvedLine) this.curvedLine.get()).parentLink.isFilling())
						((CurvedLine) this.curvedLine.get()).setParentLink(this);
			} else { // il n'y a pas de pointeur d'instance ni de path dans la
					 // scene, il faut en crer un
				//STROKE COLOR
				strokeColor = pc.getStringAttr("stroke");
				if (strokeColor == null)
					return;

				lineDescription.setColor(strokeColor);

				//STROKE WIDTH
				strokeWidth = pc.getStringAttr("stroke-width");
				if (strokeWidth == null)
					return;
				lineDescription.setWidth(Double.parseDouble(strokeWidth));

				this.curvedLine.set((CurvedLine) lineDescription.createLine()); // le
																				// path
																				// est
																				// vide
																				// au
																				// debut

				if(!((CurvedLine) this.curvedLine.get()).parentLink.isFilling())
						((CurvedLine) this.curvedLine.get()).setParentLink(this);

				// mainteannt il faut ajouter des points sur curvedLine
			}
			Component a;
			// ARROW START
			if (!this.startArrow.isFilling()) {
				arrow = pc.getStringAttr("arrowStart");
				if (arrow == null)
					return;

				a = DOMComponent.getFactory().getOwnerComponent(
						(Element) im.getNodeById(arrow));
				if (!(a instanceof Arrow))
					throw new RuntimeException(
							"the id in arrowStart doesn't point an Arrow Type :"
									+ arrow);
				this.startArrow.set(a);
				((Arrow) this.startArrow.get()).setAttachedLink(this);
				((Arrow) this.startArrow.get()).setStartArrow();
			}
			// ARROW END
			if (!this.endArrow.isFilling()) {
				arrow = pc.getStringAttr("arrowEnd");
				if (arrow == null)
					return;

				a = DOMComponent.getFactory().getOwnerComponent(
						(Element) im.getNodeById(arrow));
				if (!(a instanceof Arrow))
					throw new RuntimeException(
							"the id in arrowEnd doesn't point an Arrow Type :"
									+ arrow);
				this.endArrow.set(a);
				((Arrow) this.endArrow.get()).setAttachedLink(this);
				((Arrow) this.endArrow.get()).setEndArrow();
			}
			if (path != null) {
				LineDescription ld;
				Point2D pt;
				if (!this.endArrow.isFilling()) {
					ld = ((CurvedLine) this.curvedLine.get())
							.getLineDescription();
					pt = ld.getPoint(ld.getPath().size() - 1);
					this.setEndLocation(pt);
				}
				if (!this.startArrow.isFilling()) {
					pt = ((CurvedLine) this.curvedLine.get())
							.getLineDescription().getPoint(0);
					this.setStartLocation(pt);
				}
			}
		}

	}

	public void setStartLocation(Point2D point) {
		//Old fashion, but was missing translation listeners invokation !
//		// move the arrow
//		SVGTransformable transformable = (SVGTransformable) ((Arrow) this.startArrow
//				.get()).getWrappedElement();
//		if (((Element) transformable)
//				.getAttributeNode(SVGConstants.SVG_TRANSFORM_ATTRIBUTE) == null) {
//			((Element) transformable).setAttribute(
//					SVGConstants.SVG_TRANSFORM_ATTRIBUTE,
//					SVGConstants.TRANSFORM_TRANSLATE + "(0,0)");
//		}
//		SVGMatrix matrix = transformable.getTransform().getBaseVal()
//				.consolidate().getMatrix();
//		matrix.setE((float) point.getX());
//		matrix.setF((float) point.getY());

		Arrow a = (Arrow) this.startArrow.get();
				
		GetOrigin l = new GetOrigin();
		a.getQueryInterface(GetOrigin.class).doQuery(l);
		Translate t = new Translate(point.getX()-l.getX(), point.getY()-l.getY());
		a.getActionInterface(Translate.class).doAction(t);

		if (((CurvedLine) this.curvedLine.get()).getLineDescription().getPath()
				.isEmpty())
			((CurvedLine) this.curvedLine.get()).addPoint(point);

		a.adjustArrow(); // end and start of path
													   // definition redefined
													   // here ! redundancy...
													   // find good architecture

	}

	public void setEndLocation(Point2D point) {
		Arrow a = (Arrow) this.endArrow.get();
				
		GetOrigin l = new GetOrigin();
		a.getQueryInterface(GetOrigin.class).doQuery(l);
		Translate t = new Translate(point.getX()-l.getX(), point.getY()-l.getY());
		a.getActionInterface(Translate.class).doAction(t);
		

		int size = ((CurvedLine) this.curvedLine.get()).getLineDescription()
				.getPath().size();
		if (size < 2) {
			if (size == 0) {
				System.err
						.println("Link : you have to set the StartLocationFirst");
				return;
			}
			((CurvedLine) this.curvedLine.get()).addPoint(point);
		}

		a.adjustArrow();// end and start of path
													// definition redefined here
													// ! redundancy... find good
													// architecture
	}

	public CurvedLine getCurvedLine() {
		return (CurvedLine) this.curvedLine.get();
	}

	public void setCurvedLine(CurvedLine curvedLine) {		
		this.curvedLine.set(curvedLine);
	}

	public Arrow getEndArrow() {
		return (Arrow) endArrow.get();
	}

	public void setEndArrow(Arrow endArrow) {
		this.endArrow.set(endArrow);
	}

	public Arrow getStartArrow() {
		return (Arrow) startArrow.get();
	}

	public void setStartArrow(Arrow startArrow) {
		this.startArrow.set(startArrow);
	}
}