/*
 * Created on 12 fvr. 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package dopidom.dom;

import java.util.Hashtable;
import java.util.Vector;

import javax.xml.transform.TransformerException;

import org.apache.xpath.XPathAPI;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import dopidom.Action;
import dopidom.ActionConsumer;
import dopidom.ActionInterface;
import dopidom.Component;
import dopidom.ComponentFactory;
import dopidom.Mark;
import dopidom.ObserverInterface;
import dopidom.Query;
import dopidom.QueryConsumer;
import dopidom.QueryInterface;

public abstract class DOMComponent implements Component
{
	protected static final DOMComponentFactory FACTORY = new DOMComponentFactory();

	public static ComponentFactory getFactory()
	{
		return FACTORY;
	}

	protected Element wrappedElement;

	protected DOMComponent(Element wrappedElement)
	{
		this.wrappedElement = wrappedElement;
		new DOMObservableListener(this);
	}

	protected boolean tagDeleted = false;
	public void tagDeleted() {
		this.tagDeleted = true;
	}
	
	public boolean isDeleted() {
		return this.tagDeleted;
	}

	//===================
	// Markable interface
	//===================

	/**
	 * key=the marked node, value=a vector of assoiated marks
	 */
	protected static Hashtable markMap = new Hashtable(); // NOTE: DOM3 : dans getUserObject()...

	/* (non-Javadoc)
	 * @see dopidom.Markable#addMark(java.lang.String, boolean, java.lang.Object)
	 */
	public Mark addMark(String xpath, boolean write, Object data)
	{
		Mark mark = new DOMMark(getNodes(xpath), write, data);
		Node[] nodes = getNodes(xpath);
		for (int n = 0; n < nodes.length; n++)
		{
			Node node = nodes[n];
			if (!isMarked(node) || (!write && isMarkedRead(node)))
			{
				Vector marks = getMarks(node);
				if (marks.isEmpty())
					markMap.put(node, marks);
				marks.add(mark);
			}
		}
		return mark;
	}

	/* (non-Javadoc)
	 * @see dopidom.Markable#addMark(java.lang.String, boolean)
	 */
	public Mark addMark(String xpath, boolean write)
	{
		return addMark(xpath, write, null);
	}

	/* (non-Javadoc)
	 * @see dopidom.Markable#removeMark(java.lang.String, boolean)
	 */
	public Mark removeMark(String xpath, boolean write)
	{
		Node[] nodes = getNodes(xpath);
		Mark removedMark = null;
		for (int n = 0; n < nodes.length; n++)
		{
			Node node = nodes[n];
			if ((write && isMarkedWrite(node)) || (!write && isMarkedRead(node)))
			{
				Vector marks = getMarks(node);
				// we only need to remove one mark... 
				removedMark = (Mark) marks.remove(0);
				// and to remove the marking if there is no more marks
				if (marks.isEmpty())
					markMap.remove(node);
			}
		}
		return removedMark;
	}

	/* (non-Javadoc)
	 * @see dopidom.Markable#getMarks(java.lang.String)
	 */
	public Mark[] getMarks(String xpath)
	{
		Vector marks = getMarks(getNode(xpath));
		return (Mark[]) marks.toArray(new Mark[marks.size()]);
	}

	protected Vector getMarks(Node node)
	{
		if (markMap.containsKey(node))
			return (Vector) markMap.get(node);
		else
			return new Vector();
	}

	/* (non-Javadoc)
	 * @see dopidom.Markable#isMarked(java.lang.String)
	 */
	public boolean isMarked(String xpath)
	{
		return isMarked(getNode(xpath));
	}

	protected boolean isMarked(Node node)
	{
		return getMarks(node).size() > 0;
	}

	/* (non-Javadoc)
	 * @see dopidom.Markable#isMarkedRead(java.lang.String)
	 */
	public boolean isMarkedRead(String xpath)
	{
		return isMarkedRead(getNode(xpath));
	}

	protected boolean isMarkedRead(Node node)
	{
		Vector marks = getMarks(node);
		return (marks.size() > 0) && (((Mark) marks.get(0)).isRead());
	}

	/* (non-Javadoc)
	 * @see dopidom.Markable#isMarkedWrite(java.lang.String)
	 */
	public boolean isMarkedWrite(String xpath)
	{
		return isMarkedWrite(getNode(xpath));
	}

	protected boolean isMarkedWrite(Node node)
	{
		Vector marks = getMarks(node);
		return (marks.size() > 0) && (((Mark) marks.get(0)).isWrite());
	}

	//=====================
	// Component interface
	//=====================

	public Element getWrappedElement()
	{
		return wrappedElement;
	}

	/**
	 * @see dpidom.Component#getNode(java.lang.String)
	 */
	public Node getNode(String xpath)
	{
		if (this.isDeleted())
			return null;
		try
		{
			return XPathAPI.selectSingleNode(getWrappedElement(), xpath);
		}
		catch (TransformerException e)
		{
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * @see dpidom.Component#getNodes(java.lang.String)
	 */
	public Node[] getNodes(String xpath)
	{
		if (this.isDeleted())
			return null;
		try
		{
			NodeList nodeSet = XPathAPI.selectNodeList(getWrappedElement(), xpath);
			Node[] nodes = new Node[nodeSet.getLength()];
			for (int n = 0; n < nodes.length; n++)
				nodes[n] = nodeSet.item(n);
			return nodes;
		}
		catch (TransformerException e)
		{
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * @see dpidom.Component#clone(boolean)
	 */
	public Component clone(boolean deep)
	{
		Element cloneElement = (Element) getWrappedElement().cloneNode(true);
		// FIXME clone aussi les dpi::Component ?
		Component cloneComponent = getFactory().newComponent(cloneElement);
		return cloneComponent;
	}

	/**
	 * @see dopidom.Component#getAncestors()
	 */
	public Component[] getAncestors()
	{
		Vector result = new Vector();
		for (Component c = getParent(); c != null; c = c.getParent())
			result.add(c);
		return (Component[]) result.toArray(new Component[result.size()]);
	}

	/**
	 * @see dopidom.Component#getChildren()
	 */
	public Component[] getChildren()
	{
		Vector children = new Vector();
		checkForChildren(children, getWrappedElement());
		return (Component[]) children.toArray(new Component[children.size()]);
	}

	protected void checkForChildren(Vector children, Element element)
	{
		NodeList childElements = element.getChildNodes();
		for (int c = 0; c < childElements.getLength(); c++)
			if (childElements.item(c).getNodeType() == Node.ELEMENT_NODE)
			{
				Element childElement = (Element) childElements.item(c);
				Component childComponent = getFactory().getWrapperComponent(childElement);
				// if a Component is found, save it
				if (childComponent != null)
					children.add(childComponent);
				// else check for a child in the subtree
				else
					checkForChildren(children, childElement);
			}
	}

	/**
	 * @see dopidom.Component#getParent()
	 */
	public Component getParent()
	{
		try {
			return getFactory().getOwnerComponent((Element) getWrappedElement().getParentNode());
		} catch (NullPointerException _) {
			return null;
		} catch (ClassCastException _) {
			return null;
		}
	}

	//=========================
	// ActionConsumer interface
	//=========================

	public void registerActionInterface(ActionInterface actionInterface)
	{
	}

	public ActionInterface[] getActionInterfaces()
	{
		return null;
	}

	public ActionInterface getActionInterface(Class actionClass)
	{
		return null;
	}

	public boolean canDoAction(Class actionClass)
	{
		return false;
	}

	//========================
	// QueryConsumer interface
	//========================

	public void registerQueryInterface(QueryInterface queryInterface)
	{
	}

	public QueryInterface[] getQueryInterfaces()
	{
		return null;
	}

	public QueryInterface getQueryInterface(Class queryClass)
	{
		return null;
	}

	public boolean canDoQuery(Class queryClass)
	{
		return false;
	}

	//=========================
	// ActionProducer interface
	//=========================

	public void doActionOnceOn(Action action, ActionConsumer consumer)
	{
	}

	public void beginInteractionOn(Class actionClass, ActionConsumer consumer)
	{
	}

	public void endInteractionOn(Class actionClass, ActionConsumer consumer)
	{
	}

	public void doActionOn(Action action, ActionConsumer consumer)
	{
	}

	/**
	 * @see dopidom.Producer#echoActionsOn(dopidom.Action[], dopidom.Consumer)
	 */
	public void echoActionsOn(Action[] actions, ActionConsumer consumer)
	{
	}

	public void redoActionsOn(Action[] actions, ActionConsumer consumer)
	{
	}

	public void undoActionsOn(Action[] actions, ActionConsumer consumer)
	{
	}

	//=========================
	// QueryProducer interface
	//=========================

	public void doQueryOn(Query query, QueryConsumer consumer)
	{
	}

	//=====================
	// Observable interface
	//=====================

	protected Vector subscribedObserverInterfaces = new Vector();

	/**
	 * @see dpidom.Observable#subscribeObserver(dpidom.ObserverInterface)
	 */
	public void subscribeObserverInterface(ObserverInterface observerInterface)
	{
		subscribedObserverInterfaces.add(observerInterface);
	}

	/**
	 * @see dpidom.Observable#unsubscribeObserver(dpidom.ObserverInterface)
	 */
	public void unsubscribeObserverInterface(ObserverInterface observerInterface)
	{
		subscribedObserverInterfaces.remove(observerInterface);
	}

	/**
	 * @see dpidom.Observable#getSubscribedObservers()
	 */
	public ObserverInterface[] getSubscribedObserverInterfaces()
	{
		return (ObserverInterface[]) subscribedObserverInterfaces.toArray(
			new ObserverInterface[subscribedObserverInterfaces.size()]);
	}

	//====================
	// Observer interface
	//====================

	private Vector registeredObserverInterfaces = new Vector();

	public void registerObserverInterface(ObserverInterface observerInterface)
	{
		registeredObserverInterfaces.add(observerInterface);
		observerInterface.getObservable().subscribeObserverInterface(observerInterface);
	}

	/**																								
	 * @see dpidom.Observer#getObserverInterfaces()
	 */
	public ObserverInterface[] getObserverInterfaces()
	{
		return (ObserverInterface[]) registeredObserverInterfaces.toArray(
			new ObserverInterface[registeredObserverInterfaces.size()]);
	}

}
