/*
 * Created on 6 mai 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package dopidom.dom;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Hashtable;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.batik.swing.svg.GVTTreeBuilderAdapter;
import org.apache.batik.swing.svg.GVTTreeBuilderEvent;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.svg.SVGDefsElement;
import org.w3c.dom.svg.SVGElement;
import org.w3c.dom.svg.SVGUseElement;

import utils.InstanceManager;
import dopidom.ActionInterface;
import dopidom.Component;
import dopidom.ComponentFactory;
import dopidom.QueryInterface;
import dopidom.svg.SVGComponent;
import dopidom.svg.SVGScene;

/**
 * @author Olivier Beaudoux
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class DOMComponentFactory implements ComponentFactory
{
	/**
	 * key: a DOM element,  value: the associated wrapper DPI Component
	 * NOTE pb garbage collection... (utiliser Element.userObject quand DOM3...)
	 */
	protected Hashtable componentMap = new Hashtable();

	public DOMComponentFactory() {
		super();
		SVGScene.getInstance().getCanvas().addGVTTreeBuilderListener(new GVTTreeBuilderAdapter()
				{
					public void gvtBuildStarted(GVTTreeBuilderEvent event)
					{
						componentMap.clear();
					}
				});
	}

	/**
	 * @see dopidom.ComponentFactory#newComponent(org.w3c.dom.Element)
	 */
	public Component newComponent(Element element)
	{
		// reject already loaded elements
		if (componentMap.containsKey(element))
			return (Component) componentMap.get(element);
		// NOTE la presence des cas particuliers SVGDefsElement, SVGUseElement incite plutot a faire un SVGComponentFactory...
		// ceci dit, le mcanisme de use faon SVG est interessant au niveau gnral, ie DOM...! ( a voir...)
		// reject SVG definitions
		if (element instanceof SVGDefsElement)
			return null;
		// check for the "dpi:component" attribute
		Attr attr = element.getAttributeNodeNS(Component.NAMESPACE_URI, Component.COMPONENT_ATTR);
		if ((attr == null) && (element instanceof SVGUseElement))
		{
			// in case of a use element, we must also check in its instance root 
			// IMPORTANT NOTE: the wrapped element of the instanciated Component is the
			// use element, not the associated instance root...
			SVGUseElement useElement = (SVGUseElement) element;
			SVGElement useInstanceRoot = SVGScene.getInstance().getUseInstanceRoot(useElement);
			attr = useInstanceRoot.getAttributeNodeNS(Component.NAMESPACE_URI, Component.COMPONENT_ATTR);
		}
		Component component = null;
		// when found, build the Component
		if (attr != null)
		{
			// scan the dpi:Component attribute
			StringTokenizer itfTokenizer = new StringTokenizer(attr.getValue().trim(), ",");
			if (itfTokenizer.countTokens() == 0)
				return null;
			
			RuntimeException classError = new IllegalArgumentException("A dopidom component an optional component followed by a set of action or query interfaces.");
			for (; itfTokenizer.hasMoreTokens();) {
			
				StringTokenizer tokenizer = new StringTokenizer(itfTokenizer.nextToken().trim(), " ");
				Element[] arguments = new Element[tokenizer.countTokens() - 1];
				// 	retrieve the Component factory class name
				String className = tokenizer.nextToken();
				// create the wrapper Component
				try
				{
					Class klass = Class.forName(className);
					boolean isInterface, isActionInterface;
					Object itfOrComp;
					Class mainConstructorArgument;
					if (Arrays.asList(klass.getInterfaces()).contains(ActionInterface.class)) {
						isInterface = true;
						isActionInterface = true;
					} else if (Arrays.asList(klass.getInterfaces()).contains(QueryInterface.class)) {
						isInterface = true;
						isActionInterface = false;
					} else {
						if (component != null)
							throw classError;
						try {
							klass.asSubclass(Component.class);
							isInterface = false;
							isActionInterface = false;
						} catch (ClassCastException x) {
							throw classError;
						}
					}
					
					if (isInterface) {
						mainConstructorArgument = Component.class;
						if (component == null)
							component = new SVGComponent(element) {};
					} else {
						mainConstructorArgument = Element.class;
					}
					
					// constructor with no argument (except the wrapped element)
					if (arguments.length == 0)
					{
						Constructor constructor = klass.getConstructor(new Class[] { mainConstructorArgument });
						if (isInterface)
							constructor.newInstance(new Object[] { component });
						else
							component = (Component) constructor.newInstance(new Object[] { element });
					}
					else
					{
						// constructor with element arguments
						for (int e = 0; e < arguments.length; e++)
							arguments[e] = element.getOwnerDocument().getElementById(tokenizer.nextToken());
						Constructor constructor = klass.getConstructor(new Class[] { mainConstructorArgument, Element[].class });
						if (isInterface)
							constructor.newInstance(new Object[] { component, arguments });
						else
							component = (Component)constructor.newInstance(new Object[] { element, arguments });
					}
					
					componentMap.put(element, component);
				}
				catch (ClassNotFoundException e)
				{
					System.err.println("Component not created since its specified class " + className + " was not found");
				}
				catch (SecurityException e)
				{
					e.printStackTrace();
				}
				catch (NoSuchMethodException e)
				{
					System.err.println(
						"Component not created since its specified constructor does not exist within class " + className);
				}
				catch (IllegalArgumentException e)
				{
					e.printStackTrace();
				}
				catch (InstantiationException e)
				{
					e.printStackTrace();
				}
				catch (IllegalAccessException e)
				{
					e.printStackTrace();
				}
				catch (InvocationTargetException e)
				{
					e.printStackTrace();
				}
			}
		}
		// recurse in subtree
		for (Node node = element.getFirstChild(); node != null; node = node.getNextSibling())
			if (node.getNodeType() == Node.ELEMENT_NODE)
				newComponent((Element) node);
		// return the (root) Component
		if(element.getAttributeNode("id") != null)
			InstanceManager.addComponentId(element.getAttribute("id"), component);
		return component;
	}

	/**
	 * @see dopidom.ComponentFactory#newComponents(org.w3c.dom.Element)
	 */
	public Component[] newComponents(Element rootElement)
	{
		Vector components = new Vector();
		newComponents(components, rootElement);
		return (Component[]) components.toArray(new Component[components.size()]);
	}

	protected void newComponents(Vector components, Element rootElement)
	{
		Component component = newComponent(rootElement);
		if (component != null)
			components.add(component);
	}

	/**
	 * @see dopidom.ComponentFactory#getWrappedElement(dopidom.Component)
	 */
	public Element getWrappedElement(Component wrapperComponent)
	{
		return wrapperComponent.getWrappedElement();
	}

	/**
	 * @see dopidom.ComponentFactory#isElementWrapped(org.w3c.dom.Element)
	 */
	public boolean hasWrapperComponent(Element element)
	{
		return getWrapperComponent(element) != null;
	}

	/**
	 * @see dopidom.ComponentFactory#getWrapperComponent(org.w3c.dom.Element)
	 */
	public Component getWrapperComponent(Element wrappedElement)
	{
		newComponents(wrappedElement);
		return (Component) componentMap.get(wrappedElement);
	}

	/**
	 * @see dopidom.ComponentFactory#hasOwnerComponent(org.w3c.dom.Element)
	 */
	public boolean hasOwnerComponent(Element element)
	{
		return getOwnerComponent(element) != null;
	}

	/**
	 * @see dopidom.ComponentFactory#getOwnerComponent(org.w3c.dom.Element)
	 */
	public Component getOwnerComponent(Element element)
	{
		for (Node node = element; node != null; node = node.getParentNode())
			if (node.getNodeType() == Node.ELEMENT_NODE)
			{
				Component component = getWrapperComponent((Element) node);
				if (component != null)
					return component;
			}
		return null;
	}
	
	public Component[] getAllComponents() {
		Collection ret = this.componentMap.values(); 
		return (Component[])ret.toArray(new Component[ret.size()]);
	}
}
