/*
 * Created on 16 mars 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package dopidom.dom;

import java.util.Hashtable;

import javax.xml.transform.TransformerException;

import org.apache.xpath.CachedXPathAPI;
import org.w3c.dom.Attr;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.events.Event;
import org.w3c.dom.events.EventListener;
import org.w3c.dom.events.EventTarget;
import org.w3c.dom.events.MutationEvent;

import dopidom.Observable;
import dopidom.ObserverInterface;

/**
 * @author Olivier Beaudoux
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class DOMObservableListener implements EventListener
{
	protected Observable observable;

	public DOMObservableListener(Observable observable)
	{
		this.observable = observable;
		EventTarget eventTarget = (EventTarget) observable.getWrappedElement();
		eventTarget.addEventListener("DOMNodeInsertedIntoDocument", this, false);
		eventTarget.addEventListener("DOMNodeRemovedFromDocument", this, false);
		eventTarget.addEventListener("DOMAttrModified", this, false);
		eventTarget.addEventListener("DOMCharacterDataModified", this, false);
	}

	private static CachedXPathAPI xpathAPI = new CachedXPathAPI();

	protected boolean nodeMatchesXPaths(Node node, String[] xpaths)
	{
		try
		{
			for (int i = 0; i < xpaths.length; i++)
			{
				NodeList nodeSet = xpathAPI.selectNodeList(observable.getWrappedElement(), xpaths[i]);
				for (int n = 0; n < nodeSet.getLength(); n++)
					if (nodeSet.item(n) == node)
						return true;
			}
		}
		catch (TransformerException e)
		{
			e.printStackTrace();
		}
		return false;
	}

	private Hashtable cachedNodeSet = new Hashtable();

	protected void cacheNodeSet(String xpath, NodeList nodeSet)
	{
		cachedNodeSet.put(xpath, nodeSet);
	}

	protected NodeList getCachedNodeSet(String xpath)
	{
		NodeList nodeSet = (NodeList) cachedNodeSet.get(xpath);
		if (nodeSet == null)
		{
			try
			{
				nodeSet = xpathAPI.selectNodeList(observable.getWrappedElement(), xpath);
			}
			catch (TransformerException e)
			{
				e.printStackTrace();
			}
			cacheNodeSet(xpath, nodeSet);
		}
		return nodeSet;
	}

	protected boolean nodeMatchesCachedXPaths(Node node, String[] xpaths)
	{
		for (int i = 0; i < xpaths.length; i++)
		{
			NodeList nodeSet = getCachedNodeSet(xpaths[i]);
			for (int n = 0; n < nodeSet.getLength(); n++)
				if (nodeSet.item(n) == node)
					return true;
		}
		return false;
	}

	/* (non-Javadoc)
	* @see org.w3c.dom.events.EventListener#handleEvent(org.w3c.dom.events.Event)
	*/
	public void handleEvent(Event e)
	{
		MutationEvent event = (MutationEvent) e;
		ObserverInterface[] subscribedObservers = observable.getSubscribedObserverInterfaces();
		for (int o = 0; o < subscribedObservers.length; o++)
		{
			ObserverInterface observerInterface = subscribedObservers[o];
			Node node = (Node) event.getTarget();
			// when the node is in the xpath selection, we call the appropriate method
			// depending on the event type.
			if (event.getType().equals("DOMAttrModified"))
			{
				// concerned node is the event target except for atributes
				node = event.getRelatedNode();
				if (nodeMatchesCachedXPaths(node, observerInterface.getXPaths()))
					observerInterface.attributeValueChanged((Attr) node, event.getPrevValue(), event.getNewValue());
			}
			else if (event.getType().equals("DOMCharacterDataModified"))
			{
				if (nodeMatchesCachedXPaths(node, observerInterface.getXPaths()))
					observerInterface.nodeTextChanged(node, event.getPrevValue(), event.getNewValue());
			}
			else if (event.getType().equals("DOMNodeRemovedFromDocument"))
			{
				if (nodeMatchesXPaths(node, observerInterface.getXPaths()))
					observerInterface.nodeRemoved(node);
			}
			else if (event.getType().equals("DOMNodeInsertedIntoDocument"))
			{
				if (nodeMatchesXPaths(node, observerInterface.getXPaths()))
					observerInterface.nodeInserted(node);
			}
		}
	}

	// NOTE
	//	- Redundant events: DOMNodeInserted and DOMNodeRemoved
	//	- Not managed events: DOMElementNameChanged and DOMAttributeNameChanged
}
