/*
 * Created on 3 avr. 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package dopidom.dom;

import java.util.Vector;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Olivier Beaudoux
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class DOMPath implements NodeList
{
	protected Vector path;

	public DOMPath(Element element)
	{
		this(element.getOwnerDocument().getDocumentElement(), element);
	}

	/**
	 * Local path, from root (exclusive) to element (inclusive)
	 */
	public DOMPath(Element root, Element element)
	{
		path = new Vector();
		Node parent = element;
		while ((parent != root) && (parent != null))
		{
			path.add(0, parent);
			parent = parent.getParentNode();
		}
		// in case of an invalid path (ie root is not an ancestor of element), the path is emptied
		if (parent == null)
			path.clear();
	}

	/**
	 * Local path, from root (exclusive) and following the numbering
	 * @param root
	 * @param numbering
	 */
	public DOMPath(Element root, int[] numbering)
	{
		path = new Vector();
		Node node = root;
		for (int n = 0; n < numbering.length; n++)
		{
			NodeList children = node.getChildNodes();
			if (numbering[n] < children.getLength())
			{
				node = children.item(numbering[n]);
				path.add(node);
			}
			// in case of an invalid path (ie the numbering is wrong), the path is emptied
			else
			{
				path.clear();
				break;
			}
		}
	}

	public boolean isAfter(DOMPath other)
	{
		if (getElement() == other.getElement())
			return false;
		// search for the divergence point of  this and other paths occurs
		int minPathLength = Math.min(path.size(), other.path.size());
		for (int n = 0; n < minPathLength; n++)
		{
			Node thisPathNode = (Node) path.get(n);
			Node otherPathNode = (Node) other.path.get(n);
			// we have found the divergence point
			if (thisPathNode != otherPathNode)
				// the relative node positions among the two paths at this index gives the precedence relation
				return getNodeIndex(thisPathNode) > getNodeIndex(otherPathNode);
			// if no divergence point are found, this means that one path is include in the other: the included path is considered as
			// preceding its "container" path
			else if (n == minPathLength - 1)
				return path.size() > other.path.size();
		}
		return false;
	}

	public int[] getNumbering()
	{
		int[] uniqueID = new int[path.size()];
		for (int i = 0; i < uniqueID.length; i++)
			uniqueID[i] = getNodeIndex((Node) path.get(i));
		return uniqueID;
	}

	private static int getNodeIndex(Node node)
	{
		Node parent = node.getParentNode();
		NodeList children = parent.getChildNodes();
		for (int i = 0; i < children.getLength(); i++)
			if (children.item(i) == node)
				return i;
		return -1;
	}

	public Element getElement()
	{
		if (getLength() == 0)
			return null;
		else
			return (Element) item(getLength() - 1);
	}

	/* (non-Javadoc)
	 * @see org.w3c.dom.NodeList#item(int)
	 */
	public Node item(int index)
	{
		return (Node) path.get(index);
	}

	/* (non-Javadoc)
	 * @see org.w3c.dom.NodeList#getLength()
	 */
	public int getLength()
	{
		return path.size();
	}

}
