/*
 * Created on 2 mai 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package dopidom.points;

import java.util.Stack;
import java.util.Vector;

import org.w3c.dom.Element;
import org.w3c.dom.events.Event;

import dopidom.Action;
import dopidom.ActionConsumer;
import dopidom.ActionEvent;
import dopidom.ActionProducer;
import dopidom.svg.SVGEventPoint;

/**
 * @author Olivier Beaudoux
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class HistoryPoint extends SVGEventPoint
{
	protected Stack stack;
	protected int undoDepth;

	/**
	 * @param pointedElement
	 */
	public HistoryPoint(Element pointedElement)
	{
		super(pointedElement, HistoryPoint.class, false);
		stack = new Stack();
		undoDepth = 0;
	}

	/**
	 * @see dopidom.EventPoint#getEventTypes()
	 */
	public String[] getEventTypes()
	{
		return new String[] {
			ActionEvent.BEGIN_INTERACTION_EVENT,
			ActionEvent.DO_ACTION_EVENT,
			HistoryEvent.UNDO_LAST_ACTIONS_EVENT,
			HistoryEvent.REDO_LAST_ACTIONS_EVENT };
	}

	/**
	 * @see org.w3c.dom.events.EventListener#handleEvent(org.w3c.dom.events.Event)
	 */
	public void handleEvent(Event event)
	{
		ActionEvent actionEvent = (ActionEvent) event;
		if (event.getType().equals(ActionEvent.BEGIN_INTERACTION_EVENT))
			newEntry(actionEvent);
		else if (event.getType().equals(ActionEvent.DO_ACTION_EVENT))
		{
			Entry entry = getEntry(actionEvent);
			if (entry != null)
				entry.pushAction(actionEvent.getAction());
		}
		else if (event.getType().equals(HistoryEvent.UNDO_LAST_ACTIONS_EVENT))
			undoLastAction();
		else // if (event.getType().equals(HistoryEvent.REDO_LAST_ACTIONS_EVENT))
			redoLastAction();
		actionEvent.stopPropagation(getEventGroup());
	}

	/**
	 * Push a new action on the top of the history stack. 
	 * If some actions have been undone (i.e. undoDepth > 0), we have to pop all actions upon the
	 * undo depth index.
	 */
	protected Entry newEntry(ActionEvent beginEvent)
	{
		// remove actions from the history at the undoDepth  beginning point
		if (undoDepth > 0)
		{
			for (int e = 0; e < undoDepth; e++)
				stack.pop();
			undoDepth = 0;
		}
		// push a new entry
		Entry entry = new Entry(beginEvent);
		stack.push(entry);
		return entry;
	}

	protected Entry getEntry(ActionEvent actionEvent)
	{
		for (int e = stack.size() - 1; e >= 0; e--)
		{
			Entry entry = (Entry) stack.get(e);
			if ((entry.getConsumer() == actionEvent.getConsumer())
				&& (entry.getProducer() == actionEvent.getProducer())
				&& (entry.getActionClass() == actionEvent.getActionClass()))
				return entry;
		}
		return null;
	}

	protected boolean canUndoLastAction()
	{
		return undoDepth < stack.size();
	}

	protected boolean canRedoLastAction()
	{
		return undoDepth > 0;
	}

	protected void undoLastAction()
	{
		if (canUndoLastAction())
		{
			Entry entry = (Entry) stack.get(stack.size() - 1 - undoDepth);
			entry.getProducer().undoActionsOn(entry.getActions(), entry.getConsumer());
			undoDepth++;
		}
	}

	protected void redoLastAction()
	{
		if (canRedoLastAction())
		{
			undoDepth--;
			Entry entry = (Entry) stack.get(stack.size() - 1 - undoDepth);
			entry.getProducer().redoActionsOn(entry.getActions(), entry.getConsumer());
		}
	}

	protected class Entry
	{
		protected Vector actions;
		protected ActionEvent beginEvent;

		public Entry(ActionEvent beginEvent)
		{
			this.beginEvent = beginEvent;
			actions = new Vector();
		}

		public void pushAction(Action action)
		{
			actions.add(action);
		}

		public Action[] getActions()
		{
			return (Action[]) actions.toArray(new Action[actions.size()]);
		}

		public ActionConsumer getConsumer()
		{
			return beginEvent.getConsumer();
		}

		public ActionProducer getProducer()
		{
			return beginEvent.getProducer();
		}

		public Class getActionClass()
		{
			return beginEvent.getActionClass();
		}
	}
}
