/*
 * Created on 2 mai 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package dopidom.points;

import java.util.Hashtable;
import java.util.Vector;

import org.w3c.dom.Element;
import org.w3c.dom.events.Event;
import org.w3c.dom.events.EventTarget;

import dopidom.Action;
import dopidom.ActionConsumer;
import dopidom.ActionEvent;
import dopidom.ActionInterface;
import dopidom.ActionProducer;
import dopidom.Component;
import dopidom.dom.DOMComponent;
import dopidom.dom.DOMPath;
import dopidom.svg.SVGActionEvent;
import dopidom.svg.SVGEventPoint;

/**
 * @author Olivier Beaudoux
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class LocalReplicationPoint extends SVGEventPoint implements ActionProducer
{
	/**
	 * Key= name of the replication point , value= vector of replication points matching the name
	 */
	protected static final Hashtable map = new Hashtable();

	protected String name;

	public LocalReplicationPoint(Element pointedElement, String[] arguments)
	{
		super(pointedElement, LocalReplicationPoint.class, false);
		this.name = arguments[0];
		register();
	}

	public String getName()
	{
		return name;
	}

	protected void register()
	{
		if (!map.containsKey(getName()))
			map.put(getName(), new Vector());
		Vector replicationPoints = (Vector) map.get(getName());
		replicationPoints.add(this);
	}

	//=====================
	// EventPoint interface
	//=====================

	/* (non-Javadoc)
	 * @see dopidom.EventPoint#getEventTypes()
	 */
	public String[] getEventTypes()
	{
		return new String[] {
			ActionEvent.BEGIN_INTERACTION_EVENT,
			ActionEvent.DO_ACTION_EVENT,
			ActionEvent.END_INTERACTION_EVENT,
			ActionEvent.UNDO_ACTIONS_EVENT,
			ActionEvent.REDO_ACTIONS_EVENT };
	}

	/* (non-Javadoc)
	 * @see org.w3c.dom.events.EventListener#handleEvent(org.w3c.dom.events.Event)
	 */
	public void handleEvent(Event event)
	{
		Vector replicationPoints = (Vector) map.get(getName());
		for (int p = 0; p < replicationPoints.size(); p++)
			if (replicationPoints.get(p) != this)
			{
				LocalReplicationPoint replicationPoint = (LocalReplicationPoint) replicationPoints.get(p);
				// replicate action to all other replication points
				if (replicationPoint != this)
				{
					ActionEvent actionEvent = (ActionEvent) event;
					// check for the target consumer within this replication point
					DOMPath sourcePath =
						new DOMPath(getAttachedElement(), actionEvent.getConsumer().getWrappedElement());
					DOMPath targetPath = new DOMPath(replicationPoint.getAttachedElement(), sourcePath.getNumbering());
					Element targetElement = targetPath.getElement();
					// now replicate the action throughout the producer interface of this replication point
					if (targetElement != null)
					{
						Component consumer = DOMComponent.getFactory().getWrapperComponent(targetElement);
						if (event.getType().equals(ActionEvent.BEGIN_INTERACTION_EVENT))
							beginInteractionOn(actionEvent.getActionClass(), consumer);
						else if (event.getType().equals(ActionEvent.DO_ACTION_EVENT))
							doActionOn(actionEvent.getAction(), consumer);
						else if (event.getType().equals(ActionEvent.END_INTERACTION_EVENT))
							endInteractionOn(actionEvent.getActionClass(), consumer);
						else if (event.getType().equals(ActionEvent.UNDO_ACTIONS_EVENT))
							undoActionsOn(actionEvent.getActions(), consumer);
						else if (event.getType().equals(ActionEvent.REDO_ACTIONS_EVENT))
							redoActionsOn(actionEvent.getActions(), consumer);
					}
					// do not propagate to other local RP
					actionEvent.stopPropagation(getEventGroup());
				}
			}
	}

	//===================
	// Producer interface
	//===================

	/* (non-Javadoc)
	 * @see dopidom.Producer#getWrappedElement()
	 */
	public Element getWrappedElement()
	{
		return getAttachedElement();
	}

	/* (non-Javadoc)
	 * @see dopidom.Producer#beginInteractionOn(java.lang.Class, dopidom.Consumer)
	 */
	public void beginInteractionOn(Class actionClass, ActionConsumer consumer)
	{
		SVGActionEvent event = new SVGActionEvent();
		event.initActionEvent(ActionEvent.BEGIN_INTERACTION_EVENT, this, consumer, actionClass);
		fireActionReplicationEvent(event);
	}

	/* (non-Javadoc)
	 * @see dopidom.Producer#endInteractionOn(java.lang.Class, dopidom.Consumer)
	 */
	public void endInteractionOn(Class actionClass, ActionConsumer consumer)
	{
		SVGActionEvent event = new SVGActionEvent();
		event.initActionEvent(ActionEvent.END_INTERACTION_EVENT, this, consumer, actionClass);
		fireActionReplicationEvent(event);
	}

	/**
	 * @see dopidom.Producer#doActionOn(dopidom.Action, dopidom.Consumer)
	 */
	public void doActionOn(Action action, ActionConsumer consumer)
	{
		SVGActionEvent event = new SVGActionEvent();
		event.initActionEvent(ActionEvent.DO_ACTION_EVENT, this, consumer, action);
		fireActionReplicationEvent(event);
	}

	/* (non-Javadoc)
	 * @see dopidom.Producer#undoActionsOn(dopidom.Action[], dopidom.Consumer)
	 */
	public void undoActionsOn(Action[] actions, ActionConsumer consumer)
	{
		SVGActionEvent event = new SVGActionEvent();
		event.initActionEvent(ActionEvent.UNDO_ACTIONS_EVENT, this, consumer, actions);
		fireActionReplicationEvent(event);
	}

	/* (non-Javadoc)
	 * @see dopidom.Producer#redoActionsOn(dopidom.Action[], dopidom.Consumer)
	 */
	public void redoActionsOn(Action[] actions, ActionConsumer consumer)
	{
		SVGActionEvent event = new SVGActionEvent();
		event.initActionEvent(ActionEvent.REDO_ACTIONS_EVENT, this, consumer, actions);
		fireActionReplicationEvent(event);
	}

	protected void fireActionReplicationEvent(ActionEvent event)
	{
		EventTarget eventTarget = (EventTarget) event.getConsumer().getWrappedElement();
		ActionInterface actionInterface = event.getConsumer().getActionInterface(event.getActionClass());
		SVGActionEvent replicatedEvent = new SVGActionEvent();
		replicatedEvent.initActionEvent(ActionEvent.REPLICATION_EVENT, event);
		eventTarget.addEventListener(replicatedEvent.getType(), actionInterface, false);
		eventTarget.dispatchEvent(replicatedEvent);
		eventTarget.removeEventListener(replicatedEvent.getType(), actionInterface, false);
	}

	/**
	 * Not very usefull in the context of replication point...
	 * @see dopidom.Producer#doActionOnceOn(dopidom.Action, dopidom.Consumer)
	 */
	public void doActionOnceOn(Action action, ActionConsumer consumer)
	{
		beginInteractionOn(action.getClass(), consumer);
		doActionOn(action, consumer);
		endInteractionOn(action.getClass(), consumer);
	}

	/** 
	 * Since the action replication is performed by replication points, the producer is the replication
	 * point and echo must be thus performed by it.
	 * @see dopidom.Producer#doActionOnceOn(dopidom.Action, dopidom.Consumer)
	 */
	public void echoActionsOn(Action[] actions, ActionConsumer consumer)
	{
		SVGActionEvent event = new SVGActionEvent();
		event.initActionEvent(ActionEvent.ECHO_ACTIONS_EVENT, this, consumer, actions);
		fireActionEvent(event);
	}

	protected void fireActionEvent(ActionEvent event)
	{
		EventTarget eventTarget = (EventTarget) event.getConsumer().getWrappedElement();
		ActionInterface actionInterface = event.getConsumer().getActionInterface(event.getActionClass());
		eventTarget.addEventListener(event.getType(), actionInterface, false);
		eventTarget.dispatchEvent(event);
		eventTarget.removeEventListener(event.getType(), actionInterface, false);
	}

}
