/*
 * Created on 12 mars 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package dopidom.svg;

import java.util.Vector;

import org.apache.batik.dom.events.AbstractEvent;

import dopidom.Action;
import dopidom.ActionEvent;
import dopidom.ActionConsumer;
import dopidom.ActionProducer;

/**
 * @author Olivier Beaudoux
 *
 * Note : SVG/Batik based implementation of ActionEvent
 */
public class SVGActionEvent extends AbstractEvent implements ActionEvent
{
	protected ActionProducer producer;
	protected ActionConsumer consumer;
	protected Class actionClass;
	protected Action action;
	protected Action[] actions;
	protected ActionEvent replicatedEvent;

	protected Vector stoppingGroups;

	protected void initActionEvent(String type, ActionProducer producer, ActionConsumer consumer)
	{
		// bubbling is always true for any action events
		initEvent(type, true, false);
		this.producer = producer;
		this.consumer = consumer;
	}

	/**
	 * @see dopidom.events.InteractionEvent#initActionEvent(java.lang.String, dopidom.ActionProducer, dopidom.ActionConsumer, java.lang.Class)
	 */
	public void initActionEvent(String type, ActionProducer producer, ActionConsumer consumer, Class actionClass)
	{
		initActionEvent(type, producer, consumer);
		this.actionClass = actionClass;
	}

	/**
	 * @see dopidom.events.InteractionEvent#initActionEvent(java.lang.String, dopidom.ActionProducer, dopidom.ActionConsumer, dopidom.Action)
	 */
	public void initActionEvent(String type, ActionProducer producer, ActionConsumer consumer, Action action)
	{
		initActionEvent(type, producer, consumer, action.getClass());
		this.action = action;
	}

	/**
	 * @see dopidom.events.InteractionEvent#initActionEvent(java.lang.String, dopidom.ActionProducer, dopidom.ActionConsumer, dopidom.Action[])
	 */
	public void initActionEvent(String type, ActionProducer producer, ActionConsumer consumer, Action[] actions)
	{
		initActionEvent(type, producer, consumer, actions[0].getClass());
		this.actions = actions;
	}

	/**
	 * @see dopidom.events.InteractionEvent#initActionEvent(java.lang.String, dopidom.events.ActionEvent)
	 */
	public void initActionEvent(String type, ActionEvent replicatedEvent)
	{
		initActionEvent(
			type,
			replicatedEvent.getProducer(),
			replicatedEvent.getConsumer(),
			replicatedEvent.getActionClass());
		this.replicatedEvent = replicatedEvent;
	}

	/**
	 * @see dpidom.action.Action#getProducer()
	 */
	public ActionProducer getProducer()
	{
		return producer;
	}

	/**
	 * @see dpidom.action.ActionEvent#getConsumer()
	 */
	public ActionConsumer getConsumer()
	{
		return consumer;
	}

	/**
	 * @see dopidom.events.InteractionEvent#getActionClass()
	 */
	public Class getActionClass()
	{
		return actionClass;
	}

	/**
	 * @see dopidom.events.InteractionEvent#getAction()
	 */
	public Action getAction()
	{
		return action;
	}

	/**
	 * @see dopidom.events.InteractionEvent#getActions()
	 */
	public Action[] getActions()
	{
		return actions;
	}

	/**
	 * @see dopidom.events.InteractionEvent#getReplicatedEvent()
	 */
	public ActionEvent getReplicatedEvent()
	{
		return replicatedEvent;
	}

	/**
	 * Only works within EventPoint objects (this is a DOM Events Level 3 feature, 
	 * but Batik is based on the Level 2...)
	 * @see org.w3c.dom.events.Event#stopPropagation()
	 */
	public void stopPropagation(Object eventGroup)
	{
		if (stoppingGroups == null)
			stoppingGroups = new Vector();
		stoppingGroups.add(eventGroup);
	}

	public boolean isPropagationStopped(Object eventGroup)
	{
		return (stoppingGroups != null) && stoppingGroups.contains(eventGroup);
	}
}
