/*
 * Created on 12 fvr. 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package dopidom.svg;

import java.util.Vector;

import org.w3c.dom.Element;
import org.w3c.dom.events.EventTarget;

import test.interfaces.Deletable;

import dopidom.Action;
import dopidom.ActionConsumer;
import dopidom.ActionEvent;
import dopidom.ActionInterface;
import dopidom.Query;
import dopidom.QueryConsumer;
import dopidom.QueryInterface;
import dopidom.dom.DOMComponent;

/**
 * @author Olivier Beaudoux
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */

public abstract class SVGComponent extends DOMComponent
{
	
	protected SVGComponent(Element wrappedElement)
	{
		super(wrappedElement);
	}

	//====================
	// Markable interface
	//====================

	//=========================
	// ActionConsumer interface
	//=========================

	protected Vector actionInterfaces = new Vector();

	public void registerActionInterface(ActionInterface actionInterface)
	{
		actionInterfaces.add(actionInterface);
	}

	public ActionInterface[] getActionInterfaces()
	{
		return (ActionInterface[]) actionInterfaces.toArray(new ActionInterface[actionInterfaces.size()]);
	}

	public ActionInterface getActionInterface(Class actionClass)
	{
		ActionInterface[] actionInterfaces = getActionInterfaces();
		for (int i = 0; i < actionInterfaces.length; i++)
		{
			ActionInterface actionInterface = (ActionInterface) actionInterfaces[i];
			if (actionInterface.getAction() == actionClass)
				return actionInterface;
		}
		return null;
	}

	public boolean canDoAction(Class actionClass)
	{
		ActionInterface actionInterface = getActionInterface(actionClass);
		return (actionInterface != null) && actionInterface.canDoAction(actionClass);
	}

	//========================
	// QueryConsumer interface
	//========================

	protected Vector queryInterfaces = new Vector();

	public void registerQueryInterface(QueryInterface queryInterface)
	{
		queryInterfaces.add(queryInterface);
	}

	public QueryInterface[] getQueryInterfaces()
	{
		return (QueryInterface[]) queryInterfaces.toArray(new QueryInterface[queryInterfaces.size()]);
	}

	public QueryInterface getQueryInterface(Class queryClass)
	{
		QueryInterface[] queryInterfaces = getQueryInterfaces();
		for (int i = 0; i < queryInterfaces.length; ++i) {
			QueryInterface queryInterface = queryInterfaces[i];
			if (queryInterface.getQuery() == queryClass)
				return queryInterface;
		}
		return null;
	}

	public boolean canDoQuery(Class queryClass)
	{
		QueryInterface queryInterface = getQueryInterface(queryClass);
		return (queryInterface != null) && queryInterface.canDoQuery(queryClass);
	}

	//=========================
	// ActionProducer interface
	//=========================

	public void doActionOnceOn(Action action, ActionConsumer consumer)
	{
		beginInteractionOn(action.getClass(), consumer);
		doActionOn(action, consumer);
		endInteractionOn(action.getClass(), consumer);
	}

	public void beginInteractionOn(Class actionClass, ActionConsumer consumer)
	{
		SVGActionEvent event = new SVGActionEvent();
		event.initActionEvent(ActionEvent.BEGIN_INTERACTION_EVENT, this, consumer, actionClass);
		fireActionEvent(event);
	}

	public void endInteractionOn(Class actionClass, ActionConsumer consumer)
	{
		SVGActionEvent event = new SVGActionEvent();
		event.initActionEvent(ActionEvent.END_INTERACTION_EVENT, this, consumer, actionClass);
		fireActionEvent(event);
	}

	/* (non-Javadoc)
	 * @see dpidom.action.Producer#exec(dpidom.action.Action, dpidom.action.Consumer)
	 */
	public void doActionOn(Action action, ActionConsumer consumer)
	{
		SVGActionEvent event = new SVGActionEvent();
		event.initActionEvent(ActionEvent.DO_ACTION_EVENT, this, consumer, action);
		fireActionEvent(event);
	}

	/**
	 * This method is provided for convenience only. The echo is not produced by a Component but rather
	 * by a replication point (another kind of action producer).
	 * @see dopidom.Producer#echoActionsOn(dopidom.Action[], dopidom.Consumer)
	 */
	public void echoActionsOn(Action[] actions, ActionConsumer consumer)
	{
		SVGActionEvent event = new SVGActionEvent();
		event.initActionEvent(ActionEvent.ECHO_ACTIONS_EVENT, this, consumer, actions);
		fireActionEvent(event);
	}

	public void undoActionsOn(Action[] actions, ActionConsumer consumer)
	{
		SVGActionEvent event = new SVGActionEvent();
		event.initActionEvent(ActionEvent.UNDO_ACTIONS_EVENT, this, consumer, actions);
		fireActionEvent(event);
	}

	public void redoActionsOn(Action[] actions, ActionConsumer consumer)
	{
		SVGActionEvent event = new SVGActionEvent();
		event.initActionEvent(ActionEvent.REDO_ACTIONS_EVENT, this, consumer, actions);
		fireActionEvent(event);
	}

	protected void fireActionEvent(ActionEvent event)
	{
		EventTarget eventTarget = (EventTarget) event.getConsumer().getWrappedElement();
		ActionInterface actionInterface = event.getConsumer().getActionInterface(event.getActionClass());
		eventTarget.addEventListener(event.getType(), actionInterface, false);
		eventTarget.dispatchEvent(event);
		eventTarget.removeEventListener(event.getType(), actionInterface, false);
	}

	//=========================
	// QueryProducer interface
	//=========================

	public void doQueryOn(Query query, QueryConsumer consumer)
	{
		QueryInterface queryInterface = consumer.getQueryInterface(query.getClass());
		queryInterface.doQuery(query);
	}

}
