/*
 * Created on 7 avr. 2005
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
package interactions;

import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import org.w3c.dom.Node;
import org.w3c.dom.svg.SVGLineElement;

import test.actions.DeleteCharacter;
import test.actions.EditText;
import test.actions.InsertCharacter;
import test.interfaces.CharacterHitable;
import test.queries.HitCharacter;
import dopidom.svg.SVGCleanToolInteractor;
import dopidom.svg.SVGInteraction;
import dopidom.svg.SVGPicking;
import dopidom.svg.SVGScene;

/**
 * @author fondemen
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
public class CaretManager extends SVGInteraction {
	
	protected CharacterHitable edited = null;

	protected volatile SVGLineElement caretElement;
	protected volatile HitCharacter hitCharacter;

	/**
	 * @param interactor
	 */
	public CaretManager(SVGCleanToolInteractor interactor) {
		super(interactor);

		caretElement = (SVGLineElement) SVGScene.getInstance().getRootElement().getElementById("caret-symbol");
		caretElement = (SVGLineElement) caretElement.cloneNode(true);
	}


	protected void insertCaretQueued(final CharacterHitable c, final int x, final int y)
	{
		SVGScene.getInstance().invokeLater(new Runnable()
		{
			public void run()
			{
				insertCaret(c, x, y);
			}
		});
	}

	protected void insertCaret(CharacterHitable c, int x, int y)
	{
		removeCaret();
		// update the caret location
		hitCharacter = new HitCharacter(x, y);
		c.doQuery(hitCharacter);
		moveCaret(c, 0);
		// insert the caret
		Node caretNewParent = c.getWrappedElement().getParentNode();
		caretNewParent.appendChild(caretElement);
	}

	protected void removeCaretQueued()
	{
		SVGScene.getInstance().invokeLater(new Runnable()
		{
			public void run()
			{
				removeCaret();
			}
		});
	}

	protected void removeCaret()
	{
		if (isCaretInserted())
			caretElement.getParentNode().removeChild(caretElement);
	}

	protected boolean isCaretInserted()
	{
		return caretElement.getParentNode() != null;
	}

	protected void moveCaretQueued(final CharacterHitable c,final int offset)
	{
		SVGScene.getInstance().invokeLater(new Runnable()
		{
			public void run()
			{
				moveCaret(c, offset);
			}
		});
	}

	protected void moveCaret(CharacterHitable c, int offset)
	{
		hitCharacter = new HitCharacter(hitCharacter.getIndex() + offset);
		c.doQuery(hitCharacter);
		caretElement.getX1().getBaseVal().setValue((float)hitCharacter.getX());
		caretElement.getX2().getBaseVal().setValue((float)hitCharacter.getX());
		caretElement.getY1().getBaseVal().setValue((float)hitCharacter.getY() + 2);
		caretElement.getY2().getBaseVal().setValue((float)(hitCharacter.getY() - hitCharacter.getHeight() - 2));
	}
	
	public void mousePressed(MouseEvent event) {
		if (event.getClickCount() == 1 && event.getButton() == MouseEvent.BUTTON1) {
			SVGPicking p = SVGScene.getInstance().getPickedElementsUnder(event.getX(), event.getY(), getInteractor().getWrappedElement());
			if (EditText.canDoEdit(p.getComponent())) {
				this.edited = (CharacterHitable)p.getComponent().getQueryInterface(HitCharacter.class);
				insertCaretQueued(this.edited, event.getX(), event.getY());
			} else {
				removeCaretQueued();
				this.edited = null;
			}
		}		
	}


	public void keyTyped(KeyEvent event)
	{
		char character = event.getKeyChar();
		//TODO understand why del is 127
		if ( (character == KeyEvent.CHAR_UNDEFINED) || (character < 32) || (character == 127) )
			return;
		if (this.edited != null)
		{
			this.getInteractor().doActionOnceOn(
				new InsertCharacter(character, hitCharacter.getIndex()),
				this.edited.getOwnerComponent());
			moveCaretQueued(this.edited, +1);
		}
	}
	
	public void keyPressed(KeyEvent event)
	{
		int  key = event.getKeyCode();
		if (key == KeyEvent.VK_UNDEFINED)
			return;
		if (this.edited != null)
			switch (key)
			{
				case KeyEvent.VK_DELETE :
					this.getInteractor().doActionOnceOn(
						new DeleteCharacter(false, hitCharacter.getIndex()),
						this.edited.getOwnerComponent());
					moveCaretQueued(this.edited, 0);
					break;
				case KeyEvent.VK_BACK_SPACE :
					this.getInteractor().doActionOnceOn(
						new DeleteCharacter(true, hitCharacter.getIndex()),
						this.edited.getOwnerComponent());
					moveCaretQueued(this.edited, -1);
					break;
				case KeyEvent.VK_LEFT :
					moveCaretQueued(this.edited, -1);
					break;
				case KeyEvent.VK_RIGHT :
					moveCaretQueued(this.edited, +1);
					break;
			}
		
		super.keyPressed(event);
	}
	
}
