/*
 * Created on 7 avr. 2005
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
package interactions;

import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import test.actions.ResizeAction;
import test.actions.SelectAction;
import test.queries.Location;
import test.selection.SelectionManager;
import dopidom.Component;
import dopidom.dom.DOMComponent;
import dopidom.svg.SVGCleanToolInteractor;
import dopidom.svg.SVGInteraction;
import dopidom.svg.SVGPicking;
import dopidom.svg.SVGScene;

/**
 * @author fondemen
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
public class Selector extends SVGInteraction {
	
	protected volatile SVGPicking selectedPicking = null;
	
	protected Location selectionLocation = new Location();
	
	protected volatile boolean onMultipleSelection = false;

	protected volatile boolean canResize = false;
	protected volatile boolean onResize = false;
	
	/**
	 * @param interactor
	 */
	public Selector(SVGCleanToolInteractor interactor) {
		super(interactor);

		
		SelectionManager.INSTANCE.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				Component selected = ((SelectionManager)e.getSource()).getSelectedComponent();
				onResize = false;
				canResize = selected != null && selected.canDoAction(ResizeAction.class) && selected.canDoQuery(Location.class);
				if (canResize) {
					selected.getQueryInterface(Location.class).doQuery(selectionLocation);
				}
				
				System.out.println("Selection changed to " + selected);
		
			}});
	}

	public void mousePressed(MouseEvent e) {	
		if (e.getClickCount() == 1) {
			select(e.getX(), e.getY());
		}
	}
	
	public void keyPressed(KeyEvent e) {
		if (e.isControlDown() && e.getKeyCode() == KeyEvent.VK_A) {
			SelectionManager.INSTANCE.setSelectedComponents(DOMComponent.getFactory().getAllComponents());
		} else if (e.getKeyCode() == KeyEvent.VK_CONTROL) {
			this.onMultipleSelection = true;
		} else
			this.onMultipleSelection = false;
	}
	
	public void keyReleased(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_CONTROL)
			this.onMultipleSelection = false;
	}
	
	public void setSelectedPicking(SVGPicking selected) {
		Component s = (Component)selected.getActionConsumer(SelectAction.class);
		if (this.onMultipleSelection) {
			if (s != null) {
				if (SelectionManager.INSTANCE.isSelected(s))
					SelectionManager.INSTANCE.removeSelectedComponent(selected == null ? null : s);
				else
					SelectionManager.INSTANCE.addSelectedComponentLast(selected == null ? null : s);
			}
		} else {
			Component toSelect = (Component)selected.getActionConsumer(SelectAction.class);
			if (!SelectionManager.INSTANCE.isSelected(toSelect))
				SelectionManager.INSTANCE.setSelectedComponent(selected == null ? null : toSelect);
		}
	}
	
	/**
	 * @return Returns the selectionLocation.
	 */
	public Location getSelectionLocation() {
		return selectionLocation;
	}
	/**
	 * @param selectionLocation The selectionLocation to set.
	 */
	public void setSelectionLocation(Location selectionLocation) {
		this.selectionLocation = selectionLocation;
	}
	
	private void select(int x, int y) {
		SVGPicking picking = SVGScene.getInstance().getPickedElementsUnder(x, y, this.getInteractor().getWrappedElement());
		this.setSelectedPicking(picking);
	}
	
	private void unselect() {
		SelectionManager.INSTANCE.setSelectedComponent(null);
	}
	
}
