/*
 * Created on 7 avr. 2005
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
package interactions;

import ihm04.actions.Translate;
import interfaces.BorderFindable;
import interfaces.BorderSlidable;

import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import queries.GetBorder;
import test.queries.Location;
import test.selection.SelectionManager;
import PBXSComponents.Arrow;
import actions.BorderSlide;
import actions.Handle;
import dopidom.Component;
import dopidom.svg.SVGCleanToolInteractor;
import dopidom.svg.SVGInteraction;
import dopidom.svg.SVGPicking;
import dopidom.svg.SVGScene;

/**
 * @author fondemen
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
public class Translater extends SVGInteraction {
	private class Request {
		public final Component translatable;
		public final Location initialLocation;
		public final MouseEvent startPoint;
		public volatile MouseEvent done = null;
		public volatile MouseEvent request = null;
		public volatile boolean toBeTerminated = false;
		
		public Request(final Component translatable, final MouseEvent startPoint) {
			super();
			this.translatable = translatable;
			this.done = startPoint;
			this.startPoint = startPoint;
			if (this.translatable.canDoQuery(Location.class)) {
				this.initialLocation = new Location();
				getInteractor().doQueryOn(this.initialLocation, this.translatable);
			} else
				this.initialLocation = null;
		}
	}
	
	private class TranslaterRunner extends Thread {
		public static final int sample = 40;
		
		private final LinkedList requests = new LinkedList();
		
		public TranslaterRunner() {
			super();
			this.setDaemon(true);
		}
		
		public void run() {
			int nap;
			Iterator i;
			
			while(true) {
				int start = (int)new Date().getTime();
				synchronized (this.requests) {
					i = this.requests.iterator();
					
					while(i.hasNext()) {
						try {
							boolean terminate;
							MouseEvent from, tbd, sp;
							Component f;
							Location l;
							Request r = (Request)i.next();
							synchronized (r) {
								terminate = r.toBeTerminated;
								tbd = r.request;
								from = r.done;
								f = r.translatable;
								l = r.initialLocation;
								sp = r.startPoint;
							}
							if (terminate) {
								i.remove();
							}
							if (tbd != null) {
								boolean done = false;
								if (f != null) {
									if (f.canDoAction(Handle.class) && l != null) {
										double ix = l.getX() + l.getWidth() / 2;
										double iy = l.getY() + l.getHeight()/2;
										double ex = sp.getX() - ix;
										double ey = sp.getY() - iy;
										Handle h = new Handle(tbd.getX() - ex, tbd.getY() - ey);
										// The following line invokes the action in another thread...
										//getInteractor().doActionOn(h, f[k]);
										f.getActionInterface(Handle.class).doAction(h);
										done = true;
									} else {
										Translate t = new Translate(tbd.getX() - from.getX(), tbd.getY() - from.getY());
										// The following line invokes the action in another thread...
										//getInteractor().doActionOn(t, f[k]);
										f.getActionInterface(Translate.class).doAction(t);
										done = t.isDone();
									}
								}
								//((Pointer)it).getSelectionLocation().setX(tbd.getX() - it.getPreviousX());
								//((Pointer)it).getSelectionLocation().setY(tbd.getY() - it.getPreviousY());
								synchronized (r) {
									if (done)
										r.done = tbd;
									if (r.request == tbd)
										r.request = null;
								}
							}
						} catch (RuntimeException x) {
							x.printStackTrace();
						}
					}
				}
				
				
				try {
					synchronized(this.requests) {
						if (this.requests.isEmpty()) {
							this.requests.wait();
							start = (int)new Date().getTime();
						}
					}
					
					nap = sample - (((int)new Date().getTime()) - start);
					if (nap > 0)
						sleep(nap);
				} catch (InterruptedException e) {
					return;
				}
			}
		}
		
		protected Request getRequest(Component f) {
			synchronized (this.requests) {
				Iterator i = requests.iterator();
				Request r;
				while (i.hasNext()) {
					r = (Request)i.next();
					if (r.translatable == f  && ! r.toBeTerminated)
						return r;
				}
				return null;
			}
		}
		
		public void startTranslation (Component c, MouseEvent e) {
			Request r = new Request(c, e);
			synchronized(this.requests) {
				this.requests.addLast(r);
				this.requests.notify();
			}
		}
		
		public void startTranslation (Component [] c, MouseEvent e) {
			for (int i = 0; i < c.length; i++) {
				this.startTranslation(c[i], e);
			}
		}
		
		public void invokeTranslation (Component c, MouseEvent e) {
			Request r = this.getRequest(c);
			if (r != null) {
				synchronized (r) {
					r.request = e;
				}
			}
		}
		
		public void invokeTranslation (Component [] c, MouseEvent e) {
			for (int i = 0; i < c.length; i++) {
				this.invokeTranslation(c[i], e);
			}
		}
		
		public void stopTranslation (Component c, MouseEvent e) {
			Request r = this.getRequest(c);
			if (r != null) {
				synchronized (r) {
					r.toBeTerminated = true;
					r.request = e;
				}
			}
		}
		
		public void stopTranslation (Component [] c, MouseEvent e) {
			for (int i = 0; i < c.length; i++) {
				this.stopTranslation(c[i], e);
			}
		}
	}
	
	private final TranslaterRunner translateRunner;

	protected SVGPicking translating = null;
	protected Component [] followers = null;
	protected boolean translate = false;
	protected boolean freeSliding = false;
	//protected boolean handle = false;

	/**
	 * @param interactor
	 */
	public Translater(SVGCleanToolInteractor interactor) {
		super(interactor);
		this.translateRunner = new TranslaterRunner();
		this.translateRunner.start();
	}
	
	public void mousePressed(MouseEvent e) {
		SVGCleanToolInteractor it = this.getInteractor();
		SVGPicking picking = SVGScene.getInstance().getPickedElementsUnder(e.getX(), e.getY(), it.getWrappedElement());
		List sel = SelectionManager.INSTANCE.getSelectedComponentsAsList();
		boolean multimove = sel != null && ! sel.isEmpty() ;
				
		if (e.getButton() == MouseEvent.BUTTON1 && (picking.hasActionConsumer(Translate.class)||picking.hasActionConsumer(Handle.class))) {
			this.translating = picking;
			// if borderSlidable, don't translate it !
			//System.out.println (1);
			//System.out.println(((Arrow)picking.getActionConsumer(BorderSlide.class)).attached());
			//if (picking.hasActionConsumer(BorderSlide.class) && ((Arrow)picking.getActionConsumer(BorderSlide.class)).attached()) 
			if (picking.hasActionConsumer(BorderSlide.class) && ((BorderSlidable)picking.getActionConsumer(BorderSlide.class).getActionInterface(BorderSlide.class)).attached())
				this.getInteractor().beginInteractionOn(BorderSlide.class, this.translating.getActionConsumer(BorderSlide.class));
			/*else if (picking.hasActionConsumer(Handle.class)) {
				this.handle = true;
				//this.getInteractor().beginInteractionOn(BorderSlide.class, this.translating.getActionConsumer(BorderSlide.class));				
			}*/ else {
				if (picking.hasActionConsumer(BorderSlide.class)) {
					this.freeSliding = true;
					multimove = false;
				}
				this.translate = true;
				Component toMove = (Component)picking.getActionConsumer(Handle.class);
				if (toMove == null)
					toMove = (Component)picking.getActionConsumer(Translate.class);
				else
					multimove = false;
				if (multimove) {
					if (! sel.contains(toMove))
						sel.add(0, toMove);
					for (Iterator i = sel.iterator(); i.hasNext();) {
						Component c = (Component) i.next();
						if (!c.canDoAction(Translate.class) || (c.canDoAction(BorderSlide.class) && ((BorderSlidable)c.getActionInterface(BorderSlide.class)).attached()))
							i.remove();
					}
					boolean deleted;
					do {
						deleted = false;
						for (Iterator i = sel.iterator(); i.hasNext();) {
							boolean toDelete = false;
							Component c = (Component) i.next();
							List contained = Arrays.asList(c.getChildren());
							for (Iterator ci = contained.iterator(); !toDelete && ci.hasNext();) {
								Component cc = (Component) ci.next();
								toDelete = sel.contains(cc);
							}
							if (toDelete) {
								i.remove();
								deleted = true;
							}
						}
					} while(deleted);
					this.followers = (Component[])sel.toArray(new Component[sel.size()]);
				} else {
					this.followers = toMove == null ? null : new Component[] {toMove};
				}
				for (int i = 0; i < this.followers.length; i++) {
					if (this.followers[i].canDoAction(Handle.class))
						this.getInteractor().beginInteractionOn(Handle.class, followers[i]);
					else
						this.getInteractor().beginInteractionOn(Translate.class, followers[i]);
				}
				this.translateRunner.startTranslation(this.followers, e);
			}
		}
	}
	
	public void mouseDragged(MouseEvent e) {
		if (this.translate) {
			if (this.freeSliding) {//FIXME : incorporer au system de translation de frederic
				// car devrais faire un prob : on peut pas deplacer des arrow et des autres objets en meme temps.. le arrow prends la priorit
				for (int i = 0; i < this.followers.length; i++) {
					if (this.followers[i] instanceof Arrow) {
						if (((Arrow)this.followers[i]).isStartArrow())
							((Arrow)this.followers[i]).getAttachedLink().setStartLocation(new Point2D.Double(e.getX(), e.getY()));
						else
							((Arrow)this.followers[i]).getAttachedLink().setEndLocation(new Point2D.Double(e.getX(), e.getY()));
					}
						
				}
			} else
				this.translateRunner.invokeTranslation(this.followers, e);
		}
		/*else if (this.handle) {
			// TODO: conformer a l'architecture translateRunner
			Handle handle = new Handle (e.getX(),e.getY());
			this.translating.getActionConsumer(Handle.class).getActionInterface(Handle.class).doAction(handle);
		}*/
	}

	public void mouseReleased(MouseEvent e)
	{
		if (this.translating != null) {
			if (this.translate && !this.freeSliding) {
				this.getInteractor().endInteractionOn(Translate.class, this.translating.getActionConsumer(Translate.class));
				this.translateRunner.stopTranslation(this.followers, e);
				for (int i = 0; i < this.followers.length; i++) {
					if (this.followers[i].canDoAction(Handle.class))
						this.getInteractor().endInteractionOn(Handle.class, followers[i]);
					else
						this.getInteractor().endInteractionOn(Translate.class, followers[i]);
				}
			/*} else if (this.handle){
				this.handle = false;
			*/} else {
				anchorToBorder(e);
				this.getInteractor().endInteractionOn(BorderSlide.class, this.translating.getActionConsumer(BorderSlide.class));
			}

			this.translate = false;
			this.translating = null;
			this.followers = null;
		}
	}

	private void anchorToBorder(MouseEvent e) {
		if (this.translating.hasActionConsumer(BorderSlide.class)) {
			// changed : fin de l'interaction borderslidable
			// ancrer le composant sur son bord si il y en a un
			SVGCleanToolInteractor it = this.getInteractor();
			SVGPicking picking = SVGScene.getInstance().getPickedElementsUnder(e.getX(), e.getY(), it.getWrappedElement());
			if (picking.hasQueryConsumer(GetBorder.class)) {
				Component c = (Component)picking.getQueryConsumer(GetBorder.class);
				boolean attached = ((BorderSlidable)this.translating.getActionConsumer(BorderSlide.class).getActionInterface(BorderSlide.class)).attachTo((BorderFindable)c.getQueryInterface(GetBorder.class));
				if (attached && c.canDoQuery(GetBorder.class)) {
					GetBorder border = new GetBorder();
					this.getInteractor().doQueryOn(border, c);
					((BorderSlidable)this.translating.getActionConsumer(BorderSlide.class).getActionInterface(BorderSlide.class)).doAction(new BorderSlide(e.getX()- border.getLocation().getX(), e.getY()- border.getLocation().getY(), border.getPoints(), border.getLocation().getX(), border.getLocation().getY()));
				}
				this.freeSliding = false;
			}
		}
	}
}
