/*
 * Created on 12 mars 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package interfaces;

import listeners.DefaultListenable;
import listeners.Event;
import listeners.EventKind;
import listeners.Listenable;
import listeners.Listener;
import listeners.UncheckedEventException;

import org.apache.batik.util.SVGConstants;
import org.w3c.dom.Element;
import org.w3c.dom.svg.SVGAnimatedTransformList;
import org.w3c.dom.svg.SVGMatrix;
import org.w3c.dom.svg.SVGTransform;
import org.w3c.dom.svg.SVGTransformList;
import org.w3c.dom.svg.SVGTransformable;

import queries.GetOrigin;
import actions.DirectionAdjust;
import dopidom.Action;
import dopidom.Component;
import dopidom.svg.SVGActionInterface;
import event.CreationEvent;
import event.DirectionAdjustEvent;
import event.PositionEvent;

/**
 * @author Olivier Beaudoux
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class DirectionAdjustable extends SVGActionInterface implements Listenable
{
	public static final DefaultListenable DirAdjListener = new DefaultListenable();
	private DefaultListenable listenableDelegate = new DefaultListenable();
	
	public DirectionAdjustable(Component ownerComponent)
	{
		super(ownerComponent);
		DirAdjListener.testAndFireEvent(new CreationEvent(EventKind.DirectionAdjustableCreated, this));
	}

	public Class getAction()
	{
		return DirectionAdjust.class;
	}

	public Class[] getScope()
	{
		return new Class[] { SVGTransformable.class };
	}

	public void doAction(Action action)
	{
		DirectionAdjust da = (DirectionAdjust) action;
		setDirectionFromVector(da.getPosX(), da.getPosY());
	}

	public void undoActions(Action[] actions)
	{
	}

	public void redoActions(Action[] actions)
	{
	}

	protected void setDirectionFromVector(double px, double py)
	{
		if (px == 0 && py == 0)
			return;
		SVGTransformable transformable = (SVGTransformable) getWrappedElement();
		if (transformable == null) {
			System.err.println ("L'objet n'est pas transformable");
			return;
		}
		
		if (((Element)transformable).getAttributeNode(SVGConstants.SVG_TRANSFORM_ATTRIBUTE) == null) {
			//((Element)transformable).setAttribute(SVGConstants.SVG_TRANSFORM_ATTRIBUTE, SVGConstants.TRANSFORM_TRANSLATE + "(0,0)");
		}
		GetOrigin origin = new GetOrigin();
		if (!this.getOwnerComponent().canDoQuery(GetOrigin.class)) {
			System.err.println("Le composant suivant doit implmenter 'OriginGettable'");
			this.getOwnerComponent().toString();
			return;
		}
		this.getOwnerComponent().getQueryInterface(GetOrigin.class).doQuery(origin);
		double alpha = Math.atan((origin.getY()-py)/(origin.getX()-px));
		SVGAnimatedTransformList satl = transformable.getTransform();
		SVGTransformList stl = satl.getBaseVal();
		SVGTransform st = stl.consolidate();
		SVGMatrix matrix = st.getMatrix();
		float factor; 
		if (px<=origin.getX())
			factor = -1;
		else
			factor = 1;
		matrix.setA((float)Math.cos(alpha)*factor);
		matrix.setB((float)Math.sin(alpha)*factor);
		matrix.setC(-(float)Math.sin(alpha)*factor);
		matrix.setD((float)Math.cos(alpha)*factor);
		
		this.testAndFireEvent(new DirectionAdjustEvent(this.ownerComponent, alpha));
	}
	
	public void addListener(Listener l) {
		this.listenableDelegate.addListener(l);
	}
	
	public void removeListener(Listener l) {
		this.listenableDelegate.removeListener(l);
	}
	
	public void fireEvent(Event e) throws UncheckedEventException {
		this.listenableDelegate.fireEvent(e);
	}
	
	public void checkEvent(Event e) {
		this.listenableDelegate.checkEvent(e);
	}
	
	public boolean testAndFireEvent(Event e) {
		return this.listenableDelegate.testAndFireEvent(e);
	}
}