/*
 * Created on 14 mars 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package interfaces;

import ihm04.actions.Translate;

import java.util.Vector;

import listeners.DefaultListenable;
import listeners.Event;
import listeners.EventKind;
import listeners.Listenable;
import listeners.Listener;
import listeners.UncheckedEventException;

import test.listeners.SVGActionInterfaceListener;
import test.listeners.SVGListenableActionInterface;
import utils.ListCVarPersistant;
import PBXSComponents.Arrow;
import actions.BorderSlide;
import actions.Handle;
import actions.Stick;
import dopidom.Action;
import dopidom.ActionInterface;
import dopidom.Component;
import dopidom.svg.SVGActionInterface;
import dopidom.svg.SVGScene;
import event.CreationEvent;
import event.StickEvent;

/**
 * fired events:
 * 	-	Stick (on addSticker)
 * 	-	Unstick (on removeSticker)
 */
public class Stickable extends SVGActionInterface implements Listenable {
	public static final DefaultListenable StickableListener = new DefaultListenable();
	
	private volatile boolean listeningTranslate = false;
	private DefaultListenable listenableDelegate = new DefaultListenable();

	ListCVarPersistant stickers = new ListCVarPersistant(getWrappedElement(),
			"Stickable", "stickers") {

		protected boolean addWhileFilling(final Component obj) {
			boolean ret = super.addWhileFilling((Component) obj);
			if (ret)
				//Stickable.this is not available in constructor 
				SVGScene.getInstance().invokeLater(new Runnable() {

					public void run() {
						Stickable.this.createListeners((Component) obj);
						
					}});
			return ret;
		}

		public boolean add(Object obj) {
			boolean ret = super.add(obj);
			if (ret)
				Stickable.this.createListeners((Component) obj);
			return ret;
		}

	};

	//Vector stickers = new Vector();
	public Stickable(Component ownerComponent) {
		super(ownerComponent);
		StickableListener.testAndFireEvent(new CreationEvent(EventKind.StickableCreated, this));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see dopidom.ActionInterface#doAction(dopidom.Action)
	 */
	public void doAction(Action action) {
		// TODO Auto-generated method stub
		Stick translateAction = (Stick) action;
		for (int i = 0; i < stickers.size(); i++) {
			Component c = (Component) stickers.get(i);
			c.getActionInterface(Translate.class).doAction(
					new Translate(translateAction.getDeltaX(), translateAction
							.getDeltaY()));
			if (c instanceof Arrow)
				((Arrow) c).adjustArrow();
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see dopidom.ActionInterface#getAction()
	 */
	public Class getAction() {
		// TODO Auto-generated method stub
		return Stick.class;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see dopidom.ActionInterface#getScope()
	 */
	public Class[] getScope() {
		// TODO Auto-generated method stub
		return null;
	}

	private void createListeners(Component sticker) {
		if (!this.listeningTranslate) {
			//BorderSLidable invokes Translate : it should no be listened
			Component t = getTranslatable(this.getOwnerComponent());
			//Component b = getBorderSlidable(this.getOwnerComponent());
			Component h = getHandlable(this.getOwnerComponent());
			ActionInterface ti = t == null ? null : t.getActionInterface(Translate.class);
			//ActionInterface bi = b == null ? null : b.getActionInterface(BorderSlide.class);
			ActionInterface hi = h == null ? null : h.getActionInterface(Handle.class);
			if (ti != null /*|| bi != null*/ || hi != null) {
				SVGListenableActionInterface lti = (SVGListenableActionInterface) ti;
				//SVGListenableActionInterface lbi = (SVGListenableActionInterface) bi;
				SVGListenableActionInterface lhi = (SVGListenableActionInterface) hi;
				SVGActionInterfaceListener l = new SVGActionInterfaceListener() {
					
					public void handleEvent(String eventType,
							ActionInterface target, Action action) {
						Translate t = action instanceof Translate ? (Translate) action : null;
						//BorderSlide b = action instanceof BorderSlide ? (BorderSlide)action : null;
						Handle h = action instanceof Handle ? (Handle)action : null;
						if (h != null)
							Stickable.this.doAction(new Stick(h.getDeltaX(), h.getDeltaY()));
						//else if (b != null)
						//	Stickable.this.doAction(new Stick(b.getDeltaX(), b.getDeltaY()));
						else if (t != null)
							Stickable.this.doAction(new Stick(t.getDeltaX(), t.getDeltaY()));
					}

					public boolean isValid() {
						return !Stickable.this.getOwnerComponent().isDeleted();
					}

				};
				if (lti != null)
					lti.addListener(l);
				//if (lbi != null)
				//	lbi.addListener(l);
				if (lhi != null)
					lhi.addListener(l);
			} else {
				System.err
						.println("A stickable Component must implement a listenable border slide or translatable");
			}

			this.listeningTranslate = true;
		}
	}

	/**
	 * fired event: Unstick
	 */
	public boolean removeSticker(Component sticker) {
		Component tc = getTranslatable(sticker);
		Component bc = getBorderSlidable(sticker);
		Component hc = getHandlable(sticker);
		boolean ret = false;
		if (stickers.contains(tc)) {
			if (this.testAndFireEvent(new StickEvent(false, this, tc))) {
				stickers.remove(tc);
				ret = true;
			}
		}
		if (stickers.contains(bc)) {
			if (this.testAndFireEvent(new StickEvent(false, this, bc))) {
				stickers.remove(bc);
				ret = true;
			}
		}
		if (stickers.contains(hc)) {
			if (this.testAndFireEvent(new StickEvent(false, this, hc))) {
				stickers.remove(hc);
				ret = true;
			}
		}
		return ret;
	}

	protected Component getTranslatable(Component c) {
		return this.getSticker(c, Translate.class);
	}

	protected Component getBorderSlidable(Component c) {
		return this.getSticker(c, BorderSlide.class);
	}

	protected Component getHandlable(Component c) {
		return this.getSticker(c, Handle.class);
	}
	
	protected Component getSticker(Component c, Class action) {		
		while (c != null && !c.canDoAction(action))
			c = c.getParent();

		return c;
	}

	/**
	 * fired event: Stick
	 * @return false if the stick was refused
	 */
	public boolean addSticker(Component sticker) {
		Component tc = getTranslatable(sticker);
		Component bc = getBorderSlidable(sticker);
		Component hc = getHandlable(sticker);
		if (tc == null && bc == null && hc == null)
			System.err.println("a sticker must be either border slidable or translatable or handlable");
			
		boolean ret = false;
		if (tc != null && !stickers.contains(tc)) {
			if (this.testAndFireEvent(new StickEvent(true, this, tc))) {
				stickers.add(tc);
				ret = true;
			}
		}
		if (bc != null && !stickers.contains(bc)) {
			if (this.testAndFireEvent(new StickEvent(true, this, bc))) {
				stickers.add(bc);
				ret = true;
			}
		}
		if (hc != null && !stickers.contains(hc)) {
			if (this.testAndFireEvent(new StickEvent(true, this, hc))) {
				stickers.add(hc);
				ret = true;
			}
		}
		
		return ret;
		/*
		 * Element currentTarget = this.getWrappedElement(); SVGSVGElement root =
		 * SVGScene.getInstance().getCanvas().getSVGDocument().getRootElement();
		 * AbstractNode n =
		 * (AbstractNode)this.getOwnerComponent().getWrappedElement().getAttributeNode("transform");
		 * //SVGElement er; n.addEventListener("DOMSubtreeModified", new
		 * EventListener () { protected volatile boolean appending = false;
		 * public void handleEvent(Event evt) { if (appending) return; appending =
		 * true; GenericAttr attr = (GenericAttr)evt.getCurrentTarget();
		 * Stickable.this.doAction (new Stick
		 * (Translatable.currentTranslate.getDeltaX(),
		 * Translatable.currentTranslate.getDeltaY())); appending = false; }},
		 * false);
		 */
		
	}

	/**
	 * @return Returns the stickers.
	 */
	public Vector getStickers() {
		return stickers.getVector();
	}

	public void addListener(Listener l) {
		this.listenableDelegate.addListener(l);
	}

	public void removeListener(Listener l) {
		this.listenableDelegate.removeListener(l);
	}

	public void fireEvent(Event e) throws UncheckedEventException {
		this.listenableDelegate.fireEvent(e);
	}

	public void checkEvent(Event e) {
		this.listenableDelegate.checkEvent(e);
	}

	public boolean testAndFireEvent(Event e) {
		return this.listenableDelegate.testAndFireEvent(e);
	}

}