package interpretor;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.jmi.reflect.RefObject;

import test.model.ModelHelper;
import utils.ManipString;
import utils.SelfContainer;

public class SerializerKoala implements Serializer {
	
	public static final String Null_Literal = "<nothing>";
	
	public Object deserialize(String representation) {
		
		if (representation == null || representation.equals(Null_Literal)) return null;
		
		representation=representation.replace(" ","");
		representation=representation.replaceAll("\t","");
		representation=representation.replaceAll("\n","");
		
		String type = ManipString.extractVariableType(representation);
		String value = ManipString.extractVariableValue(representation);
		
		if (type != null && value != null && !value.equals("")){
			
			if (type.equals("Boolean")){
				return (new Boolean(value));
			} else if (type.equals("Double")){
				return new Double(value);
			} else if (type.equals("String")){
				return new String(value);
			} else if (type.equals("Integer")){
				return new Integer(value);
			} else if (type.equals("Object")){
				return SelfContainer.getInstance().getSelfByID(value);
			} else if (type.equals("Set")){
					
				List<String> obsName = ManipString.tokenize(value);
					Object[] obs = new Object[obsName.size()];
					for (int i = 0; i < obsName.size(); i++) {
						obs[i]=deserialize(obsName.get(i));
					}
					return obs;
			} else if (type.equals("Bag")){
				List<String> obsName = ManipString.tokenize(value);
				Object[] obs = new Object[obsName.size()];
				for (int i = 0; i < obsName.size(); i++) {
					obs[i]=deserialize(obsName.get(i));
				}
				return obs;
			} else if (type.equals("Sequence")){
				List<String> obsName = ManipString.tokenize(value);
				Object[] obs = new Object[obsName.size()];
				for (int i = 0; i < obsName.size(); i++) {
					obs[i]=deserialize(obsName.get(i));
				}
				return obs;
			} else if (type.equals("OrderedSet")){
				List<String> obsName = ManipString.tokenize(value);
				Object[] obs = new Object[obsName.size()];
				for (int i = 0; i < obsName.size(); i++) {
					obs[i]=deserialize(obsName.get(i));
				}
				return obs;
			} else {
				System.err.println("Unknowing type for "+representation);
			}
			
		} else {
			if (type == null) System.err.println("Type uncorrectly formulated in : "+ representation);
			if (value == null) System.err.println("Value uncorrectly formulated in : "+ representation);
		}
		return null;
	}

	public String serialize(Object o) {
		if (o instanceof Boolean)
			return this.serializeBoolean(((Boolean)o).booleanValue());
		else if (o instanceof String)
			return this.serializeString((String)o);
		else if (o instanceof Double)
			return this.serializeReal(((Double)o).doubleValue());
		else if (o instanceof Float)
			return this.serializeReal(((Float)o).floatValue());
		else if (o instanceof Integer)
			return this.serializeInteger(((Integer)o).intValue());
		else if (o instanceof Short)
			return this.serializeInteger(((Short)o).shortValue());
		else if (o instanceof Byte)
			return this.serializeInteger(((Byte)o).byteValue());
		else if (o instanceof Collection) {
			boolean isSet = o instanceof Set;
			boolean isOrdered = o instanceof List;
			
			if (isSet && isOrdered)
				return this.serializeOrderedSet(((Collection)o).toArray());
			else if (isSet && !isOrdered)
				return this.serializeSet(((Collection)o).toArray());
			else if (!isSet && isOrdered)
				return this.serializeSequence(((Collection)o).toArray());
			else //if (!isSet && !isOrdered)
				return this.serializeBag(((Collection)o).toArray());
		}
		else if (o instanceof RefObject)
			return this.serializeModelElement(o);
		else if (o == null)
			return Null_Literal;
		throw new IllegalArgumentException(o.toString() + " of class " + o.getClass().getName() + " cannot be serialized for Java (Koala) language...");
	}

	public String serializeModelElement(Object me) {
		String mofId = ModelHelper.getUniqId((RefObject)me);
		SelfContainer.getInstance().insertSelf(mofId,me);
		return ManipString.createSerialization("Object",mofId);
	}

	public String serializeBoolean(boolean b) {
		return ManipString.createSerialization("Boolean",new Boolean(b).toString());
	}

	public String serializeReal(double r) {
		return ManipString.createSerialization("Double",new Double(r).toString());
	}

	public String serializeString(String s) {
		return ManipString.createSerialization("String",s);
	}

	public String serializeInteger(int i) {
		return ManipString.createSerialization("Integer",new Integer(i).toString());
	}

	public String serializeSet(Object[] set) {
		
		String ret = "Set(";
		for (int i = 0; i < set.length; i++) {
			if(i!=0) ret=ret.concat(",");
			ret=ret.concat(serialize(set[i]));
		}
		ret=ret.concat(")");
		
		return ret;
	}

	public String serializeBag(Object[] bag) {

		String ret = "Bag(";
		for (int i = 0; i < bag.length; i++) {
			if(i!=0) ret=ret.concat(",");
			ret=ret.concat(serialize(bag[i]));
		}
		ret=ret.concat(")");
		
		return ret;
	}

	public String serializeSequence(Object[] sequence) {
		
		String ret = "Sequence(";
		for (int i = 0; i < sequence.length; i++) {
			if(i!=0) ret=ret.concat(",");
			ret=ret.concat(serialize(sequence[i]));
		}
		ret=ret.concat(")");
		
		return ret;
	}

	public String serializeOrderedSet(Object[] orderedSet) {

		String ret = "OrderedSet(";
		for (int i = 0; i < orderedSet.length; i++) {
			if(i!=0) ret=ret.concat(",");
			ret=ret.concat(serialize(orderedSet[i]));
		}
		ret=ret.concat(")");
		
		return ret;
	}

}
