package listeners;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

public class DefaultListenable implements Listenable {
	protected Map<EventKind, Set<Listener>> listeners = new HashMap<EventKind, Set<Listener>>();
	
	protected Set<Listener> getListenableListByKind(EventKind kind) {
		Set<Listener> ls = this.listeners.get(kind);
		if (ls == null) {
			ls = new TreeSet<Listener>();
			this.listeners.put(kind, ls);
		}
		return ls;
	}
	
	public void addListener(Listener l) {
		this.getListenableListByKind(l.getEventKind()).add(l);
	}
	
	public void removeListener(Listener l) {
		this.getListenableListByKind(l.getEventKind()).remove(l);
	}

	public void checkEvent(Event e) {
		for (Listener l : this.getListenableListByKind(e.getEventKind())) {
			if (l.vetoesEvent(e))
				return;
		}
		e.check();
	}
	
	public void fireEvent(Event e) throws UncheckedEventException {
		if (!e.isChecked())
			throw new UncheckedEventException(e);
		for (Listener l : this.getListenableListByKind(e.getEventKind())) {
			l.receiveEvent(e);
		}
	}
	
	public boolean testAndFireEvent(Event e) {
		this.checkEvent(e);
		if (e.isChecked()) {
			try {
				this.fireEvent(e);
			} catch (UncheckedEventException e1) {
				throw new RuntimeException("The impossible exception has occured !!!", e1);
			}
			return true;
		} else
			return false;
	}
}
