package listeners.initial;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Node;

import dopidom.Action;
import dopidom.ActionInterface;
import dopidom.Component;

import sc.StateMachine;
import test.actionqueries.Container;
import test.actionqueries.Contents;
import test.interfaces.Containable;
import test.listeners.SVGActionInterfaceListener;
import utils.DomFinding;
import utils.ListenerFunctions;
import utils.ManipString;

import listeners.Event;
import listeners.EventKind;
import listeners.Listener;


public abstract class ListenableListenersInitiator {
	
	private ListenableListenersInitiator() {}
	
	protected static void initiateContainableListener(Containable cd) {
		
		cd.addListener((SVGActionInterfaceListener)new ContainerChangedListener());
		
	}
	
	protected static void initiateContainerListener(test.interfaces.Container cr) {
		
		cr.addListener((SVGActionInterfaceListener)new ContainedChangedListener());
		
	}
}

class ContainerChangedListener implements SVGActionInterfaceListener, Listener {
	
	public void handleEvent(String eventType, ActionInterface target, Action action) {
		if (action instanceof Container) {
			
			Component container = ((Container)action).getContainer();
			Component contained /*that is self*/ = ((Container)action).getContained();
			
			if (container != null && contained!=null && !container.isDeleted() && !contained.isDeleted()){
				
				HashSet<String[]> listSpec = new HashSet<String[]>();
				List<String[]> listVars = DomFinding.domFindVariables(container.getWrappedElement());
				for (Iterator iter = listVars.iterator(); iter.hasNext();) {
					String[] element = (String[]) iter.next();
					element[0] = "containerComponent_".concat(element[0]);
					listSpec.add(element);
				}
				
				ListenerFunctions.foo("onContained",(Node)contained.getWrappedElement(),listSpec);
				ListenerFunctions.updateGraphicalElement(contained.getWrappedElement(), "container", container.getWrappedElement());
			}
		}
		this.receiveEvent(null);
	}
	
	public EventKind getEventKind() {
		// TODO Auto-generated method stub
		return null;
	}
	
	public boolean vetoesEvent(Event e) {
		return false;
	}
	
	public void receiveEvent(Event e) {
		// TODO Auto-generated method stub
		
	}

	public boolean isValid() {
		return true;
	}
	
}

class ContainedChangedListener implements SVGActionInterfaceListener, Listener {
	
	public void handleEvent(String eventType, ActionInterface target, Action action) {
		if (action instanceof Contents) {
			Component container = ((Contents)action).getContainer();
			Collection<Component> contained /*that is self*/ = ((Contents)action).getContents();
			boolean containedOK = true;
			for (Component component : contained) {
				containedOK = containedOK && !component.isDeleted();
			}
			
			if(container != null && contained!=null && container.isDeleted() && containedOK){
				
				HashSet<String[]> listSpec = new HashSet<String[]>();
				
				String[] ret = {"containedComponents", ManipString.generateSet(container, contained)};
				
				listSpec.add(ret);
				
				ListenerFunctions.foo("onContained",(Node)container.getWrappedElement(),listSpec);
				//GraphicalElement update done by onContainer event management				
			}
		}
		this.receiveEvent(null);
	}
	
	public EventKind getEventKind() {
		// TODO Auto-generated method stub
		return null;
	}
	
	public boolean vetoesEvent(Event e) {
		return false;
	}
	
	public void receiveEvent(Event e) {
		// TODO Auto-generated method stub
		
	}

	public boolean isValid() {
		return true;
	}
	
}
