package listeners.initial;

import test.interfaces.CharacterDeletable;
import test.interfaces.CharacterInsertable;
import test.interfaces.Containable;
import test.interfaces.Container;
import test.interfaces.Locatable;
import test.interfaces.Orderable;
import test.interfaces.Resizable;
import test.templates.SVGFileTemplate;
import event.CreationEvent;
import event.StickEvent;
import listeners.Event;
import listeners.EventKind;
import listeners.Listener;
import interfaces.DirectionAdjustable;
import interfaces.Positionable;
import interfaces.Stickable;
import interfaces.Translatable;

public final class ListenerInitiator {
	private ListenerInitiator(){}
	
	public static void initiateListeners() {
		Stickable.StickableListener.addListener(new Listener(){
			
			public EventKind getEventKind() {
				return EventKind.StickableCreated;
			}
			
			public boolean vetoesEvent(Event e) {
				return false;
			}
			
			public void receiveEvent(Event e) {
				StickableListenersInitiator.initiateStickableListener((Stickable)((CreationEvent)e).getCreatedElement());
			}});
		
		CharacterInsertable.CharacterInsertableListener.addListener(new Listener() {
			
			public EventKind getEventKind() {
				return EventKind.CharacterInsertableCreated;
			}
			
			public boolean vetoesEvent(Event e) {
				return false;
			}
			
			public void receiveEvent(Event e) {
				CharacterInsertableListenersInitiator.initiateCharacterInsertableListener((CharacterInsertable)((CreationEvent)e).getCreatedElement());
			}});
		
		CharacterDeletable.CharacterDeletableListener.addListener(new Listener() {
			
			public EventKind getEventKind() {
				return EventKind.CharacterDeletableCreated;
			}
			
			public boolean vetoesEvent(Event e) {
				return false;
			}
			
			public void receiveEvent(Event e) {
				CharacterDeletableListenersInitiator.initiateCharacterDeletableListener((CharacterDeletable)((CreationEvent)e).getCreatedElement());
			}});
		
		Translatable.TranslatableListener.addListener(new Listener() {
			
			public EventKind getEventKind() {
				return EventKind.TranslatableCreated;
			}
			
			public boolean vetoesEvent(Event e) {
				return false;
			}
			
			public void receiveEvent(Event e) {
				TranslatableListenersInitiator.initiateTranslatableListener((Translatable)((CreationEvent)e).getCreatedElement());
			}});
		
		Positionable.PositionableListener.addListener(new Listener() {
			
			public EventKind getEventKind() {
				return EventKind.PositionableCreated;
			}
			
			public boolean vetoesEvent(Event e) {
				return false;
			}
			
			public void receiveEvent(Event e) {
				PositionableListenersInitiator.initiatePositionableListener((Positionable)((CreationEvent)e).getCreatedElement());
			}});
		
		DirectionAdjustable.DirAdjListener.addListener(new Listener() {
			
			public EventKind getEventKind() {
				return EventKind.DirectionAdjustableCreated;
			}
			
			public boolean vetoesEvent(Event e) {
				return false;
			}
			
			public void receiveEvent(Event e) {
				DirectionAdjustableListenersInitiator.initiateDirectionAdjustableListener((DirectionAdjustable)((CreationEvent)e).getCreatedElement());
			}});
		
		Resizable.ResizableListener.addListener(new Listener() {
			
			public EventKind getEventKind() {
				return EventKind.ResizableCreated;
			}
			
			public boolean vetoesEvent(Event e) {
				return false;
			}
			
			public void receiveEvent(Event e) {
				ResizableListenersInitiator.initiateResizableListener((Resizable)((CreationEvent)e).getCreatedElement());
			}});
		
		Orderable.OrderableListener.addListener(new Listener() {
			
			public EventKind getEventKind() {
				return EventKind.OrderableCreated;
			}
			
			public boolean vetoesEvent(Event e) {
				return false;
			}
			
			public void receiveEvent(Event e) {
				OrderableListenersInitiator.initiateOrderableListener((Orderable)((CreationEvent)e).getCreatedElement());
			}});
		
		Locatable.LocatableListener.addListener(new Listener() {
			
			public EventKind getEventKind() {
				return EventKind.LocatableCreated;
			}
			
			public boolean vetoesEvent(Event e) {
				return false;
			}
			
			public void receiveEvent(Event e) {
				LocatableListenersInitiator.initiateLocatableListener((Locatable)((CreationEvent)e).getCreatedElement());
			}});
		
		Containable.ContainableListener.addListener(new Listener() {
			
			public EventKind getEventKind() {
				return EventKind.ContainableCreated;
			}
			
			public boolean vetoesEvent(Event e) {
				return false;
			}
			
			public void receiveEvent(Event e) {
				ListenableListenersInitiator.initiateContainableListener((Containable)((CreationEvent)e).getCreatedElement());
			}});
		
		Container.ContainerListener.addListener(new Listener() {
			
			public EventKind getEventKind() {
				return EventKind.ContainerCreated;
			}
			
			public boolean vetoesEvent(Event e) {
				return false;
			}
			
			public void receiveEvent(Event e) {
				ListenableListenersInitiator.initiateContainerListener((Container)((CreationEvent)e).getCreatedElement());
			}});
	}
}

