package model;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import javax.jmi.reflect.RefPackage;

public final class ModelFactory {
	private static ModelFactory instance = new ModelFactory();
	
	public static ModelFactory getInstance() {
		return instance;
	}
	
	private Map<String, ModelProvider> providers = new TreeMap<String, ModelProvider>();
	private Map<Object, ModelProvider> models = new HashMap<Object, ModelProvider>();
	
	private ModelFactory() {
		this.registerProvider(new MDRModelProvider());
	}
	
	void registerProvider(ModelProvider p) {
		this.providers.put(p.modelKind(), p);
	}
	
	public ModelProvider getProvider(String modelKind) {
		return this.providers.get(modelKind);
	}
	
	public Object getModel(String modelKind, Object [] arguments) throws UnknownModelKind, ModelCreationException {
		ModelProvider mp = this.getProvider(modelKind);
		if (mp == null) {
			throw new UnknownModelKind(modelKind);
		}
		Object model = mp.provideModel(arguments);
		models.put(model, mp);
		return model;
	}
	
	public void saveModel(File svgFile, Object model, Object [] arguments) throws ModelSaveException {
		ModelProvider provider = this.models.get(model);
		if (provider == null)
			throw new IllegalArgumentException("Provider for model " + model + " not found !");
		provider.saveModel(svgFile, model, arguments);
	}
	
	public void loadModel(Object model, Object [] arguments) throws ModelLoadException {
		ModelProvider provider = this.models.get(model);
		if (provider == null)
			throw new IllegalArgumentException("Provider for model " + model + " not found !");
		provider.loadModel(model, arguments);
	}
}
