/*
 * Created on 25 nov. 2004
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
package test;

import ihm04.actions.Translate;

import java.awt.Color;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Random;

import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.svg.SVGLineElement;
import org.w3c.dom.svg.SVGSVGElement;

import test.actions.DeleteCharacter;
import test.actions.EditText;
import test.actions.InsertCharacter;
import test.actions.ResizeAction;
import test.queries.HitCharacter;
import test.queries.Location;
import test.selection.SelectionManager;
import test.templates.DefaultSVGTemplateProperties;
import test.templates.SVGTemplateProperties;
import test.templates.SVGTemplatePropertyException;
import test.templates.test.RectTemplate;
import utils.AWK;
import dopidom.Action;
import dopidom.ActionInterface;
import dopidom.Component;
import dopidom.Query;
import dopidom.QueryInterface;
import dopidom.svg.SVGPicking;
import dopidom.svg.SVGScene;
import dopidom.svg.SVGToolInteractor;

/**
 * @author fondemen
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
public class Pointer extends SVGToolInteractor {
	
	protected volatile SVGPicking selectedPicking = null;

	protected volatile SVGLineElement caretElement;
	protected volatile HitCharacter hitCharacter;

	protected volatile boolean canResize = false;
	protected volatile boolean onResize = false;
	protected Location selectionLocation = new Location();

	public Pointer(Element wrappedElement) {
		super(wrappedElement);
		
		caretElement = (SVGLineElement) SVGScene.getInstance().getRootElement().getElementById("caret-symbol");
		caretElement = (SVGLineElement) caretElement.cloneNode(true);
		
		SelectionManager.INSTANCE.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				Component selected = ((SelectionManager)e.getSource()).getSelectedComponent();
				onResize = false;
				canResize = selected != null && selected.canDoAction(ResizeAction.class) && selected.canDoQuery(Location.class);
				if (canResize) {
					selected.getQueryInterface(Location.class).doQuery(selectionLocation);
				}
				
				System.out.println("Selection changed to " + selected);
				
			}});
	}
	
	public SVGPicking getSelectedPicking() {
		return selectedPicking;
	}
	
	public Component getSelectedComponent() {
		/*SVGPicking s = this.getSelectedPicking();
		return s == null ? null : s.getComponent();*/
		return SelectionManager.INSTANCE.getSelectedComponent();
	}
	
	//For test purpose
	/*private transient QueryListener movmentManager = new QueryListener() {
		public void handleEvent(QueryChangedEvent evt) {
			Location o = (Location)evt.getOldValue();
			Location n = (Location)evt.getNewValue();
			System.out.println("movment detected for selected element (from " + o.getX() + ',' + o.getY() + " to " + n.getX() + ',' + n.getY());
		}};*/
	public void setSelectedPicking(SVGPicking selected) {
		Component oldC = this.getSelectedComponent();
		this.selectedPicking = selected;
		SelectionManager.INSTANCE.setSelectedComponent(selected == null ? null : selected.getComponent());
		/*Component c = this.getSelectedComponent();
		if (c != null && c != oldC && c.canDoQuery(Location.class)) {
			ListenerManager.INSTANCE.unregister(movmentManager);
			ListenerManager.INSTANCE.register(movmentManager, c.getQueryInterface(Location.class), new Location());
		}*/
	}
	
	public void mousePressed(MouseEvent event) {
		super.mousePressed(event);
		
		if (event.getClickCount() > 1) {
			switch (event.getButton()) {
			//case MouseEvent.BUTTON1 : unselect(); this.addBasicShape(event.getX(), event.getY()); break;
			}
		} else {
			switch (event.getButton()) {
			case MouseEvent.BUTTON1 :
					hidePopup(); select(event.getX(), event.getY()); 
					if (EditText.canDoEdit(getSelectedComponent()))
						insertCaretQueued(event.getX(), event.getY());
					else
						removeCaretQueued();
					break;
			case MouseEvent.BUTTON3 : hidePopup(); select(event.getX(), event.getY()); showPopup(event.getX(), event.getY()); break;
			}
		}
		
		super.mousePressed(event);
		
		System.out.println();
	}

	public void mouseReleased(MouseEvent event)
	{
		SVGPicking picking = this.getSelectedPicking();
		if (picking != null)
			if (event.getButton() == 1)
				if (picking.hasActionConsumer(Translate.class))
					endInteractionOn(Translate.class, picking.getActionConsumer(Translate.class));
	}

	public void keyTyped(KeyEvent event)
	{
		char character = event.getKeyChar();
		if ((getSelectedPicking() == null) || (character == KeyEvent.CHAR_UNDEFINED) || (character < 32))
			return;
		if (EditText.canDoEdit(getSelectedComponent()))
		{
			doActionOnceOn(
				new InsertCharacter(character, hitCharacter.getIndex()),
				getSelectedPicking().getActionConsumer(EditText.actionClasses));
			moveCaretQueued(+1);
		}
	}

	public void keyPressed(KeyEvent event)
	{
		int  key = event.getKeyCode();
		if ((getSelectedPicking() == null) || (key == KeyEvent.VK_UNDEFINED))
			return;
		if (EditText.canDoEdit(getSelectedComponent()))
			switch (key)
			{
				case KeyEvent.VK_DELETE :
					doActionOnceOn(
						new DeleteCharacter(false, hitCharacter.getIndex()),
						getSelectedPicking().getActionConsumer(EditText.actionClasses));
					moveCaretQueued(0);
					break;
				case KeyEvent.VK_BACK_SPACE :
					doActionOnceOn(
						new DeleteCharacter(true, hitCharacter.getIndex()),
						getSelectedPicking().getActionConsumer(EditText.actionClasses));
					moveCaretQueued(-1);
					break;
				case KeyEvent.VK_LEFT :
					moveCaretQueued(-1);
					break;
				case KeyEvent.VK_RIGHT :
					moveCaretQueued(+1);
					break;
			}
	}
	
	private void select(int x, int y) {
		this.setSelectedPicking(SVGScene.getInstance().getPickedElementsUnder(x, y, getWrappedElement()));
		//System.out.println("selecting " + this.getSelectedPicking() + " as Component " + this.getSelectedComponent());
	}
	
	private void unselect() {
		this.setSelectedPicking(null);
	}

	private final JPopupMenu popup = new JPopupMenu("Popup");
	
	private void showPopup(int x, int y) {
		final Component p = this.getSelectedComponent();
		popup.removeAll();
		
		if (p != null) {
			System.out.println("show popup:");
			ActionInterface [] ais = p.getActionInterfaces();
			if (ais != null && ais.length > 0)
			for (int i = 0; i < ais.length; i++) {
				final ActionInterface itf = ais[i];
				String actionName = AWK.lastFieldOf(itf.getAction().getName(), ".");
				System.out.println("    " + actionName);
				try {
					final Constructor defaultConstructor = itf.getAction().getConstructor(new Class[0]);
					JMenuItem item = new JMenuItem(actionName); 
					popup.add(item);
					item.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							try {
								Action a = (Action)itf.getAction().newInstance();
								itf.beginInteraction();
								itf.doAction(a);
								itf.endInteraction();
							} catch (InstantiationException e1) {
								e1.printStackTrace();
							} catch (IllegalAccessException e1) {
								e1.printStackTrace();
							} finally {
								hidePopup();
							}
						}
					});
				} catch (NoSuchMethodException ex) {
				}
			}
			QueryInterface [] qis = p.getQueryInterfaces();
			if (qis != null && ais.length > 0)
				for (int i = 0; i < qis.length; i++) {
					final QueryInterface itf = qis[i];
					final String queryName = AWK.lastFieldOf(itf.getQuery().getName(), ".");
					System.out.println("    " + queryName);
					try {
						final Constructor defaultConstructor = itf.getQuery().getConstructor(new Class[0]);
						JMenuItem item = new JMenuItem(queryName); 
						popup.add(item);
						item.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent e) {
								try {
									Query q = (Query)itf.getQuery().newInstance();
									itf.doQuery(q);
									StringBuffer sb = new StringBuffer();
									Method [] ms = itf.getQuery().getMethods();
									boolean gotProperty = false;
									for (int i = 0; i < ms.length; ++i) {
										String msName = ms[i].getName();
										List unacceptedNames = Arrays.asList(new String [] {"getScope", "getClass", "getQuery"});
										if (ms[i].getParameterTypes().length == 0 && msName.startsWith("get") && !unacceptedNames.contains(msName)) {
											try {
												Object res = ms[i].invoke(q, new Object [0]);
												sb.append(msName.substring(3));
												sb.append(" = "); 
												sb.append(String.valueOf(res));
												sb.append(System.getProperty("line.separator"));
												gotProperty = true;
											} catch (IllegalArgumentException e2) {
												e2.printStackTrace();
											} catch (InvocationTargetException e2) {
												e2.printStackTrace();
											}
										}
											
									}
									if (gotProperty)
										JOptionPane.showMessageDialog(SVGScene.getInstance().getCanvas(), sb.toString(), queryName, JOptionPane.INFORMATION_MESSAGE);
								} catch (InstantiationException e1) {
									e1.printStackTrace();
								} catch (IllegalAccessException e1) {
									e1.printStackTrace();
								} finally {
									hidePopup();
								}
							}
						});
					} catch (NoSuchMethodException ex) {
					}
				}
		}
		if (popup.getComponentCount() > 0) {
			SVGScene.getInstance().getCanvas().add(popup);
			Point cl = SVGScene.getInstance().getCanvas().getLocationOnScreen();
			popup.setLocation(x + cl.x, y + cl.y);
			popup.setVisible(true);
		}
	}

	private void hidePopup() {
		System.out.println("hide popup");
		popup.setVisible(false);
		SVGScene.getInstance().getCanvas().remove(popup);
	}

	private transient Random rg = new Random(new Date().getTime());
	private void addBasicShape(int x, int y) {
		System.out.println("element addition attempt");
		
		SVGTemplateProperties p = new DefaultSVGTemplateProperties();
		p.set("x", new Integer(x));
		p.set("y", new Integer(y));
		p.set("width", new Integer(rg.nextInt(500)));
		p.set("height", new Integer(rg.nextInt(500)));
		p.set("color", new Color[] {new Color(rg.nextInt(256), rg.nextInt(256), rg.nextInt(256), rg.nextInt(256))});
		
		SVGSVGElement root = SVGScene.getInstance().getCanvas().getSVGDocument().getRootElement();
		Component s = null;
		try {
			s = new RectTemplate().createElement(root, p)[0];
		} catch (SVGTemplatePropertyException e) {
			e.printStackTrace();
		}
		root.appendChild(getWrappedElement());
		System.out.println("New element created  " + s);

	}

	protected void insertCaretQueued(final int x, final int y)
	{
		SVGScene.getInstance().invokeLater(new Runnable()
		{
			public void run()
			{
				insertCaret(x, y);
			}
		});
	}

	protected void insertCaret(int x, int y)
	{
		removeCaret();
		// update the caret location
		hitCharacter = new HitCharacter(x, y);
		getSelectedPicking().getQueryConsumer(HitCharacter.class).getQueryInterface(HitCharacter.class).doQuery(hitCharacter);
		moveCaret(0);
		// insert the caret
		Node caretNewParent = getSelectedPicking().getQueryConsumer(HitCharacter.class).getWrappedElement().getParentNode();
		caretNewParent.appendChild(caretElement);
	}

	protected void removeCaretQueued()
	{
		SVGScene.getInstance().invokeLater(new Runnable()
		{
			public void run()
			{
				removeCaret();
			}
		});
	}

	protected void removeCaret()
	{
		if (isCaretInserted())
			caretElement.getParentNode().removeChild(caretElement);
	}

	protected boolean isCaretInserted()
	{
		return caretElement.getParentNode() != null;
	}

	protected void moveCaretQueued(final int offset)
	{
		SVGScene.getInstance().invokeLater(new Runnable()
		{
			public void run()
			{
				moveCaret(offset);
			}
		});
	}

	protected void moveCaret(int offset)
	{
		hitCharacter = new HitCharacter(hitCharacter.getIndex() + offset);
		this.getSelectedPicking().getQueryConsumer(HitCharacter.class).getQueryInterface(HitCharacter.class).doQuery(hitCharacter);
		caretElement.getX1().getBaseVal().setValue((float)hitCharacter.getX());
		caretElement.getX2().getBaseVal().setValue((float)hitCharacter.getX());
		caretElement.getY1().getBaseVal().setValue((float)hitCharacter.getY() + 2);
		caretElement.getY2().getBaseVal().setValue((float)(hitCharacter.getY() - hitCharacter.getHeight() - 2));
	}
}
