/*
 * Created on 14 mars 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package test.interfaces;

import ihm04.actions.Translate;
import interfaces.Translatable;

import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.w3c.dom.svg.SVGSVGElement;

import test.Container;
import test.actionqueries.Contents;
import test.listeners.SVGActionInterfaceListener;
import test.listeners.SVGListenableActionInterface;
import test.queries.Location;
import utils.ComponentVarPersistant;
import dopidom.Action;
import dopidom.ActionConsumer;
import dopidom.ActionInterface;
import dopidom.Component;
import dopidom.QueryInterface;
import dopidom.svg.SVGActionInterface;
import dopidom.svg.SVGPicking;
import dopidom.svg.SVGScene;

/**
 * @author Olivier Beaudoux
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class ContainedTranslatable extends Translatable
{	
	protected static final byte ACCEPT = 1;
	protected static final byte REJECT = 2;
	protected static final byte DISCONNECT = 3;

	protected final QueryInterface containableQueryInterface;
	protected final ActionInterface containableActionInterface;
	
	protected final Collection listened = new ArrayList();
	
	protected SVGActionInterfaceListener containerUpdatedListener = new SVGActionInterfaceListener() {

		public void handleEvent(String eventType, ActionInterface target, Action action) {
			removeListeners();
			if (!ContainedTranslatable.this.ownerComponent.isDeleted())
				addListeners();
		}

		public boolean isValid() {
			return true;
		}
		
	};
	
	private class UpdateTranslate extends Translate {

		public UpdateTranslate(Translate t) {
			super(t);
		}
		
	}
	
	protected SVGActionInterfaceListener translateListener = new SVGActionInterfaceListener() {
		
		public void handleEvent(String eventType, ActionInterface target, Action action) {
			Translate t = (Translate)action;
			if (t.isDone() && ! ContainedTranslatable.this.isInGroup()) {
				doAction(new UpdateTranslate(t));
			}
		}

		public boolean isValid() {
			return !ContainedTranslatable.this.ownerComponent.isDeleted();
		}
	};
	
	public ContainedTranslatable(Component ownerComponent)
	{
		super(ownerComponent);

		this.containableQueryInterface = this.getOwnerComponent().getQueryInterface(test.actionqueries.Container.class);
		if (this.containableQueryInterface == null)
			throw new IllegalStateException("A containable translatable component has to be containable.");
		
		this.containableActionInterface = this.getOwnerComponent().getActionInterface(test.actionqueries.Container.class);
		if (this.containableActionInterface == null)
			throw new IllegalStateException("A containable translatable component has to be containable.");
		
		if (! ownerComponent.canDoQuery(Location.class))
			throw new IllegalStateException("A containable translatable component has to be locatable.");
		
		if (this.containableActionInterface instanceof SVGListenableActionInterface) {
			((SVGListenableActionInterface)this.containableActionInterface).addListener(this.containerUpdatedListener);
		}
	}


	/* (non-Javadoc)
	 * @see dopidom.ActionInterface#doAction(dopidom.Action)
	 */
	public void act(Action action) {
		//Checks if it is an update while moving the container
		if (action instanceof UpdateTranslate)
			super.act(action);
		else if (!((Translate) action).isDone())
			switch (this.acceptAction((Translate)action)) {
			case DISCONNECT: setContainer(null);
			case ACCEPT:  this.reAllocate((Translate)action); super.act(action);
			}
	}
	
	public byte acceptAction(Translate t) {
		return this.acceptAction(t, this.getContainer());
	}
		
	private byte acceptAction(Translate t, Component c) {
		
		if (c != null) {
			Location l = new Location();
			this.getOwnerComponent().getQueryInterface(Location.class).doQuery(l);
			if (t != null) {
				l.setX(l.getX() + t.getDeltaX());
				l.setY(l.getY() + t.getDeltaY());
			}
	
			Location cl = new Location();
			c.getQueryInterface(Location.class).doQuery(cl);

			double xw = l.getX(), xe = xw + l.getWidth(), yn = l.getY(), ys = yn + l.getHeight();
			double cxw = cl.getX(), cxe = cxw + cl.getWidth(), cyn = cl.getY(), cys = cyn + cl.getHeight();
			
			if (xw > cxw && xe < cxe && yn > cyn && ys < cys)
				return ACCEPT;
			
			if (xe > cxw && ys > cyn &&	xw < cxe &&	yn < cys)
				return REJECT;
			
			if (c.getWrappedElement() instanceof SVGSVGElement)
				return REJECT;
			
			return DISCONNECT;
		} else
			return ACCEPT;
	}
	
	public Component reallocation(Translate t) {
		Location l = new Location();
		this.getOwnerComponent().getQueryInterface(Location.class).doQuery(l);
		Component ac = (Component)SVGScene.getInstance().getPickedElementsUnder((int)(t.getDeltaX()+l.getX()+l.getWidth()/2), (int)(t.getDeltaY()+l.getY()+l.getHeight()/2), getWrappedElement()).getActionConsumer(Contents.class);
		ac = ac == null ? null : ac.getActionInterface(Contents.class).getOwnerComponent();
		
		if (ac != null && this.acceptAction(t, ac) != ACCEPT) {
			ac = null;
		}
		
		return ac;
	}
	
	public void reAllocate(Translate t) {
		Component ac = reallocation(t);
		
		if (ac != null && this.getContainer() != ac)
			this.setContainer(ac);
	}
	
	private final transient test.actionqueries.Container caq = new test.actionqueries.Container();
	
	public Component getContainer() {
		this.containableQueryInterface.doQuery(this.caq);
		return this.caq.getContainer();
	}
	
	public void setContainer (Component container) {
		this.caq.setContained(this.getOwnerComponent());
		this.caq.setContainer(container);
		this.containableActionInterface.doAction(this.caq);
	}
	
	private ActionInterface [] getContainerTranslations() {
		Collection ret = new ArrayList();

		ActionInterface ai;
		for (Component c = this.getContainer(); c != null; c = c.getParent()) {
			ai = c.getActionInterface(Translate.class);
			if (ai != null)
				ret.add(ai);
		}
		
		return (ActionInterface [])ret.toArray(new ActionInterface [ret.size()]);
	}
	
	private void addListeners() {
		ActionInterface [] cti = this.getContainerTranslations();
		for (int i = 0; i < cti.length; i++) {
			if (cti[i] instanceof SVGListenableActionInterface) {
				if (! this.listeners.contains(cti[i])) {
					((SVGListenableActionInterface)cti[i]).addListener(this.translateListener);
					this.listened.add(cti[i]);
				}
			}
		}
	}
	
	private void removeListeners() {
		for (Iterator iter = this.listened.iterator(); iter.hasNext();) {
			SVGListenableActionInterface itf = (SVGListenableActionInterface) iter.next();
			iter.remove();
			itf.removeListener(this.translateListener);
		}
	}

	protected boolean isInGroup() {
		Component c = this.getContainer();
		if (c == null)
			return false;
		Component p = this.getOwnerComponent().getParent();
		do {
			if (p == null)
				return false;
			else if (p == c)
				return true;
			else
				p = p.getParent();
		} while (true);
	}
}
