package test.languageloading;

import java.util.Iterator;
import java.util.List;

import javax.jmi.reflect.RefObject;
import javax.jmi.reflect.RefPackage;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import interpretor.InterpretorFactory;
import interpretor.Variable;
import interpretor.interpretorFacade;

import org.apache.batik.dom.AbstractDocument;
import org.apache.batik.extension.ExtensionElement;
import org.netbeans.api.mdr.events.AttributeEvent;
import org.netbeans.api.mdr.events.MDRChangeEvent;
import org.netbeans.api.mdr.events.MDRChangeListener;
import org.netbeans.api.mdr.events.MDRChangeSource;
import org.w3c.dom.Attr;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.w3c.dom.events.Event;
import org.w3c.dom.events.EventListener;
import org.w3c.dom.events.EventTarget;
import org.w3c.dom.events.MutationEvent;

import utils.DomFinding;

public class MDRUpdaterElement extends ExtensionElement implements Element {

	public static final String Updater_Local_Name = "updater";
	public static final String Interpretor_Kind = "Java";
	public static final String Source_Var_Name = "source";
	public static final String Slot_Name = "slot";
	public static final String Attribute_Name = "attributeName";
	public static final String Location_Name = "location";
	
	private MDRChangeSource sourceMDR = null;
	private MDRChangeListener listenerMDR = null;
	private EventTarget sourceSVG = null;
	private EventListener listenerSVG = null;
	private String kindSVG;
	
	public MDRUpdaterElement(AbstractDocument owner) {
		super(Updater_Local_Name, owner);
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = -2400572474673563561L;

	@Override
	protected Node newNode() {
		return new MDRUpdaterElement(this.getCurrentDocument());
	}

	@Override
	public String getNodeName() {
		return Updater_Local_Name;
	}
	
	public void createListener() {

		interpretorFacade interpreter = InterpretorFactory.createInterpretorFacade(Interpretor_Kind);
		
		if (interpreter == null)
			throw new IllegalStateException("No " + Interpretor_Kind + " found for mdr updater !");
		
		Object source = null;
		List<String[]> listVars = DomFinding.domFindVariables(this);
		String[] element;
		for (Iterator iter = listVars.iterator(); iter.hasNext();) {
			element = (String[]) iter.next();
			if (element[0].equals(Source_Var_Name)) {
				source = interpreter.getSerializer().deserialize(element[1]);
				break;
			}
		}
		if (! (source instanceof MDRChangeSource))
			throw new IllegalStateException("No listenable MDR source " + Interpretor_Kind + " found !");
		if (! (source instanceof RefObject))
			throw new IllegalStateException("MDR source " + Interpretor_Kind + " is not a model element !");
		
		final String slot = this.getAttribute(Slot_Name);
		
		String location = this.getAttribute(Location_Name);
		
		Node listened, owner;
		if (location != null) {
			XPath xpath = XPathFactory.newInstance().newXPath();
			Node n;
			try {
				n = (Node)xpath.evaluate(location, this, XPathConstants.NODE);
			} catch (XPathExpressionException e1) {
				throw new IllegalArgumentException("MDR updater: cannot evaluate expression " + location + ": " + e1.getMessage(), e1);
			}
			if (n instanceof Text) {
				listened = n;
				owner = n.getParentNode();
			} else if (n instanceof Attr) {
				listened = n;
				owner = ((Attr)n).getOwnerElement();
			} else {
				throw new IllegalArgumentException("MDR updater: cannot find attribute corresponding to " + location);
			}
		} else {
			String attributeName = this.getAttribute(Attribute_Name);
			if (attributeName == null)
				throw new IllegalArgumentException("MDRUpdater : You need to specify either a location attribute with an XPath expression, or an attribute to be updated !");
			listened = (Attr)this.getParentNode().getAttributes().getNamedItem(attributeName);
			owner = ((Attr)listened).getOwnerElement();
		}
		final Node attribute = listened;

		try{
			attribute.setNodeValue(((RefObject)source).refGetValue(slot).toString());
		} catch (NullPointerException x) {}

		if (! (owner instanceof EventTarget))
			throw new IllegalStateException("MDR Updater: attribute " + attribute + " is not listenable !");
		
		this.sourceMDR = (MDRChangeSource)source;
		this.listenerMDR = new MDRChangeListener() {

			public void change(MDRChangeEvent e) {
				if ((e instanceof AttributeEvent) && ((AttributeEvent)e).getAttributeName().equals(slot)) {
					if (!((AttributeEvent)e).getNewElement().toString().equals(attribute.getNodeValue())) {
						try{
							System.out.println(slot+" changed to "+((AttributeEvent)e).getNewElement().toString());
							attribute.setNodeValue(((AttributeEvent)e).getNewElement().toString());
						} catch (NullPointerException x) {}
					}
				}
			}
			
		};
		this.sourceMDR.addListener(this.listenerMDR);
		
		
		final RefObject me = (RefObject)source;
		if (listened instanceof Attr) {
			this.sourceSVG = (EventTarget)owner;
			this.listenerSVG = new EventListener() {
	
				public void handleEvent(Event e) {
					if (e instanceof MutationEvent) {
						System.out.println(slot+" changed to "+((MutationEvent)e).getNewValue().toString());
						me.refSetValue(slot, ((MutationEvent)e).getNewValue());
					}
				}
				
			};
			this.kindSVG = "DOMAttrModified";
			this.sourceSVG.addEventListener(this.kindSVG, this.listenerSVG, false);
		} else { //it's a text
			this.sourceSVG = (EventTarget)listened;
			this.listenerSVG = new EventListener() {
	
				public void handleEvent(Event e) {
					if (e instanceof MutationEvent) {
						if (!((MutationEvent)e).getPrevValue().equals(((MutationEvent)e).getNewValue())) {
							System.out.println(slot+" changed to "+((MutationEvent)e).getNewValue().toString());
							me.refSetValue(slot, ((MutationEvent)e).getNewValue());
						}
					}
				}
				
			};
			this.kindSVG = "DOMCharacterDataModified";
			this.sourceSVG.addEventListener(this.kindSVG, this.listenerSVG, false);
		}
	}
	
	public void removeListeners() {
		if (this.sourceMDR != null) {
			this.sourceMDR.removeListener(this.listenerMDR);
		}
		if (this.sourceSVG != null) {
			this.sourceSVG.removeEventListener(this.kindSVG, this.listenerSVG, false);
		}
	}

}
