/**
 * 
 */
package test.languageloading;

import interpretor.InterpretorFactory;
import interpretor.Result;
import interpretor.Variable;
import interpretor.interpretorFacade;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileReader;
import java.util.List;

import utils.AWK;

public class ScriptFileFilterClass implements FileFilter {
	protected final String kind;
	
	public ScriptFileFilterClass(String kind) {
		this.kind = kind;
	}
	
	public String getKind() {
		return this.kind;
	}
	
	public boolean accept(File pathname) {
		return pathname.isFile() && AWK.eliminateLastFieldOf(pathname.getName().toLowerCase(), ".").equals(this.getKind().toLowerCase());
	}
	
	public interpretorFacade getInterpreter(File pathname) {
		if (! this.accept(pathname))
			throw new IllegalArgumentException(pathname.getAbsolutePath() + " is not a " + this.getKind() + " script file.");
		try {
			String interpretorName = pathname.getName().substring(this.getKind().length()+1);
			return InterpretorFactory.createInterpretorFacade(interpretorName);
		} catch (Exception x) {
			return null;
		}
	}
	
	/**
	 * pathname may be a directory ; all scrpts found in that directory are run and result is always null
	 * @param pathname
	 * @return
	 */
	public Result run(File pathname, Object self, List<Variable> vars) {
		if (pathname.isDirectory()) {

			File [] files = pathname.listFiles(this);
			for (int i = 0; i < files.length; i++) {
				try {
					this.run(files[i], self, vars);
					return null;
				} catch (Exception x) {
					throw new RuntimeException(x);
				}
			}
		} else if (pathname.isFile()) {
		
			interpretorFacade interpretor = this.getInterpreter(pathname);
			if (interpretor == null)
				throw new IllegalArgumentException("Cannot interpret " + this.getKind() +" file " + pathname.getAbsolutePath());
			char [] content = new char[(int)pathname.length()];
			try {
				new BufferedReader(new FileReader(pathname)).read(content);
				System.out.println("Running script " + pathname.getName() + " using " + interpretor.getClass().getName() + " interpretor.");
				return interpretor.interprete(self, vars, new String(content));
			} catch (Exception e) {
				throw new IllegalArgumentException(e);
			}
		} else
			throw new IllegalArgumentException("To run a script, you need either a file or a directory (unlike " + pathname.getAbsolutePath() + ')');
		return null;//??? compiler asks for that this theoritically unreachable line !
	}
}