package test.languageloading;

import ihm04.actions.Translate;
import interfaces.BorderFindable;
import interfaces.BorderSlidable;
import interpretor.InterpretorFactory;
import interpretor.Result;
import interpretor.Variable;
import interpretor.interpretorFacade;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import javax.jmi.reflect.RefObject;
import javax.swing.SwingUtilities;

import org.apache.batik.dom.ExtendedNode;
import org.apache.batik.dom.svg.SVGOMGElement;
import org.apache.batik.dom.svg.SVGStylableElement;
import org.apache.batik.gvt.AbstractGraphicsNode;
import org.apache.batik.gvt.CompositeGraphicsNode;
import org.apache.batik.gvt.GraphicsNode;
import org.apache.batik.gvt.event.GraphicsNodeChangeEvent;
import org.apache.batik.gvt.event.GraphicsNodeChangeListener;
import org.netbeans.api.mdr.events.AttributeEvent;
import org.netbeans.api.mdr.events.InstanceEvent;
import org.netbeans.api.mdr.events.MDRChangeEvent;
import org.netbeans.api.mdr.events.MDRChangeListener;
import org.netbeans.api.mdr.events.MDRChangeSource;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.events.MutationEvent;
import org.w3c.dom.svg.SVGElement;
import org.w3c.dom.svg.SVGStylable;

import actions.BorderSlide;
import actions.Handle;

import queries.GetBorder;

import dopidom.ActionInterface;
import dopidom.Component;
import dopidom.QueryInterface;
import dopidom.dom.DOMComponent;
import dopidom.svg.SVGScene;

import test.interfaces.Container;
import test.actionqueries.Contents;
import test.actions.DeleteAction;
import test.interfaces.Containable;
import test.selection.SelectionManager;
import test.templates.DefaultSVGTemplateProperties;
import test.templates.SVGFileTemplate;
import test.templates.SVGTemplatePropertyException;
import utils.DomFinding;
import utils.ListenerFunctions;
import utils.ManipString;

public class TemplateInstance {
	private static final Set<TemplateInstance> INSTANCES = new HashSet<TemplateInstance>();
	
	public static Set<TemplateInstance> instances() {
		return new HashSet<TemplateInstance>(INSTANCES);
	}
	
	protected final Language language;
	protected final SVGFileTemplate template;
	protected final NodeList nodes;
	protected final Set<RefObject> ges;
	protected final Result creationResults;
	private boolean deleted = false;
	
	public TemplateInstance (Language language, SVGFileTemplate template, double x, double y) {
		this.language = language;
		this.template = template;
		this.nodes = this.create(x, y);
		this.creationResults = this.invokeScript(true);
		this.ges = this.findGes();
		this.registerUpdaters();
		this.registerGeDeletion();
		this.components = this.getComponents();
		
		INSTANCES.add(this);
	}

	transient private Set<Component> components = null;
	public Set<Component> getComponents() {
		if (this.components == null) {
			this.components = new HashSet<Component>();
			for (int i = 0; i < this.nodes.getLength(); ++i) {
				findComponents(this.nodes.item(i), this.components);
			}
		}
		return this.components;
	}
	
	protected boolean hasNode(Node n) {
		for (int i = 0; i < this.nodes.getLength(); i++) {
			if (n == this.nodes.item(i))
				return true;
		}
		return false;
	}
	
	public static TemplateInstance getTemplateInstance(Node n) {
		if (n == null)
			return null;
		for (TemplateInstance ti : instances()) {
			if (ti.hasNode(n))
				return ti;
		}
		return getTemplateInstance(n.getParentNode());
	}
	
	public static TemplateInstance getTemplateInstance(Component c) {
		if (c == null)
			return null;
		for (TemplateInstance ti : instances()) {
			if (ti.components.contains(c))
				return ti;
		}
		return null;
	}
	
	public static TemplateInstance getTemplateInstance(RefObject ge) {
		if (ge == null)
			return null;
		for (TemplateInstance ti : instances()) {
			if (ti.ges.contains(ge))
				return ti;
		}
		return null;
	}
	
	private static void findComponents(Node n, Set<Component> components) {
		if (n instanceof Element)
			components.add(DOMComponent.getFactory().getOwnerComponent((Element)n));
		Node ns = n.getNextSibling(), fs = n.getFirstChild();
		if (ns != null)
			findComponents(ns, components);
		if (fs != null)
			findComponents(fs, components);
		NamedNodeMap attrs = n.getAttributes();
		if (attrs != null) {
			Node at; int length = attrs.getLength();
			for (int i = 0; i< length ; ++i){
				at = attrs.item(i);
				findComponents(at, components);
			}
		}
	}
	
	protected void registerUpdaters() {
		for (int i = 0; i < this.nodes.getLength(); i++) {
			deepRegisterUpdaters(this.nodes.item(i));
		}
	}
	
	protected static void deepRegisterUpdaters (Node n) {
		if (n instanceof MDRUpdaterElement)
			((MDRUpdaterElement)n).createListener();
		Node ns = n.getNextSibling(), fs = n.getFirstChild();
		if (ns != null)
			deepRegisterUpdaters(ns);
		if (fs != null)
			deepRegisterUpdaters(fs);
		
	}
	
	protected NodeList create(double x, double y) {
		try {
			Component[] c = this.template.createElement(SVGScene.getInstance().getRootElement(), new DefaultSVGTemplateProperties());
			if (c == null)
				c = this.template.createElement(SVGScene.getInstance().getRootElement(), new DefaultSVGTemplateProperties());
			final org.w3c.dom.Node[] n = new org.w3c.dom.Node[c.length];
			//WARNING: side effect: as not invoked using invokeLater, the following ignores stickers (moving an element does not trigger a stick move) ; I have no idea why, but this is the intended behaviour...
			for (int i = 0; i < c.length; i++) {
				n[i] = c[i].getWrappedElement();
				ActionInterface tri = c[i].getActionInterface(Handle.class);
				if (tri != null) {
					c[i].doActionOnceOn(new Handle(x, y), c[i]);
				} else {
					tri = c[i].getActionInterface(Translate.class);
					if (tri != null) {
						c[i].doActionOnceOn(new Translate(x, y), c[i]);
					}
				}
			}
			
			return new NodeList() {

				public int getLength() {
					return n.length;
				}

				public Node item(int i) {
					return n[i];
				}};
			
		} catch (SVGTemplatePropertyException e1) {
			e1.printStackTrace();
			throw new RuntimeException("Problem instanciating a " + this.template.getName(), e1);
		}
	}
	
	protected Result invokeScript(boolean creation) {
		final String ik = creation ? "creation" : "deletion";
		String cs = DomFinding.domRec(this.template.getTree(), creation ? "onCreation" : "onDelete");
		if (cs != null) {
			String language = ManipString.extractLanguage(cs);
			String code = ManipString.extractCode(cs,language);
			
			if(language != null && code != null) {
				interpretorFacade interpreter = InterpretorFactory.createInterpretorFacade(language);
				if (interpreter == null) {
					System.err.println("The interpreter of the language "+ language+" is not implemented in this version.");
				}
				
				// interpret
				try {
					Result r = interpreter.interprete(null, creation ? null : new ArrayList<Variable>(this.creationResults.getOuputVars()), code);
					
					// handle the output variables
					if (r != null && creation) {
						for (int i = 0; i < this.nodes.getLength(); i++) {
							ListenerFunctions.propagateResults(r, this.nodes.item(i));
						}
					}
					return r;
				} catch (Throwable x) {
					System.err.println("Problems while executing " + ik + " script: " + x.getMessage());
				}
			} else {
				if(language == null) System.err.println(this.template.getName() + ": The " + ik + " script doesn't contain any language ; should be of the form \"{<Language>|<code>}\"");
				if(code == null) System.err.println(this.template.getName() + ": The " + ik + " script doesn't contain any code ; should be of the form \"{<Language>|<code>}\"");
			}
		}
		return null;
	}
	
	protected Set<RefObject> findGes() {
		return deepFindGesIn(this.nodes);
	}
		
	private static Set<RefObject> deepFindGesIn(NodeList nodes) {
		Set<RefObject> ret = new HashSet<RefObject>(nodes.getLength());
		for (int i = 0; i < nodes.getLength(); i++) {
			Object o = ListenerFunctions.getVariable(nodes.item(i), "self");
			if (o instanceof RefObject) {
				ret.add((RefObject)o);
				if ((o instanceof MDRChangeSource) && ListenerFunctions.mdrIsInstanceofGraphicalElement(o)) {
					final RefObject src = (RefObject)o;
					final Node a = nodes.item(i);
					final AbstractGraphicsNode gn = (AbstractGraphicsNode)SVGScene.getInstance().getGVTNode((SVGElement)a);
					((RefObject)src).refSetValue("visible", Boolean.valueOf(gn.isVisible()));
					((MDRChangeSource)src).addListener(new MDRChangeListener () {
		
						public void change(MDRChangeEvent e) {
							if ((e instanceof AttributeEvent) && ((AttributeEvent)e).getAttributeName().equals("visible"))
								try{
									Boolean newValue = (Boolean)((AttributeEvent)e).getNewElement();
									System.out.println("visible changed to "+ newValue);
									if (newValue.booleanValue()) {
										a.getAttributes().getNamedItem("visibility").setNodeValue("visible");
									} else {
										a.getAttributes().getNamedItem("visibility").setNodeValue("hidden");
									}
								} catch (NullPointerException x) {}
						}
					});
					gn.getRoot().addTreeGraphicsNodeChangeListener(new GraphicsNodeChangeListener() {

						public void changeCompleted(GraphicsNodeChangeEvent e) {
							// TODO Auto-generated method stub
							if (e.getSource() == gn && gn.isVisible() != ((Boolean)((RefObject)src).refGetValue("visible")).booleanValue())
								((RefObject)src).refSetValue("visible", Boolean.valueOf(gn.isVisible()));
						}

						public void changeStarted(GraphicsNodeChangeEvent e) {}});
				}
			}
			ret.addAll(deepFindGesIn(nodes.item(i).getChildNodes()));
		}
		return ret;
	}
	
	public final void delete() {
		if (! this.deleted) {
			this.deleted = true;
			this.deleteInt();
			INSTANCES.remove(this);
		}
	}
	
	protected void deleteInt() {

		SelectionManager.INSTANCE.setSelectedComponent(null);
		
		for (int i = 0; i < this.nodes.getLength(); i++) {
			deepUnRegisterUpdaters(this.nodes.item(i));
		}
		
		this.invokeScript(false);
		
		for (Component c : this.getComponents()) {
			c.tagDeleted();
		}
		
		for (int i = 0; i < this.nodes.getLength(); i++) {
			try {
				this.nodes.item(i).getParentNode().removeChild(this.nodes.item(i));
			} catch (Throwable x) {}
		}
	}
	
	public final boolean isDeleted() {
		return this.deleted;
	}
	
	protected void deepUnRegisterUpdaters (Node n) {
		if (n instanceof MDRUpdaterElement)
			((MDRUpdaterElement)n).removeListeners();
		Node ns = n.getNextSibling(), fs = n.getFirstChild();
		if (ns != null && this.hasNode(ns))
			deepUnRegisterUpdaters(ns);
		if (fs != null)
			deepUnRegisterUpdaters(fs);
		
	}
	
	protected void registerGeDeletion () {
		for (RefObject ge : this.ges) {
			((MDRChangeSource)ge).addListener(new MDRChangeListener () {

				public void change(MDRChangeEvent e) {
					if (e.isOfType(InstanceEvent.EVENT_INSTANCE_DELETE))
						SwingUtilities.invokeLater(new Runnable () {

							public void run() {
								TemplateInstance.this.delete();
							}});
				}});
		}
	}
}
