/*
 * Created on 2 dc. 2004
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
package test.listeners.active;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import dopidom.QueryInterface;

/**
 * @author fondemen
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
public class ActiveListenerManager {
	public static final ActiveListenerManager INSTANCE = new ActiveListenerManager();
	
	protected Collection threadList = new LinkedList();
	
	protected class ListenableQueryThread extends Thread {
		final QueryInterface queryable;
		final List listeners;
		final ActiveListenableQuery queryValue;
		private boolean active = false;
		private Running executable;
		
		protected class Running implements Runnable {
			
			public synchronized void run() {
				try {
					ActiveListenableQuery oldValue;
					Iterator it;
					while (isActive()) {
						this.wait(getQueryValue().sampleTimeIn_ms());
						synchronized(ListenableQueryThread.this) {
							oldValue = getQueryValue().duplicate();
							getQueryable().doQuery(getQueryValue());
							if (!oldValue.equals(getQueryValue())) {
								it = getListenersAsList().iterator();
								while (it.hasNext()) {
									((QueryListener)it.next()).handleEvent(new QueryChangedEventImpl(getQueryValue().getClass(), oldValue, getQueryValue()));
								}
							}
						}
					}
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
		}
		
		public ListenableQueryThread(final QueryInterface queryable, final ActiveListenableQuery queryValue) {
			super();
			this.queryable = queryable;
			this.listeners = new LinkedList();
			this.queryValue = queryValue;
			
			threadList.add(this);
			this.activate();
		}
		public QueryInterface getQueryable() {
			return queryable;
		}
		protected synchronized List getListenersAsList() {
			return this.listeners;
		}
		public synchronized QueryListener [] getListeners() {
			List ls = this.getListenersAsList();
			return (QueryListener[])ls.toArray(new QueryListener[ls.size()]);
		}
		public synchronized boolean hasListener(QueryListener l) {
			return this.getListenersAsList().contains(l);
		}
		public synchronized boolean addListener(QueryListener l) {
			if (this.hasListener(l))
				return true;
			else
				return this.getListenersAsList().add(l);
		}
		public synchronized boolean removeListener(QueryListener l) {
			boolean ret = this.getListenersAsList().remove(l);
			if (ret && this.getListenersAsList().size() == 0) {
				this.desactivate();
				threadList.remove(this);
			}
			return ret;
		}
		public ActiveListenableQuery getQueryValue() {
			return queryValue;
		}
		public synchronized boolean isActive() {
			return this.active;
		}
		public synchronized void desactivate() {
			this.active = false;
		}
		public void activate() {
			this.start();
		}
		public synchronized void start() {
			if (!this.isActive()) {
				this.active = true;
				super.start();
			}
		}
		public void run() {
			synchronized (this) {
				if (this.executable == null)
					this.executable = new Running();
			}
			this.executable.run();
		}
	}
	
	private ActiveListenerManager(){}
	
	protected synchronized ListenableQueryThread getQueryThread(final QueryInterface queryable, final ActiveListenableQuery q) {
		Iterator i = this.threadList.iterator();
		while (i.hasNext()) {
			ListenableQueryThread t = (ListenableQueryThread) i.next();
			if (t.getQueryable() == queryable && t.getQueryValue() == q)
				return t;
		}
		return new ListenableQueryThread(queryable, q);
	}
	
	public synchronized boolean register(final QueryListener l, final QueryInterface queryable, final ActiveListenableQuery q) {
		if (! queryable.canDoQuery(q.getClass()))
			return false;
		queryable.doQuery(q);
		ListenableQueryThread t = this.getQueryThread(queryable, q);
		return t.addListener(l);
	}
	
	public synchronized void unregister(QueryListener l) {
		Iterator i = new ArrayList(this.threadList).iterator();
		while (i.hasNext()) {
			((ListenableQueryThread) i.next()).removeListener(l);
		}
	}
}
