package test.model;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import javax.jmi.model.ModelPackage;
import javax.jmi.model.MofPackage;
import javax.jmi.reflect.RefObject;
import javax.jmi.reflect.RefPackage;
import javax.jmi.xmi.MalformedXMIException;
import javax.jmi.xmi.XmiReader;
import javax.jmi.xmi.XmiWriter;

import org.netbeans.api.mdr.CreationFailedException;
import org.netbeans.api.mdr.MDRManager;
import org.netbeans.api.mdr.MDRepository;
import org.netbeans.api.xmi.XMIOutputConfig;
import org.netbeans.api.xmi.XMIReader;
import org.netbeans.api.xmi.XMIReferenceProvider;
import org.netbeans.api.xmi.XMIReferenceResolver;
import org.netbeans.api.xmi.XMIWriter;
import org.netbeans.lib.jmi.xmi.DelegatingWriter;
import org.netbeans.lib.jmi.xmi.WriterBase;

import utils.AWK;
import utils.Mangler;
import utils.SelfContainer;

public final class ModelHelper {
	private ModelHelper(){}

	public static final XmiReader reader;
	//public static final XmiWriter writer;
	public static final MDRepository mdrRepository;
	
	public static final String MapSeparator = " <-> ";
	
	static {		
		System.out.println("Setting up MDR.");
		System.setProperty("org.openide.util.Lookup", "test.model.RepositoryLookup");
		final MDRManager mdrManager = org.netbeans.api.mdr.MDRManager.getDefault();
		reader = (XmiReader) RepositoryLookup.getDefault().lookup(XmiReader.class);
		//writer = (XmiWriter) RepositoryLookup.getDefault().lookup(XmiWriter.class);
		Runtime.getRuntime().addShutdownHook(new Thread (new Runnable () {
			public void run () {
				System.out.println("Shutting down MDR.");
				mdrManager.shutdownAll();
			}
		}));
		mdrRepository = mdrManager.getDefaultRepository();
	}
	
	private static String getFreshRepositoryName(String model) {
		String base = Mangler.mangle("Model_", model);
		String extent = base;
		String [] existingExtents = mdrRepository.getExtentNames();
		Arrays.sort(existingExtents);
		int i = 1;
		while (Arrays.binarySearch(existingExtents, extent) >= 0)
			extent = base + i++;
		return extent;
	}

	private static ModelPackage createMOFExtent() throws CreationFailedException {
		return (ModelPackage)mdrRepository.createExtent(getFreshRepositoryName("MOF"));
	}
	
	private static RefPackage createExtent(MofPackage metamodelRootPackage) throws CreationFailedException {
		return mdrRepository.createExtent(getFreshRepositoryName(metamodelRootPackage.getName()), metamodelRootPackage);
	}
	
	public static XmiReader getXmiReader() {
		return reader;
	}
	
	protected static class XMIWriter extends WriterBase {
		public String getId(RefObject o) {
			return ((String) xmiIds.get (o));
		}
	}

	public static XMIWriter getXmiWriter() {
		return new XMIWriter();
	}
	
	public static RefPackage createExtent(File metamodel, String mainMMPackage) throws IOException, CreationFailedException, MalformedXMIException {
		ModelPackage mmPackage = createMOFExtent();
		String uri = metamodel.toURI().toString();
		Iterator it = getXmiReader().read(uri, mmPackage).iterator();
		MofPackage metaPackageToInstanciate = null;
		Object o;
		List rp = Arrays.asList(AWK.split(mainMMPackage, "::"));
		while (it.hasNext() && (metaPackageToInstanciate == null)) {
			o = it.next();
			if (	(o instanceof MofPackage)
				&&	((MofPackage)o).getQualifiedName().equals(rp)
				) {
				metaPackageToInstanciate = (MofPackage)o;
			}
		}
		if (metaPackageToInstanciate == null)
			throw new CreationFailedException("Cannot find package " + mainMMPackage + " in metamodel " + metamodel);
		
		return createExtent(metaPackageToInstanciate);
	}
	
	public static void load(RefPackage extent, File model) throws IOException, MalformedXMIException {
		String uri = model.toURI().toString();
		XMIReader reader = (XMIReader)getXmiReader();
		Collection c = reader.read(uri, extent);
		XMIReferenceResolver resolver = reader.getConfiguration().getReferenceResolver();
		SelfContainer.getInstance().clear();
		for (Iterator i = c.iterator(); i.hasNext();) {
			RefObject me = (RefObject) i.next();
			SelfContainer.getInstance().insertSelf(getUniqId(me)/** FIXME: resolver to be used...*/, me);
		}
	}
	
	public static void save(RefPackage extent, File model, File svgFile) throws IOException, MalformedXMIException {
		XMIWriter writer = getXmiWriter();
		writer.write(new BufferedOutputStream(new FileOutputStream(model)), extent, "1.2");

		char [] svgChars = new char[(int)svgFile.length()];
		new BufferedReader(new FileReader(svgFile)).read(svgChars);
		String svg = new String (svgChars);
		
		Map<String,Object> objects = SelfContainer.getInstance().getTable();
		for (Object object : objects.values()) {
			String svgId = getUniqId((RefObject)object); String xmiId = writer.getId((RefObject)object);
			svg = AWK.replace(svg, "Object(" + svgId + ')', "Object(" + xmiId + ')');
		}
		
		BufferedWriter svgWriter = new BufferedWriter(new FileWriter(svgFile));
		svgWriter.write(svg);
		svgWriter.flush();
		svgWriter.close();
	}
	
	public static String getUniqId(RefObject o) {
		String ret = SelfContainer.getInstance().getKey(o);
		if (ret == null)
			ret = o.refMofId();
		return ret;
	}
}
