/*
 * Created on 9 dc. 2004
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
package test.selection;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.apache.batik.swing.svg.GVTTreeBuilderAdapter;
import org.apache.batik.swing.svg.GVTTreeBuilderEvent;

import test.Background;
import test.Container;
import test.actions.SelectAction;
import dopidom.Action;
import dopidom.Component;
import dopidom.svg.SVGScene;

/**
 * @author fondemen
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
public final class SelectionManager {
	public static final SelectionManager INSTANCE = new SelectionManager();
	private SelectionManager(){
		selectionPerformed = new SelectAction();
		selectionPerformed.setPartOfMultipleSelection(true);
		SVGScene.getInstance().getCanvas().addGVTTreeBuilderListener(new GVTTreeBuilderAdapter()
				{
					public void gvtBuildStarted(GVTTreeBuilderEvent event)
					{
						selectedComponents.clear();
					}
				});
	}

	protected List selectedComponents = Arrays.asList(new Component[0]);
	protected final SelectAction selectionPerformed;
	
	protected List listeners = null; 

	public Component getSelectedComponent() {
		return selectedComponents == null || selectedComponents.isEmpty() ? null : (Component)selectedComponents.get(0);
	}
	
	public Component[] getSelectedComponents() {
		return selectedComponents == null || selectedComponents.isEmpty() ? null : (Component[])selectedComponents.toArray(new Component[selectedComponents.size()]);
	}
	
	public boolean isSelected(Component c) {
		return this.selectedComponents == null ? c == null : this.selectedComponents.contains(c);
	}
	
	public List getSelectedComponentsAsList() {
		return selectedComponents == null ? new ArrayList(Arrays.asList(new Component[0])) : new ArrayList(selectedComponents);
	}
	
	public void addSelectedComponentFisrt(Component selectedComponent) {
		if (selectedComponent != null) {
			List ns = this.getSelectedComponentsAsList();
			ns.add(0, selectedComponent);
			this.setSelectedComponents(ns);
		}
	}
	
	public void addSelectedComponentLast(Component selectedComponent) {
		if (selectedComponent != null) {
			List ns = this.getSelectedComponentsAsList();
			ns.add(selectedComponent);
			this.setSelectedComponents(ns);
		}
	}
	
	public void removeSelectedComponent(Component selectedComponent) {
		if (selectedComponent != null) {
			List ns = this.getSelectedComponentsAsList();
			if (selectedComponent instanceof Container)
				selectedComponent = ((Container)selectedComponent).getBackground();
			ns.remove(selectedComponent);
			this.setSelectedComponents(ns);
		}
	}
	
	public void setSelectedComponent(Component selectedComponent) {
		this.setSelectedComponents(new Component[]{selectedComponent});
	}
	
	private static class SelectionDiff {
		protected final List oldS;
		protected final List newS;
		protected List added = new ArrayList();
		protected List removed = new ArrayList();
		protected List reordered = new ArrayList();
		
		public SelectionDiff(List oldSelection, List newSelection) {
			this.oldS = oldSelection == null ? new ArrayList(0) : new ArrayList(oldSelection);
			this.newS = newSelection == null ? new ArrayList(0) : filter(newSelection);
			this.init();
		}
		
		private static ArrayList filter(List selectedComponent) {
			ArrayList ret = new ArrayList(selectedComponent.size());
			for (int i = 0; i < selectedComponent.size(); i++) {
				Component c = (Component)selectedComponent.get(i);
				if (c instanceof Background)
					c = ((Background)c).getContainer();
				if (c != null && c.canDoAction(SelectAction.class) && !ret.contains(c))
					ret.add(c);
			}
			return ret;
		}
		
		private void init() {
			for (int i = 0, max = this.newS.size(); i < max; ++i) {
				Component c = (Component) newS.get(i);
				if (this.oldS.size() > i && this.oldS.get(i) == c)
					;
				else if (this.oldS.contains(c))
					this.reordered.add(c);
				else
					this.added.add(c);
			}
			for (Iterator i = this.oldS.iterator(); i.hasNext();) {
				Component c = (Component) i.next();
				if (! this.newS.contains(c))
					this.removed.add(c);
			}
		}
		
		public boolean hasDiff() {
			return !(added.isEmpty() && removed.isEmpty() && reordered.isEmpty());
		}
	}
	
	public void setSelectedComponents(Component[] selectedComponents) {
		this.setSelectedComponents(Arrays.asList(selectedComponents == null ? new Component[0] : selectedComponents));
	}
		
	public void setSelectedComponents(List selectedComponents) {
		SelectionDiff diff = new SelectionDiff(this.getSelectedComponentsAsList(), selectedComponents);
		if (!diff.hasDiff())
			return;
		this.selectedComponents = diff.newS;
		Action[] action = new Action[]{this.selectionPerformed};
		for (Iterator i = diff.removed.iterator(); i.hasNext();) {
			Component c = (Component) i.next();
			if (c.canDoAction(SelectAction.class))
				c.getActionInterface(SelectAction.class).undoActions(action);
		}
		if (diff.newS.isEmpty()) {
			this.selectedComponents = null;
		} else {
			for (Iterator i = diff.added.iterator(); i.hasNext();) {
				Component c = (Component) i.next();
				c.getActionInterface(SelectAction.class).doAction(this.selectionPerformed);
			}
		}
		this.fireChangeEvent(new ChangeEvent(this));
	}
	
	public void addChangeListener (ChangeListener listener) {
		if (this.listeners == null)
			this.listeners = new LinkedList();
		else if (this.listeners.contains(listener))
			return;
		
		this.listeners.add(listener);
	}
	
	public void removeChangeListenert(ChangeListener listener) {
		if (this.listeners != null)
			this.listeners.remove(listener);
	}
	
	protected void fireChangeEvent(ChangeEvent event) {
		if (this.listeners != null) {
			Iterator i = this.listeners.iterator();
			while (i.hasNext()) {
				ChangeListener listener = (ChangeListener) i.next();
				listener.stateChanged(event);
			}
		}
	}
}
