/*
 * Created on 25 nov. 2004
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
package test.templates.test;

import java.awt.Color;

import org.apache.batik.util.SVGConstants;
import org.w3c.dom.svg.SVGRectElement;
import org.w3c.dom.svg.SVGSVGElement;

import test.templates.DefaultSVGTemplateProperties;
import test.templates.DefaultSVGTemplatePropertyClass;
import test.templates.SVGTemplate;
import test.templates.SVGTemplateProperties;
import test.templates.SVGTemplatePropertyClass;
import test.templates.SVGTemplatePropertyException;
import test.templates.TemplateUtils;
import dopidom.Component;
import dopidom.dom.DOMComponent;
import dopidom.svg.SVGEventPoint;
import dopidom.svg.SVGScene;

/**
 * @author fondemen
 *
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
public class RectTemplate implements SVGTemplate {
	public static SVGTemplatePropertyClass RectPropertyClass;
	
	static {
		SVGTemplateProperties p = new DefaultSVGTemplateProperties();
		p.set("x", Integer.class);
		p.set("y", Integer.class);
		p.set("width", Integer.class);
		p.set("height", Integer.class);
		p.set("color", Color[].class);
		
		RectPropertyClass = new DefaultSVGTemplatePropertyClass(p);
	}

	public RectTemplate() {
		super();
	}

	public Component[] createElement(SVGSVGElement container,
			SVGTemplateProperties properties)
			throws SVGTemplatePropertyException {
		this.getPropertyClass().validate(properties);
		
		SVGRectElement e = (SVGRectElement)SVGScene.getInstance().getDocument().createElementNS(SVGTemplate.SVGElementNS, "rect");
		//events sent by the following line
		//creating the text
		//setting the text value
		//	DOMMutationEvent "DOMCharacterDataModified" (old ""; new "ihm04.BasicShape"; target on the GenericText value for the attribute "dpi:component" - sent after update) to the newly created GenericText (at target phase)
		//creating the attribute
		//adding the text value to the attribute
		//	DOMMutationEvent "DOMNodeInserted" (target the generic text, relatedNode the attributeNS) to the newly created attributeNS (capturing phase) and the newly created GenericText (at target phase) and again the attributeNS (at bubbling phase)
		//	DOMMutationEvent "DOMNodeInsertedIntoDocument" (target the generic text) to the attributeNS (capturing phase) and the GenericText (at target phase) and again the attributeNS (at bubbling phase)
		//	DOMMutationEvent "DOMSubtreeModified" (target the attributeNS) to the attributeNS (at target phase)
		//adding the attribute to the rect
		//  DOMMutationEvent "DOMAttrModified" (related the attributeNS; old=""; new="BasicShape"; attrName="component") to the rect (at target phase)
		e.setAttributeNS(SVGTemplate.SVGAttributeNS, "dpi:component", "ihm04.BasicShape");
		//again...
		e.setAttribute(SVGConstants.SVG_WIDTH_ATTRIBUTE, ((Integer)properties.get("width")).toString());
		e.setAttribute(SVGConstants.SVG_HEIGHT_ATTRIBUTE, ((Integer)properties.get("height")).toString());
		e.setAttribute(SVGConstants.SVG_TRANSFORM_ATTRIBUTE, SVGConstants.TRANSFORM_TRANSLATE + '(' + ((Integer)properties.get("x")).toString() + ',' + ((Integer)properties.get("y")).toString() + ')');
		Color[] color = ((Color[])properties.get("color"));
		if (color.length > 0) {				
			//e.setAttribute(SVGConstants.SVG_FILL_ATTRIBUTE, TemplateUtils.colorCode(color[0]));
			e.getStyle().setProperty(SVGConstants.CSS_FILL_PROPERTY, TemplateUtils.colorCode(color[0]), null);
			if (color[0].getAlpha() != 255) {
				e.getStyle().setProperty(SVGConstants.CSS_FILL_OPACITY_PROPERTY, TemplateUtils.alphaAsString(color[0]), null);
			}
		}
		
		container.appendChild(e);
		DOMComponent.getFactory().newComponent(e);
		SVGEventPoint.getFactory().newEventPoint(e);
		return new Component[] {(Component)DOMComponent.getFactory().newComponent(e)};
	}

	public SVGTemplatePropertyClass getPropertyClass() {
		return RectPropertyClass;
	}

}
