package utils;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import test.languageloading.VariableElement;

public class DomFinding {
	
	// search in the Node n (all his tree) the Node which named s
	// and return his value!
	public static String domRec(Node n, String s){
		
		// test le noeud courant
		if (n.getNodeName().equals(s)){
			return n.getNodeValue();
		}
		
		// cherche dans les attributs du noeud
		if (n.hasAttributes()){
			NamedNodeMap nnm = n.getAttributes();
			for (int i=0; i<nnm.getLength(); i++){
				if (nnm.item(i).getNodeName().equals(s)){
					return nnm.item(i).getNodeValue();
				}
			}
		}
		
		/*// puis cherche dans les enfants
		String temp = null;
		NodeList nl = n.getChildNodes();
		for (int i = 0; i<nl.getLength(); i++){
			if (nl.item(i).hasChildNodes()||nl.item(i).hasAttributes()){
				temp=domRec(nl.item(i),s);
				if (temp!=null){
					return temp;
				}
			}
		}
		
		// et sinon, dans les voisins
		try{
			temp = domRec(n.getNextSibling(),s);
		} catch(NullPointerException e){
			temp = null;
		}*/
		return null;//temp;
		
	}
	
	public static List<String> domFindAll(Node n, String s){
		
		List<String> ret = new ArrayList<String>();
		
		// test le noeud courant
		if (n.getNodeName().equals(s)){
			ret.add(n.getNodeValue());
		}
		
		// cherche dans les attributs du noeud
		if (n.hasAttributes()){
			NamedNodeMap nnm = n.getAttributes();
			for (int i=0; i<nnm.getLength(); i++){
				if (nnm.item(i).getNodeName().equals(s)){
					ret.add(nnm.item(i).getNodeValue());
				}
			}
		}
		
		// puis cherche dans les enfants
		//String temp = null;
		NodeList nl = n.getChildNodes();
		for (int i = 0; i<nl.getLength(); i++){
			if (nl.item(i).getNodeName().equals(s)){
				ret.add(nl.item(i).getNodeValue());
			} else if (nl.item(i).hasChildNodes()||nl.item(i).hasAttributes()){
				List<String> l = domFindAll(nl.item(i),s);
				for(String current : l) {
					if (!ret.contains(current))
						ret.add(current);
				}
			}
		}
		try{
			List<String> l = domFindAll(n.getNextSibling(),s);
			for(String current : l) {
				if (!ret.contains(current))
					ret.add(current);
			}
		} catch(NullPointerException e){
			
		}
		
		return ret;
	}
	
	
	public static List<String[]> domFindVariables(Node n){
		
		List<String[]> ret = new ArrayList<String[]>();
		
		// test le noeud courant
		if (n.getNodeName().matches("var_.*")){
			String[] tab = {ManipString.formatId(n.getNodeName()),n.getNodeValue()};
			ret.add(tab);
		}
		
		// cherche dans les attributs du noeud
		if (n.hasAttributes()){
			NamedNodeMap nnm = n.getAttributes();
			for (int i=0; i<nnm.getLength(); i++){
				if (nnm.item(i).getNodeName().matches("var_.*")){
					String[] tab = {ManipString.formatId(nnm.item(i).getNodeName()),nnm.item(i).getNodeValue()};
					ret.add(tab);
				}
			}
		}
		
		//cherche dans les sous-lments directs
		if (n.hasChildNodes()) {
			NodeList nnm = n.getChildNodes();
			for (int i=0; i<nnm.getLength(); i++){
				if (nnm.item(i).getNodeName().equals(VariableElement.Variable_Local_Name)) {
					try {
						Node ni = nnm.item(i);
						Node niName = ni.getAttributes().getNamedItem(VariableElement.Variable_Name_Att);
						if (niName != null) {
							Node niVal = ni.getAttributes().getNamedItem(VariableElement.Value_Name_Att);
							String varName = niName.getNodeValue();
							String varVal = niVal.getNodeValue();
							String[] tab = {varName,varVal};
							ret.add(tab);
						}
					} catch (Throwable t) {}
				}
			}
		}
		
		return ret;
	}
	
//	ex: domReplace(node, "var_","$", "t", "Object(MofId)");
	public static void domReplaceVarValue(Node n, String prefixAtt, String prefixVar, String var, String replace){		
		
		// test le noeud courant
		if (n.getNodeName().startsWith(prefixAtt)){
			if (n.getNodeValue().contains(prefixVar.concat(var))){
				n.setNodeValue(n.getNodeValue().replace(prefixVar.concat(var),replace));
			}
			if(ManipString.isWellformed(n.getNodeName(),var)){
				n.setNodeValue(replace);
			}
		}
		
		// cherche dans les attributs du noeud
		if (n.hasAttributes()){
			NamedNodeMap nnm = n.getAttributes();
			for (int i=0; i<nnm.getLength(); i++){
				if (nnm.item(i).getNodeName().startsWith(prefixAtt)){
					if (nnm.item(i).getNodeValue().contains(prefixVar.concat(var))){
						String temp = (nnm.item(i).getNodeValue()).replace(prefixVar.concat(var),replace);
						nnm.item(i).setNodeValue(temp);
					}
				}
				if(ManipString.isWellformed(nnm.item(i).getNodeName(),var)){
					nnm.item(i).setNodeValue(replace);
				}
			}
		}
		
		// puis cherche dans les enfants
		//String temp = null;
		NodeList nl = n.getChildNodes();
		for (int i = 0; i<nl.getLength(); i++){
			if (nl.item(i).getNodeName().startsWith(prefixAtt)){
				if (nl.item(i).getNodeValue().contains(prefixVar.concat(var))){
					nl.item(i).setNodeValue(nl.item(i).getNodeValue().replace(prefixVar.concat(var),replace));
				}
				if(ManipString.isWellformed(nl.item(i).getNodeName(),var)){
					nl.item(i).setNodeValue(replace);
				}
			}
			if (nl.item(i).hasChildNodes()||nl.item(i).hasAttributes()){
				domReplaceVarValue(nl.item(i),prefixAtt,prefixVar,var,replace);
			}
		}
	}
	
//	ex: domReplace(node, "variable", "name", "value","$", "t", "Object(MofId)");
	public static void domReplaceVarValue(Node n, String nodeName, String attributeName, String attributeValue, String prefixVar, String var, String replace){		
		
		// test le noeud courant
		try {
			if (n.getNodeName().equals(nodeName)){
				String varName = n.getAttributes().getNamedItem(attributeName).getNodeValue();
				Node varValN = n.getAttributes().getNamedItem(attributeValue);
				String varVal = varValN.getNodeValue();
				if (varVal.contains(prefixVar.concat(var))){
					varValN.setNodeValue(varVal.replace(prefixVar.concat(var),replace));
				}
				if(varName.equals(var)){
					varValN.setNodeValue(replace);
				}
			}
		} finally {}
		
		// puis cherche dans les enfants
		//String temp = null;
		NodeList nl = n.getChildNodes();
		for (int i = 0; i<nl.getLength(); i++){
			domReplaceVarValue(nl.item(i), nodeName, attributeName, attributeValue, prefixVar, var, replace);
		}
	}
}
