package utils;

import interpretor.InterpretorFacadeKoala;
import interpretor.InterpretorFactory;
import interpretor.Result;
import interpretor.Variable;
import interpretor.interpretorFacade;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import javax.jmi.model.MofClass;
import javax.jmi.model.MofClassClass;
import javax.jmi.reflect.RefObject;

import org.w3c.dom.Node;

import test.languageloading.VariableElement;

public class ListenerFunctions {
	
	public static void updateGraphicalElement (Node current, String graphicalElementSlot, Object parameter) {
		updateGraphicalElement (current, graphicalElementSlot, parameter, false);
	}
		
	public static void updateGraphicalElement (Node current, String graphicalElementSlot, Object parameter, boolean add) {
		
		if (graphicalElementSlot != null) {
			Object selfO = getVariable(current, "self");
			Object paramO = alignToMdr(parameter);
			//MDR management of self
			if (mdrIsInstanceofGraphicalElement(selfO)) {
				RefObject self = (RefObject)selfO;
				try {
					Object val = self.refGetValue(graphicalElementSlot);
					if (val instanceof Collection) {
						Collection valC = (Collection)val;
						if (paramO instanceof Collection) {
							Collection params = (Collection)paramO;
							self.refSetValue(graphicalElementSlot, params);
							System.out.println(InterpretorFacadeKoala.getInstance().getSerializer().serialize(self) + "." + graphicalElementSlot + " := " + InterpretorFacadeKoala.getInstance().getSerializer().serialize(params));
						} else if (add) {
							valC.add(paramO);
							System.out.println(InterpretorFacadeKoala.getInstance().getSerializer().serialize(self) + "." + graphicalElementSlot + " is added " + InterpretorFacadeKoala.getInstance().getSerializer().serialize(paramO));
						} else {
							valC.remove(paramO);
							System.out.println(InterpretorFacadeKoala.getInstance().getSerializer().serialize(self) + "." + graphicalElementSlot + " is removed " + InterpretorFacadeKoala.getInstance().getSerializer().serialize(paramO));
						}
					} else {
						if (val != paramO)
							self.refSetValue(graphicalElementSlot, paramO);
							System.out.println(InterpretorFacadeKoala.getInstance().getSerializer().serialize(self) + "." + graphicalElementSlot + " := " + InterpretorFacadeKoala.getInstance().getSerializer().serialize(paramO));
					}
				} catch (Throwable x) {}
			}
		}
	}
	
	public static Result foo(String onEvent, Node current, HashSet<String[]> tabSpec){
		
		//search the important information
		onEvent = DomFinding.domRec(current, onEvent);
		
		if(onEvent != null){
			
			String language = ManipString.extractLanguage(onEvent);
			String code = ManipString.extractCode(onEvent,language);
			
			if(language != null && code != null) {
				
				final interpretorFacade interpreter = InterpretorFactory.createInterpretorFacade(language);
				
				if (interpreter != null) {
					
					ArrayList<Variable> l = new ArrayList<Variable>();
					
					// put the special variables
					if (tabSpec != null){
						for (Iterator iter = tabSpec.iterator(); iter.hasNext();) {
							String[] element = (String[]) iter.next();
							final String name = element[0];
							final Object val = interpreter.getSerializer().deserialize(element[1]);
							l.add(new Variable(){
								public String getName() {
									return name;
								}
								public Object getValue() {
									return val;
								}
							});
						}
					}
					
					// put the context variables
					List<String[]> listVars = DomFinding.domFindVariables(current);
					for (Iterator iter = listVars.iterator(); iter.hasNext();) {
						String[] element = (String[]) iter.next();
						final String name = element[0];
						final Object val = interpreter.getSerializer().deserialize(element[1]);
						if (name!=null && val!=null){
							l.add(new Variable(){
								public String getName() {
									return name;
								}
								public Object getValue() {
									return val;
								}
							});
						}
					}
					
					// interpret
					try {
						Result r = interpreter.interprete(null, l, code);
						
						// handle the output variables
						if (r != null) {
							propagateResults(r, current);
						}
						return r;
					} catch (Throwable x) {
						System.err.println("Problems while executing " + onEvent + "script: " + x.getMessage());
					}
				} else {
					System.err.println("The interpreter of the language "+ language+" is not implemented in this version.");
				}
			} else {
				if(language == null) System.err.println("The attribute "+onEvent+" doesn't contain any language in tag "+current.getNodeName());
				if(code == null) System.err.println("The attribute "+onEvent+" doesn't contain any code in tag "+current.getNodeName());
			}
		}		
		return null;	
	}
	
	public static void propagateResults(Result r, Node n) {
		for (Variable v : r.getOuputVars()) {
			Object val = v.getValue();
			try {
				DomFinding.domReplaceVarValue(n, "var_", "$", v.getName(), r.getInterpreter().getSerializer().serialize(val));
			} catch (IllegalArgumentException e) {}
			try {
				DomFinding.domReplaceVarValue(n, VariableElement.Variable_Local_Name, VariableElement.Variable_Name_Att, VariableElement.Value_Name_Att, "$", v.getName(), r.getInterpreter().getSerializer().serialize(val));
			} catch (IllegalArgumentException e) {}
		}
	}
	
	public static Object getVariable(Node current, String name) {
		List<String[]> listVars = DomFinding.domFindVariables(current);
		for (Iterator iter = listVars.iterator(); iter.hasNext();) {
			String[] element = (String[]) iter.next();
			if (element[0].equals("self")) {
				return InterpretorFacadeKoala.getInstance().getSerializer().deserialize(element[1]);
			}
		}
		return null;
	}
	
	public static Object alignToMdr(Object in) {
		if (in instanceof Collection) {
			Collection ret = new ArrayList(((Collection)in).size());
			for (Iterator iterator = ((Collection)in).iterator(); iterator.hasNext();) {
				Object object = alignToMdr(iterator.next());
				if (object != null)
					ret.add(object);
			}
			return ret;
		} else {
			if (in instanceof Node) {
				Object self = getVariable((Node)in, "self");
				return self;
			} else if (in instanceof Number) {
				return in;
			} else if (in instanceof Boolean) {
				return in;
			} else if (in instanceof String) {
				return in;
			} else
				return null;
		}
	}
	
	public static boolean mdrIsInstanceofGraphicalElement (Object self) {
		if (self == null)
			return false;
		
		if (self instanceof Collection) {
			for (Iterator iterator = ((Collection)self).iterator(); iterator.hasNext();) {
				if (! mdrIsInstanceofGraphicalElement(iterator.next())) {
					return false;
				}
			}
			return true;
		}
		
		if (! (self instanceof RefObject))
			return false;
		
		//Attemps to find a class named GraphicalElement in supertypes
		MofClass mc = (MofClass)((RefObject)self).refClass().refMetaObject();
		for (Iterator iterator = mc.allSupertypes().iterator(); iterator
				.hasNext();) {
			MofClass amc = (MofClass) iterator.next();
			if (amc.getName().equals("GraphicalElement"))
				return true;
		}
		return false;
	}
}
