package utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import dopidom.Component;

public class ManipString {
	
	/**
	 * Take a String of the template and manipule it
	 * to allow to interprete it. From all the string, 
	 * extract only the code to interpret.
	 * 
	 * ex:
	 * "t = model.getTransition().createTransition();" =
	 * 		extractCode("{Java| t = model.getTransition().createTransition();}")
	 * 
	 * @return modified String
	 */
	public static String extractCode(String s, String languageName){
		
		String ret = null;
		
		if (s.matches("\\{.*\\|.*\\}") && s.contains(languageName)){
			if (s.charAt(0) == '{'){
				s=s.substring(1,s.length()-1);
			}
			if (s.charAt(s.length()-1) == '{'){
				s=s.substring(0,s.length()-2);
			}
			s=s.replace('|',' ');
			s=s.replaceAll("\t"," ");
			s=s.replaceAll("\n"," ");
			ret=s.replaceAll(languageName,"");
		} else {
			if (!s.contains(languageName)) System.err.println("Attribute value : "+s+" don't contain "+languageName+".");
			if (!s.matches("\\{.*\\|.*\\}")) System.err.println("Attribute value : "+s+" is not correctly formed");
		}
		return ret;
	}
	
	/**
	 * From all the string, 
	 * extract only the language of the
	 * interpretor
	 * 
	 * ex:
	 * "Java" = extractCode("{Java| t = model.getTransition().createTransition();}")
	 * 
	 * @return modified String
	 */
	public static String extractLanguage(String s){
		
		if (s.matches("\\{.*\\|.*\\}")){
			String ret = s;
			
			int pos = 0;
			
			while (ret.charAt(pos)=='{' || ret.charAt(pos)=='|' || ret.charAt(pos)==' ' || ret.charAt(pos)=='\t' || ret.charAt(pos)=='\n' || pos==(s.length()-1)){
				pos++;
			}
			
			if (pos!=(s.length()-1)) {
				
				ret = ret.substring(pos, s.length()-1);
				
				pos = 0;
				
				while (ret.charAt(pos) != '|' || pos == ret.length()){
					pos++;
				}
				
				if (pos!=ret.length()){
					ret = ret.substring(0,pos);
					ret=ret.replace(" ","");
					ret=ret.replaceAll("\t","");
					ret=ret.replaceAll("\n","");
				} else {
					ret = null;
				}
			}
			return ret;
		} else {
			System.err.println("Attribute value : "+s+" is not correctly formed");
			return null;
		}
	}
	
	/**
	 * 
	 * Extract the value of a variable
	 * 
	 * ex:
	 * "true" = extractType("Boolean(true)")
	 * 
	 * @return modified String
	 */
	public static String extractVariableValue(String s){
		
		String ret = null;
		
		if (s.matches(".*\\(.*\\)")){
			ret = s;
			
			int pos = 0;
			while(ret.charAt(pos)!='('){
				pos++;
			}
			ret = ret.substring(pos+1,ret.length()-1);
			ret=ret.replace(" ","");
			ret=ret.replaceAll("\t","");
			ret=ret.replaceAll("\n","");
		} else {
			ret = s;
		}
		return ret;
	}
	
	/**
	 * 
	 * Extract the type of a variable
	 * 
	 * ex:
	 * "Boolean" = extractType("Boolean(true)")
	 * 
	 * @return modified String
	 */
	public static String extractVariableType(String s){
		
		String ret = null;
		
		if (s.matches(".*\\(.*\\)")){
			ret = s;
			
			int pos = 0;
			while(ret.charAt(pos)!='('){
				pos++;
			}
			ret = ret.substring(0,pos);
			ret=ret.replace(" ","");
			ret=ret.replaceAll("\t","");
			ret=ret.replaceAll("\n","");
		} else {
			System.err.println("Variable "+s+" is not correctly formulate.");
		}
		return ret;
	}
	
	/**
	 * 
	 * Create a String which represent a serialization
	 * 
	 * ex:
	 * "Boolean(true)" = createSerialization("Boolean","true")
	 * 
	 * @return modified String
	 */
	public static String createSerialization(String type, String value){
		
		String ret = null;
		if (type == null || type.equals("String"))
			ret = value;
		else
			ret = type.concat("(").concat(value).concat(")");
		return ret;
	}
	
	public static String formatId(String s){
		
		if (s.contains("_")){
			s=s.replaceAll(".*_","");
		}
		return s;
	}
	
	public static boolean isWellformed(String s, String var){
		
		boolean b = false;
		
		if (s.equals("var_".concat(var))){
			b=true;
		}
		return b;
	}
	
	public static String searchLastStatment(String s){
		
		int l = s.length();
		
		if (s.charAt(l-1) == ';'){
			s=s.substring(0,l-1);
			l--;
		}
		
		int pos = l-1;
		char current = s.charAt(pos);
		
		while (current != ';' && pos > 0){
			pos--;
			current = s.charAt(pos);
		}
		
		if (pos > 0) {
			s=s.substring(pos+1, l);
			s=s.replace(" ","");
			s=s.replaceAll("\t","");
			s=s.replaceAll("\n","");
			return s;
		} else {
			return null;
		}
	}
	
	public static List<String> tokenize(String s) {
		List<String> ret = new ArrayList<String>();
		int nv = 0;
		int pos = 0;
		int beforeToken = 0;
		do{
			char c = s.charAt(pos);
			switch(c) {
				case '(': nv++; break;
				case ')': nv--; break;
				case ',': if(nv == 0) {
					ret.add(s.substring(beforeToken, pos));
					beforeToken = pos+1;
					break;
				}
				default: break;
			}
			pos++;
		}
		while(pos<s.length());
		ret.add(s.substring(beforeToken, pos));
		return ret;
	}

	public static String generateSet(Component container, Collection<Component> contained) {
		
		String ret = "Set(";
		
		int i=0;
		
		for (Component cont: contained) {
			
			if(cont!=null){
				i++;
				
				List<String[]> listVars = DomFinding.domFindVariables(cont.getWrappedElement());
				String temp = "Set(";
				
				int j=0;
				for (Iterator iter = listVars.iterator(); iter.hasNext();) {
					String[] element = (String[]) iter.next();
					temp = temp.concat("Set(String(").concat(element[0]).concat("),").concat(element[1]).concat(")");
					if(j!=listVars.size()-1) temp = temp.concat(",");
					j++;
				}
				
				temp = temp.concat(")");
				if (i!=contained.size()) temp = temp.concat(",");
				
				ret = ret.concat(temp);
			}
		}
		
		ret=ret.concat(")");
		
		while(ret.contains("Set()")){
			if(ret.contains(",Set()")) ret=ret.replace(",Set()","");
			if(ret.contains("Set(),")) ret=ret.replace("Set(),","");
			if(ret.contains("Set()")) ret=ret.replace("Set()","");					
		}
		
		if(ret.equals("")) {
			//System.err.println("Variables of the containable object badly instanciated.");
			ret="Set(Set(Set(String(null), Boolean(false))))";
		}
		
		return ret;
		
	}
}
