/*
 * Created on 22 juin 2005
 *
 * TODO To change the template for this generated file go to
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
package utils;

import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.apache.batik.gvt.GraphicsNode;
import org.w3c.dom.Element;

import dopidom.svg.SVGScene;

/**
 * @author fondemen
 *
 * TODO To change the template for this generated type comment go to
 *  * Copyright 2004-2008 Frdric FONDEMENT
 */
public final class PointsDecomposer {

	private PointsDecomposer() {}
	
	final static double PathRecommendedDistance = 5;
	final static double PathDistancePrecision = 0.10;
	
	// REM: this algorithm is critical in terms of performance
	public static List getPoints(Element node) {
		LinkedList pts = new LinkedList();
		GraphicsNode gn = SVGScene.getInstance().getGVTNode(node);
		
		PathIterator pi = gn.getOutline().getPathIterator(null);
		double [] coo = new double[6];
		while (!pi.isDone()) {
			switch (pi.currentSegment(coo)) {
	          case PathIterator.SEG_MOVETO:
	          case PathIterator.SEG_LINETO: {
	          	Point2D np = new Point2D.Double(coo[0], coo[1]);
	          	if (pts.isEmpty() || ! pts.getLast().equals(np))
	          		pts.add(np);
	            break;
	          }
	          case PathIterator.SEG_CLOSE: {
	          	//if (! pts.isEmpty() && ! pts.getLast().equals(pts.getFirst()))
	          	//	pts.add(pts.getFirst());
	          	break;
	          }
	          case PathIterator.SEG_QUADTO: {
				Point2D cp = (Point2D)pts.get(pts.size()-1),
						p1 = new Point2D.Double(coo[0], coo[1]),
						p2 = new Point2D.Double(coo[2], coo[3]);
				double inc = stepNumFinder(cp, p1, p2, null);
				for (double t = inc; t <= 1; t += inc) {
					pts.add(quad(t, cp, p1, p2));
				}
				pts.add(p2);
	            break;
	          }
	          case PathIterator.SEG_CUBICTO: {
				Point2D cp = (Point2D)pts.get(pts.size()-1),
						p1 = new Point2D.Double(coo[0], coo[1]),
						p2 = new Point2D.Double(coo[2], coo[3]),
						p3 = new Point2D.Double(coo[4], coo[5]);
				double inc = stepNumFinder(cp, p1, p2, p3);
				for (double t = inc; t <= 1; t += inc) {
					pts.add(cub(t, cp, p1, p2, p3));
				}
	          	if (! pts.getLast().equals(p3))
	          		pts.add(p3);
	            break;
	          }
			}
			pi.next();
		}
		
		if (pts.getFirst().equals(pts.getLast()))
			pts.removeLast();
		return pts;
	}
	
	public static double stepNumFinder(Point2D cp, Point2D p1, Point2D p2, Point2D p3) {
		boolean isCub = p3 != null;
		
		double ret = 1.0d;
		double lastDist = Double.MAX_VALUE;
		int i = 1;
		do {
			Point2D p = isCub ? cub(ret, cp, p1, p2, p3) : quad(ret, cp, p1, p2);
			double dist = cp.distance(p);
			if (Math.abs(PathRecommendedDistance-dist)/PathRecommendedDistance < PathDistancePrecision)
				return ret;
			// TODO is that really the best algorithm ? 
			ret *= PathRecommendedDistance/dist;
			if (ret > 1.0d)
				return 1.0d;
			if (i++ > 100)
				return ret;
		} while (true);
	}
	
	public static Point2D quad(double t, Point2D cp, Point2D p1, Point2D p2) {
		double a = B(2,0, t), b = B(2,1, t), c = B(2,2, t);
		double x = a*cp.getX() + b*p1.getX() + c*p2.getX();
		double y = a*cp.getY() + b*p1.getY() + c*p2.getY();
		return new Point2D.Double(x, y);
	}
	
	public static Point2D cub(double t, Point2D cp, Point2D p1, Point2D p2, Point2D p3) {
		double a = B(3,0, t), b = B(3,1, t), c = B(3,2, t), d = B(3,3, t);
		double x = a*cp.getX() + b*p1.getX() + c*p2.getX() + d*p3.getX();
		double y = a*cp.getY() + b*p1.getY() + c*p2.getY() + d*p3.getY();
		return new Point2D.Double(x, y);
	}
	
	public static double B(int n, int m, double t) {
		return C(n,m) * Math.pow(t,m) * Math.pow(1 - t, n-m);
	}
	
	public static int C(int n, int m) {
		return fact(n) / (fact(m) * fact(n-m));
	}
	
	public static int fact(int n) {
		int ret = 1;
		for(int i = 2 ; i <= n; i++)
			ret *= i;
		return ret;
	}
	

// 	//Removed from interfaces.BorderFindable 	
//	private ArrayList old_getPoints(Element node) {
//		// element = polygon
//		// attribut points
//		// nombre pair de points
//		ArrayList pts = new ArrayList();
//		String elementType; 
//		ParameterChecker pc = new ParameterChecker (node, "BorderFindable graphic");
//		elementType = pc.checkElementType(BorderFindable.acceptedTypes);
//		
//		if (elementType.equalsIgnoreCase("polygon")) {
//			String pointsString = pc.getStringAttr("points");
//			if (pointsString == null)
//				return null;
//			pointsString = AWK.replace (pointsString, ",", " ");
//			StringTokenizer tokenizer = new StringTokenizer(pointsString, " ");
//			String token;
//			String nextOne;
//			while (tokenizer.hasMoreTokens())
//			{
//				token = tokenizer.nextToken();				
//				nextOne = tokenizer.nextToken();
//				pts.add(new Point2D.Double(Float.valueOf(token).floatValue(), Float.valueOf(nextOne).floatValue()));
//			}
//		} else if (elementType.equalsIgnoreCase("rect")) {
//			String x;
//			String y;
//			String width;
//			String height;
//			x = pc.getStringAttr("x");
//			y = pc.getStringAttr("y");
//			width = pc.getStringAttr("width");
//			height = pc.getStringAttr("height");
//			if (x==null || y==null || width==null || height==null)
//				return null;
//			pts.add(new Point2D.Double(Float.valueOf(x).floatValue(), Float.valueOf(y).floatValue()));
//			pts.add(new Point2D.Double(Float.valueOf(x).floatValue() + Float.valueOf(width).floatValue(), Float.valueOf(y).floatValue()));
//			pts.add(new Point2D.Double(Float.valueOf(x).floatValue() + Float.valueOf(width).floatValue(), Float.valueOf(y).floatValue() + Float.valueOf(height).floatValue()));
//			pts.add(new Point2D.Double(Float.valueOf(x).floatValue(), Float.valueOf(y).floatValue() + Float.valueOf(height).floatValue()));
//
//			/*
//			pts.add(new Point2D.Double(0, 0));
//			pts.add(new Point2D.Double(Float.valueOf(width).floatValue(), Float.valueOf(y).floatValue()));
//			pts.add(new Point2D.Double(Float.valueOf(width).floatValue(), Float.valueOf(height).floatValue()));
//			pts.add(new Point2D.Double(0, Float.valueOf(height).floatValue()));*/
//		} else if (elementType.equalsIgnoreCase("circle")) {
//			String cxStr;
//			double cx;
//			String cyStr;
//			double cy;
//			String rStr;
//			double r;
//			double alpha = 0;
//			cxStr = pc.getStringAttr("cx");
//			cyStr = pc.getStringAttr("cy");
//			rStr = pc.getStringAttr("r");
//			if (cxStr==null || cyStr==null || rStr==null)
//				return null;
//			cx = Double.valueOf(cxStr).doubleValue();
//			cy = Double.valueOf(cyStr).doubleValue();
//			r = Double.valueOf(rStr).doubleValue();
//			double numberCircleSteps = (2*Math.PI*r)/this.stepLength;
//			for (int i = 0; i < numberCircleSteps; i++) {
//				pts.add(new Point2D.Double(cx + r*Math.sin(alpha), cx + r*Math.cos(alpha)));				
//				alpha = alpha + (Math.PI*2)/numberCircleSteps;
//			}
//			
//		} else if (elementType.equalsIgnoreCase("line")) {
//			String x1;
//			String y1;
//			String x2;
//			String y2;
//			x1 = pc.getStringAttr("x1");
//			y1 = pc.getStringAttr("y1");
//			x2 = pc.getStringAttr("x2");
//			y2 = pc.getStringAttr("y2");
//			if (x1==null || y1==null || x2==null || y2==null)
//				return null;
//			pts.add(new Point2D.Double(Float.valueOf(x1).floatValue(), Float.valueOf(y1).floatValue()));
//			pts.add(new Point2D.Double(Float.valueOf(x2).floatValue(), Float.valueOf(y2).floatValue()));
//
//		}
//	}

}
